/**
 * @file aiot_bootstrap_api.h
 * @brief The header file of the Bootstrap module. This module obtains the IoT Platform endpoint.
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 */

#ifndef _AIOT_BOOTSTRAP_API_H_
#define _AIOT_BOOTSTRAP_API_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x0700 to -0x07FF are status codes of the Bootstrap module.
 */
#define STATE_BOOTSTRAP_BASE                                        (-0x0700)

/**
 * @brief The MQTT connection failed to be released. After @ref aiot_bootstrap_deinit was called, other API operations were still running but the timeout period expired.
 */
#define STATE_BOOTSTRAP_DEINIT_TIMEOUT                              (-0x0701)

/**
 * @brief Call @ref aiot_bootstrap_send_request first to send a Bootstrap request.
 */
#define STATE_BOOTSTRAP_NEED_SEND_REQUEST                           (-0x0702)

/**
 * @brief The Bootstrap server returned an HTTP status code that indicates an error.
 */
#define STATE_BOOTSTRAP_INVALID_STATUS_CODE                         (-0x0703)

/**
 * @brief The received Bootstrap connection information was invalid.
 */
#define STATE_BOOTSTRAP_INVALID_CONNECTION_INFO                     (-0x0704)

/**
 * @brief The type of the Bootstrap packet.
 *
 * @details
 *
 * The type of the MQTT packet that is passed to @ref aiot_bootstrap_recv_handler_t.
 */
typedef enum {
    /**
     * @brief The HTTP status code that is returned by the Bootstrap server.
     */
    AIOT_BOOTSTRAPRECV_STATUS_CODE,
    /**
     * @brief The IoT Platform endpoint. The endpoint includes the domain name and the port number.
     */
    AIOT_BOOTSTRAPRECV_CONNECTION_INFO,
    /**
     * @brief The Bootstrap notification.
     */
    AIOT_BOOTSTRAPRECV_NOTIFY
} aiot_bootstrap_recv_type_t;

/**
 * @brief The data that the Bootstrap module receives.
 */
typedef struct {
    /**
     * @brief The type of the Bootstrap packet. For more information, see @ref aiot_bootstrap_recv_type_t.
     */
    aiot_bootstrap_recv_type_t type;
    /**
     * @brief The union of the Bootstrap packet. The content depends on the packet type.
     */
    union {
        /**
         * @brief The HTTP status code that is returned by the Bootstrap server.
         */
        struct {
            uint32_t code;
        } status_code;
        /**
         * @brief The IoT Platform endpoint.
         */
        struct {
            char *host;
            uint16_t port;
        } connection_info;
        struct {
            uint32_t cmd;
        } notify;
    } data;
} aiot_bootstrap_recv_t;

/**
 * @brief The prototype of the function that is called when Bootstrap data is received.
 *
 * @param[in] handle The Bootstrap session handler.
 * @param[in] packet The Bootstrap union that contains the received Bootstrap data.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (*aiot_bootstrap_recv_handler_t)(void *handle, const aiot_bootstrap_recv_t *packet, void *userdata);

/**
 * @brief The type of the Bootstrap module event.
 */
typedef enum {
    /**
     * @brief Fields in the received Bootstrap response were invalid.
     */
    AIOT_BOOTSTRAPEVT_INVALID_RESPONSE,
    /**
     * @brief The format of the cmd field in the received Bootstrap response was invalid.
     */
    AIOT_BOOTSTRAPEVT_INVALID_CMD,
} aiot_bootstrap_event_type_t;

/**
 * @brief The Bootstrap module event.
 */
typedef struct {
    /**
     * @brief The type of the Bootstrap module event. For more information, see @ref aiot_bootstrap_event_type_t.
     *
     */
    aiot_bootstrap_event_type_t type;
} aiot_bootstrap_event_t;

/**
 * @brief The callback function of Bootstrap module events.
 *
 * @details
 *
 * The function is called when a Bootstrap module event is triggered.
 *
 */
typedef void (*aiot_bootstrap_event_handler_t)(void *handle, const aiot_bootstrap_event_t *event, void *userdata);

typedef enum {
    /**
     * @brief The MQTT connection handler that is required for a Bootstrap session. An MQTT connection must be established before you set the handler.
     *
     * @details
     *
     * Data type: (void *).
     */
    AIOT_BOOTSTRAPOPT_MQTT_HANDLE,

    /**
     * @brief The security credential that is used for the establishment of the connection to the Bootstrap server.
     *
     * @details
     *
     The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * Set the option of @ref aiot_sysdep_network_cred_t to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA so that the connection is established over TLS.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_BOOTSTRAPOPT_NETWORK_CRED,

    /**
     * @brief The domain name or IP address of the Bootstrap server.
     *
     * @details
     *
     * The endpoint of IoT Platform Bootstrap servers.
     *
     * | Domain name                                        | Region    | Port
     * |-------------------------------------------------|---------|---------
     * | iot-auth-global.aliyuncs.com                    | All    | 443
     *
     * Data type: (char *). Default value: iot-auth-global.aliyuncs.com.
     */
    AIOT_BOOTSTRAPOPT_HOST,

    /**
     * @brief The number of the port that is used by the Bootstrap server.
     *
     * @details
     *
     * For connections to the Bootstrap server of IoT Platform:
     *
     * TLS must be used and the port number must be 443.
     *
     * Data type: (uint16_t *). Default value: 443.
     */
    AIOT_BOOTSTRAPOPT_PORT,

    /**
     * @brief The product key of the device. You can obtain the product key from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_BOOTSTRAPOPT_PRODUCT_KEY,

    /**
     * @brief The name of the device. You can obtain the device name from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_BOOTSTRAPOPT_DEVICE_NAME,

    /**
     * @brief The ID of the region where the device resides.
     * 
     * @details
     * 
     * |                 Region name                 | Region ID |        Region name        | Region ID |
     * |---------------------------------------------|-----------|---------------------------|-----------|
     * |               China (Shanghai)              |     0     |   Singapore (Singapore)   |     1     |
     * |                Japan (Tokyo)                |     2     |       US (Virginia)       |     3     |
     * |             Germany (Frankfurt)             |     4     |    US (Silicon Valley)    |     5     |
     * |               China (Hangzhou)              |     6     |     China (Shenzhen)      |     7     |
     * |               China (Beijing)               |     8     |      China (Qingdao)      |     9     |
     * | China (Zhangjiakou-Beijing Winter Olympics) |    10     |      China (Hohhot)       |    11     |
     * |                China (Heyuan)               |    12     |      China (Chengdu)      |    13     |
     * |              China (Hong Kong)              |    14     |     Australia (Sydney)    |    15     |
     * |           Malaysia (Kuala Lumpur)           |    16     |    Indonesia (Jakarta)    |    17     |
     * |               India (Mumbai)                |    18     |        UK (London)        |    19     |
     * |                 UAE (Dubai)                 |    20     |                           |           |
     * 
     * Data type: (char *).
     */
    AIOT_BOOTSTRAPOPT_REGIONID,

    /**
     * @brief The period of time during which a message sent in the session is consumable.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_BOOTSTRAPOPT_SEND_TIMEOUT_MS,

    /**
     * @brief The period of time during which a message received in the session is consumable.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_BOOTSTRAPOPT_RECV_TIMEOUT_MS,

    /**
     * @brief The callback function that notifies your applications when data is received from the Bootstrap server.
     *
     * @details
     *
     *
     * Data type: (aiot_bootstrap_recv_handler_t).
     */
    AIOT_BOOTSTRAPOPT_RECV_HANDLER,

    /**
     * @brief The callback function that notifies your applications when a Bootstrap module event is triggered.
     *
     * @details
     *
     * Data type: ( @ref aiot_bootstrap_event_handler_t ).
     */
    AIOT_BOOTSTRAPOPT_EVENT_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * The context is returned by the callback function that is specified by @ref AIOT_BOOTSTRAPOPT_RECV_HANDLER.
     *
     * Data type: (void *).
     */
    AIOT_BOOTSTRAPOPT_USERDATA,

    /**
     * @brief The timeout period of receiving a response from the Bootstrap server.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_BOOTSTRAPOPT_TIMEOUT_MS,

    /**
     * @brief The timeout period for other API operations to complete when you end a Bootstrap session.
     *
     * @details
     *
     * If you call @ref aiot_bootstrap_deinit to end an MQTT connection and then call another aiot_bootstrap_xxx API operation, the operation returns the @ref STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_bootstrap_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_BOOTSTRAPOPT_DEINIT_TIMEOUT_MS,
    AIOT_BOOTSTRAPOPT_MAX
} aiot_bootstrap_option_t;

/**
 * @brief Creates a Bootstrap session and retains the default settings for the session parameters.
 *
 * @return void*
 * @retval Non-null string: indicates the Bootstrap session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a failure in memory allocation.
 *
 */
void *aiot_bootstrap_init(void);

/**
 * @brief Sets the Bootstrap parameters.
 *
 * @param[in] handle The Bootstrap session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_bootstrap_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_bootstrap_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed.
 * @retval >=STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_bootstrap_setopt(void *handle, aiot_bootstrap_option_t option, void *data);

/**
 * @brief Ends the Bootstrap session and releases the resources that are allocated to the Bootstrap session handler
 *
 * @param[in] handle The pointer that points to the Bootstrap session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The operation failed.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_bootstrap_deinit(void **handle);

/**
 * @brief Sends a request to the Bootstrap server.
 *
 * @param handle The Bootstrap session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The request failed to be sent.
 * @retval >=STATE_SUCCESS The request was sent.
 */
int32_t aiot_bootstrap_send_request(void *handle);

/**
 * @brief Receives a response from the Bootstrap server.
 *
 * @param handle The Bootstrap handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS No data is received.
 * @retval >=STATE_SUCCESS Data is received.
 */
int32_t aiot_bootstrap_recv(void *handle);

#if defined(__cplusplus)
}
#endif

#endif

