/**
 * @file aiot_coap_api.h
 * @brief The header file of the CoAP module. This module establishes CoAP connections to IoT Platform.
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 * @details
 *
 * The CoAP module is used to establish connections to IoT Platform. The following procedure describes how to use the API of the CoAP module:
 *
 * 1. Call @ref aiot_coap_init to initialize a CoAP session and save the session handler.
 *
 * 2. Call @ref aiot_coap_setopt to configure session parameters. For the information about the configuration items, see @ref aiot_coap_setopt.
 *
 * 3. Call @ref aiot_mqtt_connect to establish a connection to IoT Platform. The connection must be established before messages can be sent or received.
 *
 * 4. Call @ref aiot_coap_send to publish a message to IoT Platform.
 *
 * 5. Start a process that continently calls @ref aiot_coap_recv to process received CoAP messages.
 *
 */

#ifndef __AIOT_COAP_API_H__
#define __AIOT_COAP_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x0800 to -0x08FF are status codes of the COAP module.
 */
#define STATE_COAP_BASE                                             (-0x0800)

/**
 * @brief The status codes related to the CoAP messages that are sent or received.
 *
 */
#define STATE_COAP_LOG_HEXDUMP                                      (-0x0801)

/**
 * @brief The type field in the sent or received packet header was invalid.
 *
 */
#define STATE_COAP_HEADER_TYPE_ERROR                                (-0x0802)

/**
 * @brief The first three bits of the code field in the sent or received packet header are not 0, 2, 4, or 5.
 *
 */
#define STATE_COAP_HEADER_CODE_ERROR                                (-0x0802)

/**
 * @brief The length of the received packet header is less than 4 bytes.
 *
 */
#define STATE_COAP_MSG_LEN_ERROR                                    (-0x0803)

/**
 * @brief The length of the received token is less than the tkl field in the header.
 *
 */
#define STATE_COAP_MSG_TOKEN_LEN_TOO_SHORT                          (-0x0804)

/**
 * @brief The received option parameter exceeds the limit that is specified by COAP_DEFAULT_OPTION_CNT_MAX(15).
 *
 */
#define STATE_COAP_RX_OPTION_CNT_EXCEED_LIMIT                       (-0x0805)

/**
 * @brief The first sub-field of the opt_delta field of the option parameter in the received packet is 13. However, the packet length is less than expected. The option parameter failed to be parsed.
 *
 */
#define STATE_COAP_OPT_DELTA_THIRTEEN_OPT_LEN_ERROR                 (-0x0806)

/**
 * @brief The option parameter is not ended with 0XFF.
 *
 */
#define STATE_COAP_OPTION_END_NOT_FOUND                             (-0x0807)

/**
 * @brief The number of URL segments that are separated with forward slashes (/) in the sent packet exceeds 13 (COAP_DEFAULT_OPTION_CNT_MAX-2).
 *
 */
#define STATE_COAP_TX_URI_OPTION_CNT_EXCEED_LIMIT                   (-0x0808)

/**
 * @brief Memory allocation for the CoAP packet to be sent failed.
 *
 */
#define STATE_COAP_MALLOC_MSG_FAILED                                (-0x0809)

/**
 * @brief Memory allocation for the payload field in the CoAP packet to be sent failed.
 *
 */
#define STATE_COAP_MALLOC_MSG_BUFFER_FAILED                         (-0x080A)

/**
 * @brief The pointer specified by the request output parameter in the CoAP packet to be sent is null.
 *
 */
#define STATE_COAP_REQUEST_IS_NULL                                  (-0x080B)

/**
 * @brief The first sub-field of the opt_delta field in the CoAP packet to be sent is 14. The length of the option parameter exceeds the corresponding buffer size.
 *
 */
#define STATE_COAP_OPT_DELTA_FOURTEEN_OPT_LEN_ERROR                 (-0x080C)

/**
 * @brief The pointer of the output parameters is null or the type field is of the NON type in the CoAP packet to be sent.
 *
 */
#define STATE_COAP_EMPTY_MSG_PARAM_ERROR                            (-0x080D)

/**
 * @brief Memory allocation failed when an empty CoAP packet is created.
 *
 */
#define STATE_COAP_EMPTY_MSG_MALLOC_FAILED                          (-0x080E)

/**
 * @brief The options are disordered when the CoAP message is created.
 *
 */
#define STATE_COAP_OPTION_NOT_IN_ORDER                              (-0x080F)

/**
 * @brief The version number is incorrect or the length of the token is longer than 8 in the received CoAP message.
 *
 */
#define STATE_COAP_HEADER_FORMAT_ERROR                              (-0x0810)

/**
 * @brief Too many options are set and the buffer is full when the CoAP packet is created.
 *
 */
#define STATE_COAP_OPTION_BUFFER_TOO_SHORT                          (-0x081A)

/**
 * @brief The CoAP packet to be sent is empty.
 *
 */
#define STATE_COAP_SEND_MESSAGE_IS_NULL                             (-0x081B)

/**
 * @brief The space that is reserved for the header of the CoAP packet to be created is less than 4 bytes.
 *
 */
#define STATE_COAP_HEADER_BUFFER_IS_TOO_SHORT                       (-0x081C)

/**
 * @brief The first sub-field of the opt_delta field in the CoAP packet to be sent is 13. The length of the option parameter exceeds the corresponding buffer size.
 *
 */
#define STATE_COAP_OPT_LEN_THIRTEEN_OPT_LEN_ERROR                   (-0x081D)

/**
 * @brief The PSK ID failed to be created during authentication.
 *
 */
#define STATE_COAP_CREATE_PSK_FAILED                                (-0x081E)

/**
 * @brief The network parameters failed to be set for the CoAP session.
 *
 * @details The parameters includes the host, port, and cred fields and the field that specifies the protocol type.
 *
 */
#define STATE_COAP_SET_NWK_PARMS_FAILED                             (-0x081F)

/**
 * @brief The CoAP session is not initialized when the packet is to be sent.
 *
 */
#define STATE_COAP_SEND_HANDLE_IS_NULL                              (-0x0820)

/**
 * @brief The topic field in the packet to be sent is empty.
 *
 */
#define STATE_COAP_SEND_TOPIC_IS_NULL                               (-0x0821)

/**
 * @brief The pointer specified by the request parameter in the packet to be sent is null.
 *
 */
#define STATE_COAP_SEND_REQUEST_IS_NULL                             (-0x0822)

/**
 * @brief The pointer specified by the payload field of the request parameter in the packet to be sent is null.
 *
 */
#define STATE_COAP_SEND_PAYLOAD_IS_NULL                             (-0x0823)

/**
 * @brief The PSK parameters of the CoAP session fail to be set.
 *
 */
#define STATE_COAP_SET_PSK_FAILED                                   (-0x0824)

/**
 * @brief The DTLS PSK handshake with the CoAP server fails.
 *
 */
#define STATE_COAP_NWK_ESTABLISH_FAILED                             (-0x0825)

/**
 * @brief The CoAP session handler is not initialized when @ref aiot_coap_recv is called to receive packets.
 *
 */
#define STATE_COAP_RECV_HANDLE_IS_NULL                              (-0x0826)

/**
 * @brief The CoAP session handler is not initialized when @ref aiot_coap_auth is called to obtain the token.
 *
 */
#define STATE_COAP_AUTH_HANDLE_IS_NULL                              (-0x0827)

/**
 * @brief The 3-tuple of the device is empty when @ref aiot_coap_auth is called to obtain the token.
 *
 */
#define STATE_COAP_AUTH_DEVICE_INFO_MISSING                         (-0x0828)

/**
 * @brief Memory allocation for signature calculation failed when @ref aiot_coap_auth is called to obtain the token.
 *
 */
#define STATE_COAP_SIGN_SRC_MALLOC_FAILED                           (-0x0829)

/**
 * @brief The options (including the options in the URI and other options) specified in the packet to be sent exceeds the limit that is specified by COAP_DEFAULT_OPTION_CNT_MAX(15).
 *
 */
#define STATE_COAP_TX_TOTAL_OPTION_CNT_EXCEED_LIMIT                 (-0x082A)

/**
 * @brief The @ref aiot_coap_recv call to receive packets failed because no token is available.
 *
 */
#define STATE_COAP_RECV_WITHOUT_AUTHORIZATION                       (-0x082B)

/**
 * @brief After the request for the token is sent, no token is received within the timeout period specified by @ref AIOT_COAPOPT_AUTH_TIMEOUT.
 *
 */
#define STATE_COAP_AUTH_TIMEOUT                                     (-0x082C)

/**
 * @brief The auth token field in the received packet is empty.
 *
 */
#define STATE_COAP_AUTH_RSP_TOKEN_IS_NULL                           (-0x082D)

/**
 * @brief The status codes related to the token.
 *
 */
#define STATE_COAP_AUTH_DEBUG                                       (-0x082E)

/**
 * @brief Memory allocation failed when the SDK client attempts to store the received token for the CoAP session handler.
 *
 */
#define STATE_COAP_AUTH_TOKEN_MALLOC_FAILED                         (-0x082F)

/**
 * @brief The message to be sent is neither of the CON type nor the NON type.
 *
 * @details The CoAP module automatically sends an ACK packet as the response to a received CON message.
 * The types of messages that you can send are CON and NON.
 *
 */
#define STATE_COAP_SEND_INVALID_MSG_TYPE                            (-0x0830)

/**
 * @brief The initialization of the network handler in the CoAP session handler failed.
 *
 */
#define STATE_COAP_NWK_HANDLE_IS_NULL                               (-0x0831)

/**
 * @brief When @ref aiot_coap_setopt is called to configure the CoAP session handler, the handler is not initialized or the input parameters are empty.
 *
 */
#define AIOT_COAP_OPT_PARAMS_INVALID                                (-0x0832)

/**
 * @brief The option field is invalid when @ref aiot_coap_setopt is called to configure the CoAP session handler.
 *
 */
#define STATE_COAP_INVALID_OPTION                                   (-0x0833)

/**
 * @brief When @ref aiot_coap_send is called to send messages, the content format field is out of range.
 *
 */
#define STATE_COAP_UNSUPPORTED_CONTENT_FORMAT                       (-0x0834)

/**
 * @brief The @ref aiot_coap_recv call to send packets failed because no token is available.
 *
 */
#define STATE_COAP_SEND_WITHOUT_AUTHORIZATION                       (-0x0835)

/**
 * @brief The first sub-field of the opt_delta field in the CoAP packet to be sent is 14. The length of the option parameter exceeds the corresponding buffer size.
 *
 */
#define STATE_COAP_OPT_LEN_FOURTEEN_OPT_LEN_ERROR                   (-0x0836)

/**
 * @brief The space that is reserved for the header of the CoAP packet to be created is less than 4 bytes.
 *
 */
#define STATE_COAP_PAYLOAD_BUFFER_IS_TOO_SHORT                      (-0x0837)

/**
 * @brief The claimed length of the options in the received CoAP packet is greater than the length of the received options.
 *
 */
#define STATE_COAP_OPTION_LEN_ERROR                                 (-0x0838)

/**
 * @brief The log that records the received token.
 *
 */
#define STATE_COAP_LOG_AUTH_TOKEN                                   (-0x0839)

/**
 * @brief The log that records the received data expect the token.
 *
 */
#define STATE_COAP_LOG_CONTENT                                      (-0x083A)


/**
 * @brief The type of the CoAP packet.
 *
 * @details
 *
 * The packet type is passed to @ref aiot_coap_recv_handler_t.
 */
typedef enum {
    AIOT_COAPRECV_RESPONSE,
} aiot_coap_recv_type_t;

typedef struct {

    /**
     * @brief The type of the CoAP packet. For more information, see @ref aiot_coap_recv_type_t.
     */

    aiot_coap_recv_type_t type;

    /**
     * @brief The struct of a CoAP message.
     */
    struct {
        uint8_t    *payload;
        uint32_t    payload_len;
        uint8_t     rsp_code;
        uint32_t    msg_token;
    } data;
} aiot_coap_recv_t;

/**
 * @brief The prototype of the callback function that receives CoAP packets.
 *
 * @param[in] handle The CoAP session handler.
 * @param[in] packet The pointer that points to the memory address where the struct of the CoAP message is stored.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (*aiot_coap_recv_handler_t)(void *handle,
        aiot_coap_recv_t *packet,
        void *userdata);

/**
 * @brief The type of the CoAP module event.
 */
typedef enum {

    /**
     * @brief This event is triggered when the token of the CoAP session is received from IoT Platform.
     */
    AIOT_COAPEVT_AUTH_TOKEN_RECEIVED,

    /**
     * @brief This event is triggered when the token of the CoAP session expires. The validity period of a token is 7 days.
     */
    AIOT_COAPEVT_AUTH_TOKEN_EXPIRED,
} aiot_coap_event_type_t;

typedef struct {

    /**
     * @brief The type of the CoAP event. For more information, see @ref aiot_coap_event_type_t.
     */
    aiot_coap_event_type_t type;
} aiot_coap_event_t;

/**
 * @brief The callback function of CoAP events.
 *
 * @details
 *
 * This function is called when a CoAP module event is triggered, for example, when a token is obtained.
 *
 */
typedef void (*aiot_coap_event_handler_t)(void *handle,
        aiot_coap_event_t *event,
        void *userdata);


/**
 * @brief The option parameter of @ref aiot_coap_setopt The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_coap_setopt.
 *
 * @details
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_COAPOPT_HOST is set.
 *
 *    char *host = "xxx";
 *
 *    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_HOST, host);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_COAPOPT_PORT is set.
 *
 *    uint16_t port = 5684;
 *
 *    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_PORT, (void *)&port);
 */

typedef enum {

    /**
     * @brief The domain name or IP address of the CoAP server.
     *
     * @details
     *
     * The endpoints of IoT Platform MQTT servers. Replace ${pk} with the product key.
     *
     * The endpoint for connections based on TLS-PSK certificates
     *
     * | Domain name                             | Region            | Port
     * |-----------------------------------------|-------------------|---------
     * | ${pk}.coap.cn-shanghai.link.aliyuncs.com     | China (Shanghai)  | 1883
     * Data type: (char *).
     */
    AIOT_COAPOPT_HOST,

    /**
     * @brief The number of the port that is used by the CoAP server.
     *
     * @details
     *
     * For connections to IoT Platform:
     *
     * DTLS-PSK is used. Set the port number to 5684.
     *
     * Data type: (uint16_t *).
     */
    AIOT_COAPOPT_PORT,

    /**
     * @brief The product key of the device. You can obtain the product key from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_COAPOPT_PRODUCT_KEY,

    /**
     * @brief The name of the device. You can obtain the device name from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_COAPOPT_DEVICE_NAME,

    /**
     * @brief The secret of the device. You can obtain the device secret from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_COAPOPT_DEVICE_SECRET,

    /**
     * @brief The security credential that is used for CoAP connection establishment.
     *
     * @details
     *
     * The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * Only DTSL-PSK is supported.
     * Set the option of @ref aiot_sysdep_network_cred_t to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_PSK.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_COAPOPT_CRED,

    /**
     * @brief The period of time to wait before a request for the token times out.
     *
     * Data type: (uint32_t *). Default value: (2 *1000) ms.
     *
     */
    AIOT_COAPOPT_AUTH_TIMEOUT,

    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when data is received over CoAP.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_COAPOPT_RECV_TIMEOUT,

    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when data is sent over CoAP.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */

    AIOT_COAPOPT_SEND_TIMEOUT,

    /**
     * @brief The callback function that notifies your applications when data is received from the CoAP server.
     *
     * @details
     *
     * 1. If the callback function is not specified, received messages are dropped.
     *
     * 2. If the callback function is specified, it returns received messages.
     *
     * Data type: ( @ref aiot_coap_recv_handler_t ).
     */
    AIOT_COAPOPT_RECV_HANDLER,

    /**
     * @brief The callback function that notifies your applications of CoAP client events. For example, the function is called when a token is received.
     *
     * @details
     *
     * Data type: ( @ref aiot_coap_event_handler_t ).
     */
    AIOT_COAPOPT_EVENT_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * The user context will be returned by @ref AIOT_COAPOPT_RECV_HANDLER or @ref AIOT_COAPOPT_EVENT_HANDLER.
     *
     * Data type: (void *).
     */
    AIOT_COAPOPT_USERDATA,
    AIOT_COAPOPT_MAX,
} aiot_coap_option_t;

/**
 * @brief The content_format field in the aiot_coap_request_t parameter of the @ref aiot_coap_send function.
 *
 * @details
 *
 * The format that Alibaba Cloud uses is AIOT_COAP_CT_APP_JSON (JSON format). The following script provides an example:
 *
 *   aiot_coap_request_t req = {
 *        ...
 *        .content_format = AIOT_COAP_CT_APP_JSON,
 *    };
 *
 */
typedef enum {
    AIOT_COAP_CT_TEXT_PLAIN           = 0,   /**< text/plain (UTF-8) */
    AIOT_COAP_CT_APP_LINK_FORMAT      = 40,  /**< application/link-format */
    AIOT_COAP_CT_APP_XML              = 41,  /**< application/xml */
    AIOT_COAP_CT_APP_OCTET_STREAM     = 42,  /**< application/octet-stream */
    AIOT_COAP_CT_APP_RDF_XML          = 43,  /**< application/rdf+xml */
    AIOT_COAP_CT_APP_EXI              = 47,  /**< application/exi  */
    AIOT_COAP_CT_APP_JSON             = 50,  /**< application/json  */
    AIOT_COAP_CT_APP_CBOR             = 60   /**< application/cbor  */
} aiot_coap_content_format_t;

/**
 * @brief The msg_type field in the aiot_coap_request_t parameter of the @ref aiot_coap_send function.
 *
 * @details
 *
 * The ACK and RST messages are automatically sent by the CoAP session handler.
 * The types of messages that you can send are CON and NON.
 * The following script uses an AIOT_COAP_MSG_TYPE_CON message as an example:
 *
 *   aiot_coap_request_t req = {
 *        ...
 *        .msg_type = AIOT_COAP_MSG_TYPE_CON,
 *    };
 *
 */
typedef enum {

    /**
     * @brief The CON type. CON indicates that the message is confirmable.
     *
     */
    AIOT_COAP_MSG_TYPE_CON  = 0x00,

    /**
     * @brief The NON type. NON indicates that the message is not confirmable.
     *
     */
    AIOT_COAP_MSG_TYPE_NON  = 0x01,

    /**
     * @brief The ACK type.
     *
     * */
    AIOT_COAP_MSG_TYPE_ACK  = 0x02,  /**< CoAP ack message type */

    /**
     * @brief The RST type. RST is short for reset.
     *
     */
    AIOT_COAP_MSG_TYPE_RST  = 0x03,  /**< CoAP reset message type */

    AIOT_COAP_MSG_TYPE_MAX  = 0x04,
} aiot_coap_msg_type_t;

/**
 * @brief The aiot_coap_request_t parameter of the @ref aiot_coap_send function.
 *
 * @details
 * The format of a CoAP request message.
 */
typedef struct {

    /**
     * @brief The data to be reported. The format that is specified by content_format must be used.
     *
     */
    uint8_t                    *payload;

    /**
     * @brief The type of message. For more information, see @ref aiot_coap_msg_type_t.
     *
     * @details The CoAP module automatically sends an ACK packet as the response to a received CON message.
     * Therefore, the types of messages that you can send are AIOT_COAP_MSG_TYPE_CON and AIOT_COAP_MSG_TYPE_NON.
     *
     */
    aiot_coap_msg_type_t       msg_type;

    /**
     * @brief The format of the payload. For more information, see @ref aiot_coap_content_format_t.
     *
     */
    aiot_coap_content_format_t content_format;

    /**
     * @brief The pointer that stores the token. After the CoAP module sends a request for a token, IoT Platform returns a unique token.
     * The pointer stores the token. You can use the pointer outside the SDK client.
     */
    uint32_t                   *msg_token;
} aiot_coap_request_t;


/**
 * @brief Initializes a CoAP session and retains the default settings for the session parameters.
 *
 * @return void*
 * @retval Non-null string: indicates the COAP session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_coap_init(void);

/**
 * @brief Configures the CoAP session.
 *
 * @details
 *
 * The following list describes the required configuration items.
 *
 * You can retain the default values for the optional configuration items or change the values based on your business requirements.
 *
 * + `AIOT_COAPOPT_HOST`: The address of the Alibaba Cloud CoAP host.
 *
 * + `AIOT_COAPOPT_PORT`: The number of the CoAP host port that is used for connection.
 *
 * + `AIOT_COAPOPT_PRODUCT_KEY`: The product key.
 *
 * + `AIOT_COAPOPT_DEVICE_NAME`: The device name.
 *
 * + `AIOT_COAPOPT_DEVICE_SECRET`: The device secret.
 *
 * + `AIOT_COAPOPT_NETWORK_CRED`: The security credentials that are used to establish a CoAP connection.
 *
 * + `AIOT_COAPOPT_RECV_HANDLER`: The default callback function that receives data.
 *
 * + `AIOT_COAPOPT_EVENT_HANDLER`: The callback function that notifies your applications of CoAP events.
 *
 * @param[in] handle The COAP session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_coap_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_coap_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see the error codes in the file.
 * @retval =STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_coap_setopt(void *handle, aiot_coap_option_t option, void *data);

/**
 * @brief Sends a CoAP packet.
 *
 * @param[in] handle The COAP session handler.
 * @param[in] topic The topic of the CoAP packet.
 * @param[in] message The pointer that points to the content of the packet.
 *
 * @return int32_t
 * @retval STATE_COAP_SEND_HANDLE_IS_NULL The coap_handle parameter is empty.
 * @retval STATE_COAP_SEND_TOPIC_IS_NULL The topic parameter is empty.
 * @retval STATE_COAP_SEND_REQUEST_IS_NULL The request parameter is empty.
 * @retval STATE_COAP_SEND_PAYLOAD_IS_NULL The payload parameter is empty.
 * @retval STATE_COAP_SEND_WITHOUT_AUTHORIZATION  No token is available.
 * @retval STATE_COAP_SEND_INVALID_MSG_TYPE The message type is neither CON nor NON.
 * @retval <STATE_SUCCESS In most cases, other error codes are related to network issues. For more information, see the descriptions of the error codes in this file.
 * @retval STATE_SUCCESS The CoAP packet is sent.
 */
int32_t aiot_coap_send(void *handle, const char *topic, aiot_coap_request_t *message);

/**
 * @brief Receives a CoAP packet from the network.
 *
 * @param[in] handle The COAP session handler.
 *
 * @return int32_t
 * @retval STATE_COAP_RECV_HANDLE_IS_NULL The coap handle field is empty.
 * @retval STATE_COAP_RECV_WITHOUT_AUTHORIZATION No token is available.
 * @retval <STATE_SUCCESS In most cases, other error codes are related to network issues. For more information, see the descriptions of the error codes in this file.
 * @retval STATE_SUCCESS A CoAP packet is received.
 */
int32_t aiot_coap_recv(void *handle);

/**
 * @brief Releases the resources of the CoAP session handler.
 *
 * @param[in] handle The pointer that points to the MQTT connection handler.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_coap_deinit(void **handle);

/**
 * @brief Sends messages to the CoAP server and obtains a token.
 *
 * @param[in] handle The COAP session handler.
 *
 * @return int32_t
 * @retval STATE_COAP_AUTH_HANDLE_IS_NULL The coap handle field is empty.
 * @retval STATE_COAP_AUTH_DEVICE_INFO_MISSING The 3-tuple is empty.
 * @retval STATE_COAP_SIGN_SRC_MALLOC_FAILED Memory allocation for signature calculation failed.
 * @retval STATE_COAP_AUTH_TIMEOUT The request for the token times out. The timeout period is specified by @ref AIOT_COAPOPT_AUTH_TIMEOUT.
 * @retval <STATE_SUCCESS In most cases, other error codes are related to network issues. For more information, see the descriptions of the error codes in this file.
 * @retval =STATE_SUCCESS A token is received.
 *
 */
int32_t aiot_coap_auth(void *handle);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_COAP_API_H__ */

