/**
 * @file aiot_dm_api.h
 * @brief The header file of the data-model module. This module enables TSL model data to be sent and received. The data include properties, events, and services and can be in the binary format.
 * @date 2020-01-20
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 *
 * The following procedure describes how to use the API of the data-model module:
 *
 * 1. Create an MQTT session.
 *
 * 2. Call aiot_dm_init to create a data-model session and save the session handler.
 *
 * 3. Call aiot_dm_setopt to set the AIOT_DMOPT_MQTT_HANDLE option. This option specifies an MQTT connection handler and is required.
 *
 * 4. Call aiot_dm_setopt to set the AIOT_DMOPT_RECV_HANDLER and AIOT_DMOPT_USERDATA options. AIOT_DMOPT_RECV_HANDLER specifies the callback function that receives data and AIOT_DMOPT_USERDATA specifies the user context.
 *
 * 5. Establish an MQTT connection before calling aiot_dm_send.
 *
 * 6. Call aiot_dm_send to send different types of messages to IoT Platform. The callback function processes the messages that are received from IoT Platform.
 *
 */

#ifndef __AIOT_DM_API_H__
#define __AIOT_DM_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x0A00 to -0x0AFF are status codes of the data-model module.
 */
#define STATE_DM_BASE                                           (-0x0A00)

/**
 * @brief When an @ref AIOT_DMMSG_EVENT_POST message was sent, event_id in the message data was set to NULL.
 */
#define STATE_DM_EVENT_ID_IS_NULL                               (-0x0A01)

/**
 * @brief When an @ref AIOT_DMMSG_ASYNC_SERVICE_REPLY or @ref AIOT_DMMSG_SYNC_SERVICE_REPLY message was sent, event_id in the message data was set to NULL.
 */
#define STATE_DM_SERVICE_ID_IS_NULL                             (-0x0A02)

/**
 * @brief When an @ref AIOT_DMMSG_SYNC_SERVICE_REPLY message was sent, rrpc_id in the message data was set to NULL.
 */
#define STATE_DM_RRPC_ID_IS_NULL                                (-0x0A03)

/**
 * @brief When a request message was sent, param in the message data was set to NULL.
 */
#define STATE_DM_MSG_PARAMS_IS_NULL                             (-0X0A04)

/**
 * @brief When a response message was sent, data in the message data was set to NULL.
 */
#define STATE_DM_MSG_DATA_IS_NULL                               (-0X0A05)

/**
 * @brief An error occurred when the topic of the received data is parsed.
 */
#define STATE_DM_INTERNAL_TOPIC_ERROR                           (-0x0A06)

/**
 * @brief No MQTT connection handler was set. You can call @ref aiot_dm_setopt to set the handler.
 */
#define STATE_DM_MQTT_HANDLE_IS_NULL                            (-0x0A07)

/**
 * @brief The log of receiving a message from IoT Platform.
 */
#define STATE_DM_LOG_RECV                                       (-0x0A08)

/**
 * @brief The log of failure in parsing a message that is received from IoT Platform.
 */
#define SATAE_DM_LOG_PARSE_RECV_MSG_FAILED                      (-0x0A09)


/**
 * @brief The options that you can set by using the option parameter of @ref aiot_dm_setopt. The data type of the data parameter that corresponds to each option varies.
 *
 */
typedef enum {
    /**
     * @brief The MQTT connection handler that is associated with the data-model module.
     *
     * @details
     *
     * The data-model module depends on the MQTT module. You must specify a valid MQTT connection handler to ensure that the data-model module can work as expected.
     *
     * Data type: (void *).
     */
    AIOT_DMOPT_MQTT_HANDLE,

    /**
     * @brief The callback function that notifies your applications when data is received. The function is called when the data-model module receives a message.
     *
     * @details
     *
     * Data type: (aiot_dm_recv_handler_t). For more information, see the callback function prototype in @ref aiot_dm_recv_handler_t.
     */
    AIOT_DMOPT_RECV_HANDLER,

    /**
     * @brief The pointer that points to the user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * The context pointer will be returned as userdata when the callback function that is used to receive data is called. You can call @ref aiot_dm_recv_handler_t to specify the callback function.
     *
     * Data type: (void *).
     */
    AIOT_DMOPT_USERDATA,

    /**
     * @brief user indicates if he/she wants to receive reply from server
     *
     * @details
     *
     *  1 means that user wants to receive reply from server. 0 means not.
     *
     * data type: (uint8_t) *)
     */
    AIOT_DMOPT_POST_REPLY,

    /**
     * @brief The maximum number of configuration options. The number is not configurable.
     */
    AIOT_DMOPT_MAX,
} aiot_dm_option_t;

/**
 * @brief The type of the message that is sent by the data-model module.
 *
 * @details
 *
 * The enumeration includes all types of messages that the data-module module can send. The message struct varies depending on the message type.
 
 *
 */
typedef enum {
    /**
     * @brief Property report. For information about the message struct, see @ref aiot_dm_msg_property_post_t.
     * After IoT Platform receives a message of this type, it sends a message of the @ref AIOT_DMRECV_GENERIC_REPLY type as the response.
     */
    AIOT_DMMSG_PROPERTY_POST,

    /**
     * @brief Event report. For information about the message struct, see @ref aiot_dm_msg_event_post_t.
     * After IoT Platform receives a message of this type, it sends a message of the @ref AIOT_DMRECV_GENERIC_REPLY type as the response.
     */
    AIOT_DMMSG_EVENT_POST,

    /**
     * @brief Response to property configuration. For information about the message struct, see @ref aiot_dm_msg_property_set_reply_t.
     */
    AIOT_DMMSG_PROPERTY_SET_REPLY,

    /**
     * @brief Response to an asynchronous service call. For information about the message struct, see @ref aiot_dm_msg_async_service_reply_t.
     */
    AIOT_DMMSG_ASYNC_SERVICE_REPLY,

    /**
     * @brief Response to a synchronous service call. For information about the message struct, see @ref aiot_dm_msg_sync_service_reply_t.
     */
    AIOT_DMMSG_SYNC_SERVICE_REPLY,

    /**
     * @brief Binary TSL data. For information about the message struct, see @ref aiot_dm_msg_raw_data_t.
     */
    AIOT_DMMSG_RAW_DATA,

    /**
     * @brief Binary response to a synchronous service call. For information about the message struct, see @ref aiot_dm_msg_raw_service_reply_t.
     */
    AIOT_DMMSG_RAW_SERVICE_REPLY,

    /**
     * @brief Request to obtain an expected property value. For information about the message struct, see @ref aiot_dm_msg_get_desired_t.
     * After IoT Platform receives a message of this type, it sends a message of the @ref AIOT_DMRECV_GENERIC_REPLY type as the response.
     */
    AIOT_DMMSG_GET_DESIRED,

    /**
     * @brief Request to delete an expected value. For information about the message struct, see @ref aiot_dm_msg_delete_desired_t.
     * After IoT Platform receives a message of this type, it sends a message of the @ref AIOT_DMRECV_GENERIC_REPLY type as the response.
     */
    AIOT_DMMSG_DELETE_DESIRED,

    /**
     * @brief post multi messages in one shot
     */
    AIOT_DMMSG_PROPERTY_BATCH_POST,

    /**
     * @brief The maximum number of messages. This is not a message type.
     */
    AIOT_DMMSG_MAX,
} aiot_dm_msg_type_t;

/**
 * @brief Message struct of a <b>property report</b>.
 */
typedef struct {
    /**
     * @brief The JSON struct that is stored as a string. <b>The string must end with '\0'.</b> The struct includes the property data to be reported, for example, <i>"{\"LightSwitch\":0}"</i>.
     */
    char *params;
} aiot_dm_msg_property_post_t;

/**
 * @brief Message struct of an <b>event report</b>.
 */
typedef struct {
    /**
     * @brief The ID of the event. <b>It must be set to a string that ends with '\0'.</b>
     */
    char *event_id;
    /**
     * @brief The JSON struct that is stored as a string. <b>The string must end with '\0'.</b> The struct includes the event data to be reported, for example, <i>"{\"ErrorNum\":0}"</i>.
     */
    char *params;
} aiot_dm_msg_event_post_t;

/**
 * @brief Message struct of a <b>response to property configuration</b>. You can send a message of this type as the response after the device receives an @ref AIOT_DMRECV_PROPERTY_SET message.
 */
typedef struct {
    /**
     * @brief The ID of <b>the message that carries the request to set a property</b>. The data type is uint64_t.
     */
    uint64_t msg_id;
    /**
     * @brief The status code that is returned by the device. 200 indicates that the property is set. 
     */
    uint32_t code;
    /**
     * @brief The JSON struct of the response data. The JSON struct is stored as a string. <b>The string must end with '\0'.</b> If you specify <i>"{}"</i>, it indicates that the response data is empty.
     */
    char    *data;
} aiot_dm_msg_property_set_reply_t;

/**
 * @brief Message struct of a <b>response to an asynchronous service call</b>. You must send a message of this type as the response after the device receives an @ref AIOT_DMRECV_ASYNC_SERVICE_INVOKE message.
 */
typedef struct {
    /**
     * @brief The ID of <b>the message that carries the asynchronous service call</b>. The data type is uint64_t.
     */
    uint64_t msg_id;
    /**
     * @brief The ID of the service the call of which you want to respond to.
     */
    char    *service_id;
    /**
     * @brief The status code that is returned by the device. 200 indicates that the call is successful. 
     */
    uint32_t code;
    /**
     * @brief The JSON struct of the response data. The JSON struct is stored as a string. <b>The string must end with '\0'.</b> If you specify <i>"{}"</i>, it indicates that the response data is empty.
     */
    char    *data;
} aiot_dm_msg_async_service_reply_t;

/**
 * @brief Message struct of a <b>response to a synchronous service call</b>. You must send a response within the timeout period (7s by default) after the device receives an @ref AIOT_DMRECV_SYNC_SERVICE_INVOKE message.
 */
typedef struct {
    /**
     * @brief The ID of <b>the message that carries the synchronous service call</b>. The data type is uint64_t.
     */
    uint64_t msg_id;
    /**
     * @brief The RRPC ID that uniquely identifies a synchronous service. <b>The ID in the response must be the same as that in the call.</b>
     */
    char    *rrpc_id;
    /**
     * @brief The ID of the service the call of which you want to respond to.
     */
    char    *service_id;
    /**
     * @brief The status code that is returned by the device. 200 indicates that the call is successful. 
     */
    uint32_t code;
    /**
     * @brief The JSON struct of the response data. The JSON struct is stored as a string. <b>The string must end with '\0'.</b> If you specify <i>"{}"</i>, it indicates that the response data is empty.
     */
    char    *data;
} aiot_dm_msg_sync_service_reply_t;

/**
 * @brief Message struct of <b>binary TSL data</b>. IoT Platform uses a JavaScript script to convert the binary data into JSON data. Before you send a message of this type, make sure that a parsing script is available in IoT Platform.
 */
typedef struct {
    /**
     * @brief The pointer that points to the binary data that you want to report.
     */
    uint8_t *data;
    /**
     * @brief The length of the binary data that you want to report.
     */
    uint32_t data_len;
} aiot_dm_msg_raw_data_t;

/**
 * @brief Message struct of <b>binary response to a synchronous service</b>. You must send a response within the timeout period (7s by default) after the device receives an @ref AIOT_DMRECV_RAW_SYNC_SERVICE_INVOKE message.
 * Before you send a message of this type, make sure that a parsing script is available in IoT Platform and can work as expected.
 */
typedef struct {
    /**
     * @brief The RRPC ID that uniquely identifies a synchronous service. <b>The ID in the response must be the same as that in the call.</b>
     */
    char    *rrpc_id;
    /**
     * @brief The pointer that points to the binary data that you want to report.
     */
    uint8_t *data;
    /**
     * @brief The length of the binary data that you want to report.
     */
    uint32_t data_len;
} aiot_dm_msg_raw_service_reply_t;

/**
 * @brief Message struct of a <b>request to obtain an expected property value</b>.
 */
typedef struct {
    /**
     * @brief The JSON <b>array</b> that is stored as a string. <b>The string must end with '\0'.</b> The JSON array must include the ID of the property, for example, <i>"[\"LightSwitch\"]"</i>.
     */
    char *params;
} aiot_dm_msg_get_desired_t;

/**
 * @brief Message struct of a <b>request to delete an expected value.</b>.
 */
typedef struct {
    /**
     * @brief The JSON struct that is stored as a string. <b>The string must end with '\0'.</b> The JSON struct must include the ID of the property and the number of the version, for example, <i>"{\"LightSwitch\":{\"version\":1},\"Color\":{}}"</i>.
     */
    char *params;
} aiot_dm_msg_delete_desired_t;

/**
 * @brief The struct of the message that is sent by the data-model module.
 */
typedef struct {
    /**
     * @brief The product key of the device. If you specify NULL, call aiot_dm_setopt to specify the product key.
     * If the device is a backend device of a gateway, specify the product key of the backend device.
     */
    char *product_key;
    /**
     * @brief The name of the device. If you specify NULL, call aiot_dm_setopt to specify the device name.
     * If the device is a backend device of a gateway, specify the product key of the backend device.
     */
    char *device_name;
    /**
     * @brief The type of the message. For more information, see @ref aiot_dm_msg_type_t.
     */
    aiot_dm_msg_type_t type;
    /**
     * @brief The union of the message data. The message struct varies with the message type.
     */
    union {
        aiot_dm_msg_property_post_t         property_post;
        aiot_dm_msg_event_post_t            event_post;
        aiot_dm_msg_property_set_reply_t    property_set_reply;
        aiot_dm_msg_sync_service_reply_t    sync_service_reply;
        aiot_dm_msg_async_service_reply_t   async_service_reply;
        aiot_dm_msg_raw_data_t              raw_data;
        aiot_dm_msg_raw_service_reply_t     raw_service_reply;
        aiot_dm_msg_get_desired_t           get_desired;
        aiot_dm_msg_delete_desired_t        delete_desired;
    } data;
} aiot_dm_msg_t;


/**
 * @brief The types of the messages that the data-model module can receive.
 *
 * @details
 *
 * The enumeration includes all types of messages that the data-module module can receive. The message struct varies depending on the message type.
 *
 */
typedef enum {
    /**
     * @brief The response to a property or event report. For information about the message struct, see @ref aiot_dm_recv_generic_reply_t.
     */
    AIOT_DMRECV_GENERIC_REPLY,

    /**
     * @brief The request to set a property. For information about the message struct, see @ref aiot_dm_recv_property_set_t.
     */
    AIOT_DMRECV_PROPERTY_SET,

    /**
     * @brief The request to call an asynchronous service. For information about the message struct, see @ref aiot_dm_recv_async_service_invoke_t.
     */
    AIOT_DMRECV_ASYNC_SERVICE_INVOKE,

    /**
     * @brief The request to call a synchronous service. For information about the message struct, see @ref aiot_dm_recv_sync_service_invoke_t.
     */
    AIOT_DMRECV_SYNC_SERVICE_INVOKE,

    /**
     * @brief The server's reply to the raw message post by the device. For more, see  @ref aiot_dm_recv_raw_data_t
     */
    AIOT_DMRECV_RAW_DATA_REPLY,

    /**
     * @brief The binary TSL data. For information about the message struct, see @ref aiot_dm_recv_raw_data_t.
     */
    AIOT_DMRECV_RAW_DATA,

    /**
     * @brief The binary request to call a synchronous service. For information about the message struct, see @ref aiot_dm_recv_raw_service_invoke_t.
     */
    AIOT_DMRECV_RAW_SYNC_SERVICE_INVOKE,

    /**
     * @brief The maximum number of messages. This is not a message type.
     */
    AIOT_DMRECV_MAX,
} aiot_dm_recv_type_t;

/**
 * @brief Message struct of a <b>common response from IoT Platform</b>. IoT Platform sends a message of this type after it receives an @ref AIOT_DMMSG_PROPERTY_POST, @ref AIOT_DMMSG_EVENT_POST, or @ref AIOT_DMMSG_GET_DESIRED message.
 */
typedef struct {
    /**
     * @brief The ID of <b>the message that carries the reported property or event</b>. The data type is uint64_t.
     */
    uint32_t msg_id;
    /**
     * @brief The status code that is returned by the device. 200 indicates that the report is successful.
     */
    uint32_t code;
    /**
     * @brief The pointer that points to the response data from IoT Platform.
     */
    char *data;
    /**
     * @brief The length of the response data in IoT Platform.
     */
    uint32_t data_len;
    /**
     * @brief The pointer that points to the message string. The report is successful if the response includes "success". If the report fails, the response includes the error information.
     */
    char *message;
    /**
     * @brief The length of the message string.
     */
    uint32_t message_len;
} aiot_dm_recv_generic_reply_t;

/**
 * @brief Message struct of a <b>request to set a property</b>.
 */
typedef struct {
    /**
     * @brief The ID of the message. The data type is uint64_t.
     */
    uint64_t    msg_id;
    /**
     * @brief The property data that is received from IoT Platform. The data is a JSON struct that is stored as a string. The string must <b>not</b> end with '\0', for example, <i>"{\"LightSwitch\":0}"</i>.
     */
    char       *params;
    /**
     * @brief The length of the property data string.
     */
    uint32_t    params_len;
} aiot_dm_recv_property_set_t;

/**
 * @brief Message struct of a <b>synchronous service call</b>. You must respond to the call within 7s after you receive a message.
 */
typedef struct {
    /**
     * @brief The ID of the message. The data type is uint64_t.
     */
    uint64_t    msg_id;
    /**
     * @brief The RRPC ID that uniquely identifies a synchronous service.
     */
    char       *rrpc_id;
    /**
     * @brief The ID of the service. The ID depends on the TSL model.
     */
    char       *service_id;
    /**
     * @brief The input parameters of the service call. The data is a JSON struct that is stored as a string. The string must <b>not</b> end with '\0', for example, <i>"{\"LightSwitch\":0}"</i>.
     */
    char       *params;
    /**
     * @brief The length of the input parameter string.
     */
    uint32_t    params_len;
} aiot_dm_recv_sync_service_invoke_t;

/**
 * @brief Message struct of a <b>synchronous service call</b>.
 */
typedef struct {
    /**
     * @brief The ID of the message. The data type is uint64_t.
     */
    uint64_t    msg_id;
    /**
     * @brief The ID of the service. The ID depends on the TSL model.
     */
    char       *service_id;
    /**
     * @brief The input parameters of the service call. The data is a JSON struct that is stored as a string. The string must <b>not</b> end with '\0', for example, <i>"{\"LightSwitch\":0}"</i>.
     */
    char       *params;
    /**
     * @brief The length of the input parameter string.
     */
    uint32_t    params_len;
} aiot_dm_recv_async_service_invoke_t;

/**
 * @brief Message struct of <b>binary TSL data</b>. IoT Platform uses a JavaScript script to convert JSON data into binary data and publishes the binary data. Make sure that a parsing script is available in advance.
 */
typedef struct {
    /**
     * @brief The pointer that points to the buffer of received data.
     */
    uint8_t    *data;
    /**
     * @brief The length of the binary data.
     */
    uint32_t    data_len;
} aiot_dm_recv_raw_data_t;

/**
 * @brief Message struct of <b>binary synchronous service call</b>. IoT Platform uses a JavaScript script to convert JSON data into binary data and publishes the binary data. Make sure that a parsing script is available in advance.
 */
typedef struct {
    /**
     * @brief The RRPC ID that uniquely identifies a synchronous service.
     */
    char       *rrpc_id;
    /**
     * @brief The pointer that points to the buffer of received data.
     */
    uint8_t    *data;
    /**
     * @brief The length of the binary data.
     */
    uint32_t    data_len;
} aiot_dm_recv_raw_service_invoke_t;

/**
 * @brief The struct of the message that is received by the data-model module.
 */
typedef struct {
    /**
     * @brief The product key of the device. If you do not specify the product key, the product key that is configured for the MQTT module is used.
     */
    char *product_key;
    /**
     * @brief The name of the device. If you do not specify the device name, the device name that is configured for the MQTT module is used.
     */
    char *device_name;
    /**
     * @brief The type of the received message. For more information, see @ref aiot_dm_recv_type_t.
     */
    aiot_dm_recv_type_t type;
    /**
     * @brief The union of the message data. The message struct varies with the message type.
     */
    union {
        aiot_dm_recv_generic_reply_t        generic_reply;
        aiot_dm_recv_property_set_t         property_set;
        aiot_dm_recv_async_service_invoke_t async_service_invoke;
        aiot_dm_recv_sync_service_invoke_t  sync_service_invoke;
        aiot_dm_recv_raw_data_t             raw_data;
        aiot_dm_recv_raw_service_invoke_t   raw_service_invoke;
    } data;
} aiot_dm_recv_t;

/**
 * @brief The prototype of the callback function that receives messages. This function is called when the data-model module receives a message. The function then returns the message data as <i>recv</i>.
 * The user context will be returned by @ref AIOT_MQTTOPT_RECV_HANDLER or @ref AIOT_MQTTOPT_EVENT_HANDLER.
 *
 * @param[in] handle The data-model session handler.
 * @param[in] recv   The received message data. <b>The pointers in a message struct are valid only within the callback function. If you need to save the message data, copy the data to the memory.</b>
 * @param[in] userdata The pointer that points to the user context cached by the SDK. You can set the @ref AIOT_DMOPT_USERDATA parameter of @ref aiot_dm_setopt to specify the context that needs to be cached.
 *
 * @return void
 */
typedef void (*aiot_dm_recv_handler_t)(void *handle, const aiot_dm_recv_t *recv, void *userdata);

/**
 * @brief Initializes a data-model session.
 *
 * @return void*
 * @retval Non-null string: indicates the data-model connection handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 */
void *aiot_dm_init(void);

/**
 * @brief Configures the data-model session.
 *
 * @param[in] handle The data-model session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_dm_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_dm_option_t.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The configuration succeeded.
 * @retval STATE_USER_INPUT_NULL_POINTER The handle or data input parameter was null.
 * @retval STATE_USER_INPUT_OUT_RANGE The enumeration value of the option input parameter is greater than or equal to AIOT_DMOPT_MAX.
 * @retval others For more information, see @ref aiot_state_api.h.
 *
 */
int32_t aiot_dm_setopt(void *handle, aiot_dm_option_t option, void *data);

/**
 * @brief Sends a data-model message to IoT Platform. The msg input parameter specifies the message type and data.
 *
 * @param[in] handle The data-model session handler.
 * @param[in] msg    The struct of the message. The struct includes the information such as the <i>product key</i> and <i>device name</i> of the device; the message type, the message data. For more information, see @ref aiot_dm_msg_t.
 *
 * @return int32_t
 * @retval >=STATE_SUCCESS The message was sent. The type of the message was @ref AIOT_DMMSG_PROPERTY_POST, @ref AIOT_DMMSG_EVENT_POST, @ref AIOT_DMMSG_GET_DESIRED, or @ref AIOT_DMMSG_DELETE_DESIRED.
 * If the returned value is greater than STATE_SUCCESS, the value indicates the <i>msg_id</i>.
 * @retval STATE_USER_INPUT_NULL_POINTER The <i>handle</i> or <i>msg</i> input parameter was null.
 * @retval STATE_USER_INPUT_OUT_RANGE The <i>type</i> attribute in the <i>msg</i> struct was greater than or equal to AIOT_DMMSG_MAX.
 * @retval STATE_SYS_DEPEND_MALLOC_FAILED Memory allocation failed.
 * @retval STATE_DM_MQTT_HANDLE_IS_NULL No MQTT connection handler was specified by calling @ref aiot_dm_setopt.
 * @retval others For more error codes, see @ref aiot_state_api.h or @ref STATE_DM_BASE.
 *
 */
int32_t aiot_dm_send(void *handle, const aiot_dm_msg_t *msg);

/**
 * @brief Ends the data-model session and releases allocated resources.
 *
 * @param[in] p_handle The pointer that points to the data-model session handler.
 * @return int32_t
 * @retval STATE_SUCCESS The operation succeeded.
 * @retval <STATE_SUCCESS The operation failed.
 *
 */
int32_t aiot_dm_deinit(void **p_handle);


#if defined(__cplusplus)
}
#endif

#endif /* #ifndef __AIOT_DM_API_H__ */

