/**
 * @file aiot_devinfo_api.h
 * @brief The header file of the devinfo module.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 *
 * The devinfo module sends requests to update or delete device tags in IoT Platform. The following procedure describes how to use the API of the devinfo module:
 *
 * 1. Make sure that an MQTT connection to IoT Platform is established. For more information, see @ref aiot_mqtt_api.h.
 *
 * 2. Call @ref aiot_devinfo_init to initialize a devinfo session and obtain the session handler.
 *
 * 3. Call @ref aiot_devinfo_setopt to set the parameters of the devinfo session. For information about the parameters, see @ref aiot_devinfo_setopt.
 *
 * 4. Call @ref aiot_devinfo_send to send a request to manage device tags. For example, you can update or delete a tag.
 *
 * 5. The SDK client process the received response and calls the callback function that is specified by @ref AIOT_DEVINFOOPT_RECV_HANDLER of @ref aiot_devinfo_setopt to notify your applications.
 *
 */
#ifndef __AIOT_DEVINFO_API_H__
#define __AIOT_DEVINFO_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x1200 to -0x12FF are status codes of the devinfo module.
 */
#define STATE_DEVINFO_BASE                                             (-0x1200)

/**
 * @brief The MQTT connection handler is not specified. Call @ref aiot_devinfo_setopt to specify the MQTT connection handler.
 */
#define STATE_DEVINFO_MISSING_MQTT_HANDLE                              (-0x1201)

/**
 * @brief The type of the message that is returned when the devinfo module receives a packet from the network.
 */
typedef enum {
    AIOT_DEVINFORECV_GENERIC_REPLY,
} aiot_devinfo_recv_type_t;

typedef struct {
    /**
     * @brief The message ID. The data type of the ID is uint64_t, similar to the messages that are used to report properties or events.
     */
    uint32_t msg_id;
    /**
     * @brief The status code that is returned by the device. 200 indicates that the report is successful.
     */
    uint32_t code;
    /**
     * @brief The pointer that points to the response data in IoT Platform.
     */
    char *data;
    /**
     * @brief The length of the response data from IoT Platform.
     */
    uint32_t data_len;
    /**
     * @brief The pointer that points to the message string. The report is successful if the response includes "success". If the report fails, the response includes the error information.
     */
    char *message;
    /**
     * @brief The length of the message string.
     */
    uint32_t message_len;
} aiot_devinfo_recv_generic_reply_t;

/**
 * @brief The content of the message that is returned when the devinfo module receives a packet from the network.
 */
typedef struct {
    char *product_key;
    char *device_name;
    /**
     * @brief The type of the message. For more information, see @ref aiot_devinfo_recv_type_t.
     */
    aiot_devinfo_recv_type_t  type;
    union {
        /**
         * @brief The response that receives from IoT Platform to the request to manage device tags.
         */
        aiot_devinfo_recv_generic_reply_t generic_reply;
    } data;
} aiot_devinfo_recv_t;

/**
 * @brief The callback function that notifies your applications when the devinfo module receives a packet from the network. 
 *
 * @param[in] handle The devinfo session handler.
 * @param[in] packet The struct of the devinfo message. The struct stores the content of the devinfo message.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_devinfo_recv_handler_t)(void *handle, const aiot_devinfo_recv_t *packet, void *userdata);

/**
 * @brief The callback function that notifies your applications when a devinfo module event is triggered.
 */
typedef enum {
    /**
     * @brief The device information in the received response is invalid. The product key and device name cannot be parsed from the response.
     */
    AIOT_DEVINFOEVT_INVALID_DEVINFO,
    /**
     * @brief Fields in the response are invalid.
     */
    AIOT_DEVINFOEVT_INVALID_RESPONSE,
    /**
     * @brief The format of fields in the response is invalid.
     */
    AIOT_DEVINFOEVT_INVALID_RESPONSE_FORMAT,
} aiot_devinfo_event_type_t;

/**
 * @brief The data of the devinfo module event that is returned.
 */
typedef struct {
    /**
     * @brief The type of the event. For more information, see @ref aiot_devinfo_event_type_t.
     */
    aiot_devinfo_event_type_t  type;
} aiot_devinfo_event_t;

/**
 * @brief The callback function that notifies your applications when a devinfo module event is triggered.
 *
 * @param[in] handle The devinfo session handler.
 * @param[in] event The content of the devinfo module event.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (*aiot_devinfo_event_handler_t)(void *handle, const aiot_devinfo_event_t *event, void *userdata);

/**
 * @brief The type of the message to be sent. For more information, see @ref aiot_devinfo_msg_t.
 *
 * @details
 *
 * You can send a message to update or delete device tags.
 */
typedef enum {
    /**
     * @brief The message to update device tags.
     */
    AIOT_DEVINFO_MSG_UPDATE,
    /**
     * @brief The message to delete device tags.
     */
    AIOT_DEVINFO_MSG_DELETE
} aiot_devinfo_msg_type_t;

/**
 * @brief The parameters for the message to update or delete device tags.
 */
typedef struct {
    char *params;
} aiot_devinfo_msg_data_t;

typedef struct {
    /**
     * @brief The product key of the device.
     */
    char *product_key;
    /**
     * @brief The name of the device.
     */
    char *device_name;
    /**
     * @brief The type of the message. For more information, see @ref aiot_devinfo_msg_type_t.
     */
    aiot_devinfo_msg_type_t type;
    union {
        /**
         * @brief The information about the device tag to be updated. Format: "[{\"attrKey\":\"xxx\",\"attrValue\":\"yyy\"}]".
         *
         * @details
         *
         * The information is stored in a JSON array. You can specify multiple key-value pairs to update multiple tags.
         */
        aiot_devinfo_msg_data_t update;
        /**
         * @brief The information about the device tag to be deleted. Format: "[{\"attrKey\":\"xxx\"}]".
         *
         * @details
         *
         * The information is stored in a JSON array. You can specify multiple key-value pairs to delete multiple tags.
         */
        aiot_devinfo_msg_data_t delete;
    } data;
} aiot_devinfo_msg_t;

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_devinfo_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_devinfo_setopt.
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_DEVINFOOPT_MQTT_HANDLE is set.
 *
 *    void *mqtt_handle = aiot_mqtt_init();
 *    aiot_devinfo_setopt(devinfo_handle, AIOT_DEVINFOOPT_MQTT_HANDLE, mqtt_handle);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_DEVINFOOPT_DEINIT_TIMEOUT_MS is set.
 *
 *    uint32_t deinit_timeout_ms = 443;
 *    aiot_devinfo_setopt(devinfo_handle, AIOT_DEVINFOOPT_DEINIT_TIMEOUT_MS, (void *)&deinit_timeout_ms);
 */
typedef enum {
    /**
     * @brief The MQTT handler that is required for a devinfo session. An MQTT connection must be established before you set the MQTT handler.
     */
    AIOT_DEVINFOOPT_MQTT_HANDLE,

    /**
     * @brief The callback function that notifies your applications when the devinfo module receives a packet from the network.
     *
     * @details
     *
     * Data type: ( @ref aiot_devinfo_recv_handler_t).
     */
    AIOT_DEVINFOOPT_RECV_HANDLER,

    /**
     * @brief The function that is called when a devinfo module event is triggered.
     *
     * @details
     *
     * Data type: ( @ref aiot_devinfo_event_handler_t).
     */
    AIOT_DEVINFOOPT_EVENT_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client. Data type: (void *).
     *
     * @details The SDK client returns the context pointer when the callback function that is specified by the AIOT_DEVINFOOPT_RECV_HANDLER or AIOT_DEVINFOOPT_EVENT_HANDLER option is called.
     */
    AIOT_DEVINFOOPT_USERDATA,

    /**
     * @brief The timeout period for other API operations to complete when you end a devinfo session.
     *
     * @details
     *
     * If you call @ref aiot_devinfo_deinit to end an MQTT connection and then call another aiot_devinfo_xxx API operation, the operation returns the @ref STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_devinfo_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_DEVINFOOPT_DEINIT_TIMEOUT_MS,
    AIOT_DEVINFOOPT_MAX
} aiot_devinfo_option_t;

/**
 * @brief Creates a devinfo session and retains the default settings for the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the devinfo connection handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_devinfo_init(void);

/**
 * @brief Configures the devinfo session.
 *
 * @param[in] handle The devinfo session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_devinfo_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_devinfo_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The configuration fails.
 * @retval >=STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_devinfo_setopt(void *handle, aiot_devinfo_option_t option, void *data);

/**
 * @brief Ends the devinfo session and releases the allocated resources.
 *
 * @param[in] handle The pointer that points to the devinfo session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The operation fails.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_devinfo_deinit(void **handle);

/**
 * @brief Sends a devinfo request to the devinfo server.
 *
 * @param handle The devinfo session handler.
 * @param msg The to-be-sent packet that carries the request to delete or update device tags.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The request fails to be sent.
 * @retval >=STATE_SUCCESS The request was sent.
 */
int32_t aiot_devinfo_send(void *handle, aiot_devinfo_msg_t *msg);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_DEVINFO_API_H__ */

