/**
 * @file aiot_diag_api.h
 * @brief The header file of the diag module. This module diagnoses the SDK client.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 */
#ifndef __AIOT_DIAG_API_H__
#define __AIOT_DIAG_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x0.00 to -0x0.FF are status codes of the diag module.
 */
#define STATE_DIAG_BASE                                             (-0x1400)

/**
 * @brief The description of STATE_DIAG_CODE1.
 */
#define STATE_DIAG_LOG_UNKNOWN_STATE_CODE_BASE                      (-0x1401)

/**
 * @brief The description of STATE_DIAG_CODE2.
 */
#define STATE_DIAG_CODE2                                            (-0x1402)

/**
  * @brief Pub message failed
  */
#define STATE_DIAG_PUB_FAILED                                       (-0x1403)

/**
 * @brief The type of the message that is returned when the diag module receives a packet from the network.
 */
typedef enum {
    AIOT_DIAGRECV_DIAG_CONTROL
} aiot_diag_recv_type_t;

/**
 * @brief The content of the message that is returned when the diag module receives a packet from the network.
 */
typedef struct {
    /**
     * @brief The type of the message. For more information, see @ref aiot_diag_recv_type_t.
     */
    aiot_diag_recv_type_t  type;
    union {
        /**
         * @brief The command that is received from IoT Platform.
         */
        struct {
            /**
             * @brief 0: disables the diagnosis feature. 1: enables the diagnosis feature.
             */
            uint32_t data;
        } diag_control;
    } data;
} aiot_diag_recv_t;

/**
 * @brief The diag function that notifies your applications when the dynregmq module receives a packet from the network.
 *
 * @param[in] handle The diag session handler.
 * @param[in] packet The struct of the diag packet. The struct stores the content of the diag packet.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_diag_recv_handler_t)(void *handle, const aiot_diag_recv_t *packet, void *userdata);

/**
 * @brief The type of the diag module event that is returned.
 */
typedef enum {
    /**
     * @brief The alert information that is generated by the diag module.
     */
    AIOT_DIAGEVT_ALERT
} aiot_diag_event_type_t;

/**
 * @brief The content of the diag module event that is returned.
 */
typedef struct {
    /**
     * @brief The type of the event. For more information, see @ref aiot_diag_event_type_t.
     */
    aiot_diag_event_type_t  type;
    union {
        struct {
            /**
             * @brief The name of the alerting module.
             */
            char *module_name;
            /** 
             * @brief The level of the alert.
             */
            char *level;
            /**
             * @brief The description of the alert.
             */
            char *desc;
        } alert;
    } data;
} aiot_diag_event_t;

/**
 * @brief The callback function that notifies your applications when a diag module event is triggered.
 *
 * @param[in] handle The diag session handler.
 * @param[in] event, The content of the diag module event.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (*aiot_diag_event_handler_t)(void *handle, const aiot_diag_event_t *event, void *userdata);

/**
 * @brief The settings of the diagnosis items.
 */
typedef struct {
    /**
     * @brief Specifies whether to diagnose the item.
     * 
     * @details
     * 
     * 0: does not diagnose the item. 1: diagnoses the item.
     */
    uint8_t enabled;
    /**
     * @brief The minimum interval between two consecutive alerts for the item.
     */
    uint32_t interval_ms;
    /**
     * @brief The threshold of warning alerts.
     */
    int64_t warning_threashold;
    /**
     * @brief The threshold of fatal alerts.
     */
    int64_t fatal_threshold;
} aiot_diag_config_t;

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_diag_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_diag_setopt.
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_DIAGOPT_MQTT_HANDLE is set.
 *
 *    void *mqtt_handle = aiot_mqtt_init();
 *    aiot_diag_setopt(diag_handle, AIOT_DIAGOPT_MQTT_HANDLE, mqtt_handle);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_DIAGOPT_LOCAL_REPORT_ENABLED is set.
 *
 *    uint8_t local_report_enabled = 1;
 *    aiot_mqtt_setopt(diag_handle, AIOT_DIAGOPT_LOCAL_REPORT_ENABLED, (void *)&local_report_enabled);
 */
typedef enum {
    /**
     * @brief The MQTT connection handler that is required for a diag session. An MQTT connection must be established before you set the handler.
     *
     * @details
     *
     * Data type: (void *).
     */
    AIOT_DIAGOPT_MQTT_HANDLE,

    /**
     * @brief Specifies whether the callback function of diag module events returns alert information.
     * 
     * @details
     * 
     * 0: does not return alert information. 1: returns alert information.
     * 
     * Data type: (uint8_t *).
     */
    AIOT_DIAGOPT_LOCAL_REPORT_ENABLED,

    /**
     * @brief Specifies whether to report alert information to IoT Platform.
     * 
     * @details
     * 
     * 0: do not report alert information to IoT Platform. 1: report alert information to IoT Platform.
     * 
     * Data type: (uint8_t *).
     */
    AIOT_DIAGOPT_CLOUD_REPORT_ENABLED,

    /**
     * @brief The settings of the alert about the time that is required for establishment of an MQTT connection.
     * 
     * @details
     * 
     * Data type: ( @ref aiot_diag_config_t ).
     */
    AIOT_DIAGOPT_MQTT_CONNECTION,

    /**
     * @brief The settings of the alert about MQTT heartbeat loss.
     * 
     * @details
     * 
     * Data type: ( @ref aiot_diag_config_t ).
     */
    AIOT_DIAGOPT_MQTT_HEARTBEAT,

    /**
     * @brief The settings of the alert about the waiting time before an Alink response is received.
     * 
     * @details
     * 
     * Data type: ( @ref aiot_diag_config_t ).
     */
    AIOT_DIAGOPT_ALINK_UPLINK,

    /**
     * @brief The function that notifies your applications when the SDK client receives a packet.
     * 
     * @details
     * 
     * Data type: ( @ref aiot_diag_recv_handler_t ).
     */
    AIOT_DIAGOPT_RECV_HANDLER,

    /**
     * @brief The callback function that notifies your applications when a diag module event is triggered.
     *
     * @details
     *
     * Data type: ( @ref aiot_diag_event_handler_t ).
     */
    AIOT_DIAGOPT_EVENT_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details The SDK client returns the context pointer when the callback function that is specified by the AIOT_DIAGOPT_RECV_HANDLER or AIOT_DIAGOPT_EVENT_HANDLER option is called.
     * 
     * Data type: (void *).
     */
    AIOT_DIAGOPT_USERDATA,
    AIOT_DIAGOPT_MAX
} aiot_diag_option_t;

/**
 * @brief Creates a diag session and retains the default settings for the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the diag session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_diag_init(void);

/**
 * @brief Configures the diag session.
 *
 * @param[in] handle The diag session handler.
 * @param[in] option The configuration data. For more information, see @ref aiot_diag_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_diag_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The configuration failed.
 * @retval >=STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_diag_setopt(void *handle, aiot_diag_option_t option, void *data);

/**
 * @brief Ends the diag session and releases the allocated resources.
 *
 * @param[in] handle The pointer that points to the diag session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The operation failed.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_diag_deinit(void **handle);

/**
 * @brief Diagnoses the SDK client.
 *
 * @param handle The diag session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request was sent.
 */
int32_t aiot_diag_start(void *handle);

/**
 * @brief Stops diagnosing the SDK client.
 *
 * @param handle The diag session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  No data was received.
 * @retval >=STATE_SUCCESS Data was received.
 */
int32_t aiot_diag_stop(void *handle);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_DIAG_API_H__ */

