/**
 * @file aiot_dynregmq_api.h
 * @brief The header file of the dynregmq module. This module provides dynamic device registration feature based on MQTT.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 */
#ifndef __AIOT_DYNREGMQ_API_H__
#define __AIOT_DYNREGMQ_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x0500 to -0x05FF are status codes of the Bootstrap module.
 */
#define STATE_DYNREGMQ_BASE                                             (-0x0500)

/**
 * @brief The DYNREGMQ session failed to be ended. After @ref aiot_dynregmq_deinit was called, other API operations were still running but the timeout period expired.
 */
#define STATE_DYNREGMQ_DEINIT_TIMEOUT                                   (-0x0501)

/**
 * @brief Call @ref aiot_dynregmq_send_request first to send a dynamic registration request.
 */
#define STATE_DYNREGMQ_NEED_SEND_REQUEST                                (-0x0502)

/**
 * @brief No response was received before the timeout period expired.
 */
#define STATE_DYNREGMQ_AUTH_TIMEOUT                                     (-0x0503)

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_dynregmq_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_dynregmq_setopt.
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_DYNREGMQOPT_HOST is set.
 *
 *    char *host = "xxx";
 *    aiot_dynregmq_setopt(dynregmq_handle, AIOT_DYNREGMQOPT_HOST, host);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_DYNREGMQOPT_PORT is set.
 *
 *    uint16_t port = 443;
 *    aiot_mqtt_setopt(dynregmq_handle, AIOT_DYNREGMQOPT_PORT, (void *)&port);
 */
typedef enum {
    /**
     * @brief The security credential that is used for connection establishment. To implement MQTT dynamic registration, the connection must be established over TLS.
     *
     * @details
     *
     * The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * Set the option parameter of @ref aiot_sysdep_network_cred_t to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA. Use TSL to establish the connection.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_DYNREGMQOPT_NETWORK_CRED,

    /**
     * @brief The domain name or IP address of the server for MQTT dynamic registration.
     *
     * @details
     *
     * The endpoints of IoT Platform MQTT dynamic registration servers. Replace ${pk} with the product key.
     *
     * Use TLS certificates to establish the connection.
     *
     * | Domain name                                     | Region                | Port
     * |-------------------------------------------------|-----------------------|---------
     * | ${pk}.iot-as-mqtt.cn-shanghai.aliyuncs.com      | China (Shanghai)      | 443
     * | ${pk}.iot-as-mqtt.ap-southeast-1.aliyuncs.com   | Singapore (Singapore) | 443
     * | ${pk}.iot-as-mqtt.ap-northeast-1.aliyuncs.com   | Japan (Tokyo)         | 443
     * | ${pk}.iot-as-mqtt.us-west-1.aliyuncs.com        | US (Silicon Valley)   | 443
     * | ${pk}.iot-as-mqtt.eu-central-1.aliyuncs.com     | Germany (Frankfurt)   | 443
     *
     * Data type: (char *).
     */
    AIOT_DYNREGMQOPT_HOST,

    /**
     * @brief The number of the port that is used by the MQTT dynamic registration server.
     *
     * @details
     *
     * For connections to the MQTT dynamic registration server of IoT Platform:
     *
     * TLS must be used to establish the connection. Therefore, set the port number to 443.
     *
     * Data type: (uint16_t *).
     */
    AIOT_DYNREGMQOPT_PORT,

    /**
     * @brief The product key of the device. You can obtain the product key from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_DYNREGMQOPT_PRODUCT_KEY,

    /**
     * @brief The product secret of the device. You can obtain the product secret from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_DYNREGMQOPT_PRODUCT_SECRET,

    /**
     * @brief The name of the device. You can obtain the device name from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_DYNREGMQOPT_DEVICE_NAME,

    /**
     * @brief The period of time during which a message sent in the dynregmq session is consumable.
     *
     * @details
     *
     * Data type: (uint32_t). Default value: (5 * 1000) ms.
     */
    AIOT_DYNREGMQOPT_SEND_TIMEOUT_MS,

    /**
     * @brief The period of time during which a message received in the dynregmq session is consumable.
     *
     * @details
     *
     * Data type: (uint32_t). Default value: (5 * 1000) ms.
     */
    AIOT_DYNREGMQOPT_RECV_TIMEOUT_MS,

    /**
     * @brief The function that notifies your applications when the SDK client receives a packet.
     *
     * @details
     *
     * Data type: (aiot_dynregmq_http_recv_handler_t).
     */
    AIOT_DYNREGMQOPT_RECV_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * The SDK client returns the context pointer when the callback function that is specified by the AIOT_DYNREGMQOPT_RECV_HANDLER or AIOT_DYNREGMQOPT_EVENT_HANDLER option is called.
     *
     * Data type: (void *).
     */
    AIOT_DYNREGMQOPT_USERDATA,

    /**
     * @brief The timeout period of receiving a message.
     *
     * @details
     *
     * Data type: (uint32_t). Default value: (5 * 1000) ms.
     */
    AIOT_DYNREGMQOPT_TIMEOUT_MS,

    /**
     * @brief The timeout period for other API operations to complete when you end a dynregmq session.
     *
     * @details
     *
     * If you call @ref aiot_dynregmq_deinit to end an MQTT connection and then call another aiot_dynregmq_xxx API operation, the operation returns the @ref STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_dynregmq_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_DYNREGMQOPT_DEINIT_TIMEOUT_MS,

    /**
     * @brief Specifies whether to enable the whitelist feature.
     *
     * @details
     *
     *  1. If you set this option to 0, the whitelist feature is enabled. In this case, you must register a device name in the console in advance. After the dynamic registration is complete, the device secret is returned.
     *  You can obtain the device secret from AIOT_DYNREGMQRECV_DEVICEINFO_WL.
     *  2. If you set this option to 1, the whitelist feature is disabled. In this case, you do not need to register device names in the console in advance. After the dynamic registration is complete, an MQTT connection is returned.
     *  You can obtain the client ID, username, and password from AIOT_DYNREGMQRECV_DEVICEINFO_NWL.  
     *  You can then call the aiot_mqtt_setopt API operation to set the AIOT_MQTTOPT_CLIENTID, AIOT_MQTTOPT_USERNAME, and AIOT_MQTTOPT_PASSWORD options for the MQTT connection handler.
      
     *
     * Data type: (uint8_t *). Default value: (0).
     *
     */
    AIOT_DYNREGMQOPT_NO_WHITELIST,

    /**
     * @brief The ID of a purchased IoT Platform instance. The instance ID is required if a purchased instance is used and the whitelist feature is disabled.
     *
     * @details
     *
     * Data type: (char *).
     *
     */
    AIOT_DYNREGMQOPT_INSTANCE_ID,

    AIOT_DYNREGMQOPT_MAX
} aiot_dynregmq_option_t;


/**
 * @brief The type of the message that is returned when the dynregmq module receives a packet from the network.
 */
typedef enum {
    /**
     * @brief The device information that the server returns when the whitelist feature is enabled.
     */
    AIOT_DYNREGMQRECV_DEVICEINFO_WL,

    /**
     * @brief The device information that the server returns when the whitelist feature is disabled.
     */
    AIOT_DYNREGMQRECV_DEVICEINFO_NWL,
} aiot_dynregmq_recv_type_t;

/**
 * @brief The content of the message that is returned when the dynregmq module receives a packet from the network.
 */
typedef struct {
    /**
     * @brief The type of the message. For more information, see @ref aiot_dynregmq_recv_type_t.
     */
    aiot_dynregmq_recv_type_t  type;
    union {
        /**
         * @brief The device information that the server returns when the whitelist feature is enabled.
         */
        struct {
            char *device_secret;
        } deviceinfo_wl;

        /**
         * @brief The device information that the server returns when the whitelist feature is disabled.
         */
        struct {
            char *clientid;
            char *username;
            char *password;
        } deviceinfo_nwl;
    } data;
} aiot_dynregmq_recv_t;

/**
 * @brief The callback function that notifies your applications when the dynregmq module receives a packet from the network.
 *
 * @param[in] handle The dynregmq session handler.
 * @param[in] packet The struct of the dynregmq packet. The struct stores the content of the dynregmq packet.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_dynregmq_recv_handler_t)(void *handle,
        const aiot_dynregmq_recv_t *packet, void *userdata);


/**
 * @brief Creates a dynregmq session and retains the default settings for the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the dynregmq session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_dynregmq_init(void);

/**
 * @brief Configures the dynregmq session.
 *
 * @param[in] handle The dynregmq session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_dynregmq_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_dynregmq_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The configuration failed.
 * @retval >=STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_dynregmq_setopt(void *handle, aiot_dynregmq_option_t option, void *data);

/**
 * @brief Ends the dynregmq session and releases the allocated resources.
 *
 * @param[in] handle The pointer that points to the dynregmq session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The operation failed.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_dynregmq_deinit(void **handle);

/**
 * @brief Sends a dynregmq request to the dynregmq server.
 *
 * @param handle The dynregmq session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request was sent.
 */
int32_t aiot_dynregmq_send_request(void *handle);

/**
 * @brief Receives a dynregmq message.
 *
 * @param handle The dynregmq session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  No data was received.
 * @retval >=STATE_SUCCESS Data was received.
 */
int32_t aiot_dynregmq_recv(void *handle);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_DYNREGMQMQ_API_H__ */

