/**
 * @file aiot_dynreg_api.h
 * @brief The header file of the dynreg module. This module supports obtaining device information.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 */
#ifndef __AIOT_DYNREG_API_H__
#define __AIOT_DYNREG_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x0600 to -0x06FF are status codes of the dynreg module.
 */
#define STATE_DYNREG_BASE                                             (-0x0600)

/**
 * @brief The MQTT connection failed to be ended. After @ref aiot_dynreg_deinit was called, other API operations were still running but the timeout period expired.
 */
#define STATE_DYNREG_DEINIT_TIMEOUT                                   (-0x0601)

/**
 * @brief Call @ref aiot_dynreg_send_request first to send a dynreg request.
 */
#define STATE_DYNREG_NEED_SEND_REQUEST                                (-0x0602)

/**
 * @brief The dynreg module returned an HTTP status code that indicates an error.
 */
#define STATE_DYNREG_INVALID_STATUS_CODE                              (-0x0603)

/**
 * @brief An invalid device secret is received.
 */
#define STATE_DYNREG_INVALID_DEVICE_SECRET                            (-0x0604)

/**
 * @brief The type of the packet that is returned when the dynreg module receives a message from the network.
 */
typedef enum {
    /**
     * @brief The HTTP status code that is returned by the dynreg module.
     */
    AIOT_DYNREGRECV_STATUS_CODE,
    /**
     * @brief The device information that is returned by the dynreg module.
     */
    AIOT_DYNREGRECV_DEVICE_INFO,
} aiot_dynreg_recv_type_t;

/**
 * @brief The content of the packet that is returned when the dynreg module receives a message from the network.
 */
typedef struct {
    /**
     * @brief The type of the message. For more information, see @ref aiot_dynreg_recv_type_t.
     */
    aiot_dynreg_recv_type_t  type;
    union {
        /**
         * @brief The HTTP status code that is returned by the dynreg module.
         */
        struct {
            uint32_t code;
        } status_code;
        /**
         * @brief The device information that is returned by the dynreg module.
         */
        struct {
            char *device_secret;
        } device_info;
    } data;
} aiot_dynreg_recv_t;

/**
 * @brief The function that notifies your applications when the dynreg module receives a message from the network.
 *
 * @param[in] handle The dynreg session handler.
 * @param[in] packet The struct of the message that the dynreg module receives. The struct stores the message content.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_dynreg_recv_handler_t)(void *handle,
        const aiot_dynreg_recv_t *packet, void *userdata);

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_dynreg_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_dynreg_setopt.
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_DYNREGOPT_HOST is set.
 *
 *    char *host = "xxx";
 *    aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_HOST, host);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_DYNREGOPT_PORT is set.
 *
 *    uint16_t port = 443;
 *    aiot_mqtt_setopt(dynreg_handle, AIOT_DYNREGOPT_PORT, (void *)&port);
 */
typedef enum {
    /**
     * @brief The security credential that is used for the establishment of the HTTP connection to the firmware server.
     *
     * @details
     *
     * The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * Set the option parameter of @ref aiot_sysdep_network_cred_t to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA. Use TSL to establish the connection.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_DYNREGOPT_NETWORK_CRED,

    /**
     * @brief The domain name or IP address of the server for HTTP dynamic registration.
     *
     * @details
     *
     * The endpoints of IoT Platform HTTP dynamic registration servers.
     *
     * | Domain name                                        | Region    | Port
     * |-------------------------------------------------|---------|---------
     * | iot-auth.cn-shanghai.aliyuncs.com               | Regions in mainland China    | 443
     * | iot-auth.ap-southeast-1.aliyuncs.com            | Regions outside mainland China     | 443
     *
     * Data type: (char *).
     */
    AIOT_DYNREGOPT_HOST,

    /**
     * @brief The number of the port that is used by the HTTP dynamic registration server.
     *
     * @details
     *
     * For connections to the HTTP dynamic registration server of IoT Platform:
     *
     * TLS must be used to establish the connection. Therefore, set the port number to 443.
     *
     * Data type: (uint16_t *).
     */
    AIOT_DYNREGOPT_PORT,

    /**
     * @brief The product key of the device. You can obtain the product key from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_DYNREGOPT_PRODUCT_KEY,

    /**
     * @brief The product secret of the device. You can obtain the product secret from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_DYNREGOPT_PRODUCT_SECRET,

    /**
     * @brief The name of the device. You can obtain the device name from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_DYNREGOPT_DEVICE_NAME,

    /**
     * @brief The period of time during which a message sent in the dynreg session is consumable.
     *
     * @details
     *
     * Data type: (uint32_t). Default value: (5 * 1000) ms.
     */
    AIOT_DYNREGOPT_SEND_TIMEOUT_MS,

    /**
     * @brief The period of time during which a message received in the dynreg session is consumable.
     *
     * @details
     *
     * Data type: (uint32_t). Default value: (5 * 1000) ms.
     */
    AIOT_DYNREGOPT_RECV_TIMEOUT_MS,

    /**
     * @brief The callback function that notifies your applications when the dynreg module receives a packet from the network.
     *
     * @details
     *
     * Data type: (aiot_dynreg_http_recv_handler_t).
     */
    AIOT_DYNREGOPT_RECV_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * The SDK client returns the context pointer when the callback function that is specified by the AIOT_DYNREGOPT_RECV_HANDLER or AIOT_DYNREGOPT_EVENT_HANDLER option is called.
     *
     * Data type: (void *).
     */
    AIOT_DYNREGOPT_USERDATA,

    /**
     * @brief The timeout period for the dynreg module to receive a message.
     *
     * @details
     *
     * Data type: (uint32_t). Default value: (5 * 1000) ms.
     */
    AIOT_DYNREGOPT_TIMEOUT_MS,

    /**
     * @brief The timeout period for other API operations to complete when you end a dynreg session.
     *
     * @details
     *
     * If you call @ref aiot_dynreg_deinit to end an MQTT connection and then call another aiot_dynreg_xxx API operation, the operation returns the @ref STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_dynreg_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_DYNREGOPT_DEINIT_TIMEOUT_MS,
    AIOT_DYNREGOPT_MAX
} aiot_dynreg_option_t;

/**
 * @brief Creates a dynreg session and retains the default settings for the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the dynreg session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_dynreg_init(void);

/**
 * @brief Configures the dynreg session.
 *
 * @param[in] handle The dynreg session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_dynreg_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_dynreg_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The configuration succeeded.
 * @retval >=STATE_SUCCESS The configuration failed.
 *
 */
int32_t aiot_dynreg_setopt(void *handle, aiot_dynreg_option_t option, void *data);

/**
 * @brief Ends the dynreg session and releases the allocated resources.
 *
 * @param[in] handle The pointer that points to the dynreg session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The operation failed.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_dynreg_deinit(void **handle);

/**
 * @brief Sends a dynreg request to the dynreg server.
 *
 * @param handle The dynreg session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request was sent.
 */
int32_t aiot_dynreg_send_request(void *handle);

/**
 * @brief Receives a dynreg message.
 *
 * @param handle The dynreg session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  No data is received.
 * @retval >=STATE_SUCCESS Data is received.
 */
int32_t aiot_dynreg_recv(void *handle);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_DYNREG_API_H__ */

