/**
 * @file aiot_logpost_api.h
 * @brief The header file of the logpost module. This module sends device logs to IoT Platform.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 *
 * 1. Create an MQTT connection.
 *
 * 2. Call aiot_logpost_init to create a logpost session and save the session handler.
 *
 * 3. Call aiot_logpost_setopt to set the AIOT_LOGPOSTOPT_MQTT_HANDLE option. This option specifies an MQTT connection handler and is required.
 *
 * 4. Call aiot_logpost_setopt to set the AIOT_LOGPOSTOPT_EVENT_HANDLER and AIOT_LOGPOSTOPT_USER_DATA options. AIOT_LOGPOSTOPT_EVENT_HANDLER specifies the callback function that receives data and AIOT_LOGPOSTOPT_USER_DATA specifies the user context.
 *
 * 5. Establish an MQTT connection before calling aiot_logpost_send.
 *
 */
#ifndef __AIOT_LOGPOST_API_H__
#define __AIOT_LOGPOST_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x1500 to -0x15FF are status codes of the logpost module.
 */
#define STATE_LOGPOST_BASE                                              (-0x1500)

/**
 * @brief No MQTT connection handler was specified by calling @ref aiot_logpost_setopt.
 */
#define STATE_LOGPOST_MQTT_HANDLE_IS_NULL                               (-0x1501)

/**
 * @brief The log report feature was disabled in IoT Platform.
 */
#define STATE_LOGPOST_POST_TURN_OFF                                     (-0x1502)

/**
 * @brief The log level was invalid.
 */
#define STATE_LOGPOST_LOGLEVEL_ERROR                                    (-0x1503)

/**
 * @brief The module name field of the log message was NULL.
 */
#define STATE_LOGPOST_LOG_MODULE_NAME_IS_NULL                           (-0x1504)

/**
 * @brief The log content field of the log message was NULL.
 */
#define STATE_LOGPOST_LOG_CONTENT_IS_NULL                               (-0x1505)

/**
 * @brief The length of the log content string in the log message was greater than 4,096 bytes.
 */
#define STATE_LOGPOST_LOG_CONTENT_TOO_LONG                              (-0x1506)

/**
 * @brief The log of receiving a message from IoT Platform.
 */
#define STATE_LOGPOST_LOG_RECV                                          (-0x1507)

/**
 * @brief The log of failure in parsing a message that is received from IoT Platform.
 */
#define SATAE_LOGPOST_LOG_PARSE_MSG_FAILED                              (-0x1508)


/**
 * @brief The options that you can set by using the option parameter of @ref aiot_logpost_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_logpost_setopt.
 */
typedef enum {
    /**
     * @brief The MQTT connection handler that is associated with the logpost module.
     *
     * @details
     *
     * * The logpost module depends on the MQTT module. You must specify a valid MQTT connection handler to ensure that the logpost module can work as expected. Data type: (void *).
     */
    AIOT_LOGPOSTOPT_MQTT_HANDLE,

    /**
     * @brief The callback function that notifies your applications when packets are received. The function is called when the logpost module receives a packet. Data type: (aiot_logpost_recv_handler_t).
     */
    AIOT_LOGPOSTOPT_EVENT_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client. Data type: (void *).
     *
     * @details The SDK client returns the context pointer when the callback function that is specified by the AIOT_LOGPOSTOPT_RECV_HANDLER option is called.
     */
    AIOT_LOGPOSTOPT_USERDATA,

    /**
     * @brief log switch for SDK internal log.
     *
     * @details 1 means post SDK internal log to cloud. 0 means NOT.
     */
    AIOT_LOGPOSTOPT_SYS_LOG,

    /**
     * @brief The maximum number of configuration options. The number is not configurable.
     */
    AIOT_LOGPOSTOPT_MAX,
} aiot_logpost_option_t;


/**
 * @brief The enumeration of log levels.
 */
typedef enum {
    AIOT_LOGPOST_LEVEL_FATAL,
    AIOT_LOGPOST_LEVEL_ERR,
    AIOT_LOGPOST_LEVEL_WARN,
    AIOT_LOGPOST_LEVEL_INFO,
    AIOT_LOGPOST_LEVEL_DEBUG,
} aiot_logpost_level_t;

/**
 * @brief The struct of log data.
 *
 */
typedef struct {
    /**
     * @brief The millisecond-level UTC timestamp. The timestamp will be displayed on the device log page of the IoT Platform console.
     */
    uint64_t timestamp;

    /**
     * @brief The level of the log. For more information, see @ref aiot_logpost_level_t.
     */
    aiot_logpost_level_t loglevel;

    /**
     * @brief The module name. <b>It must end with '\0'.</b>
     */
    char *module_name;

    /**
     * @brief The code that specifies the log status.
     */
    int32_t code;

    /**
     * @brief The identifier of the message that is received from IoT Platform. You can obtain the identifier from the callback function that the data-model module uses to receive data. If you specify 0, this field is not reported.
     */
    uint64_t msg_id;

    /**
     * @brief The log content. <b>It must end with '\0'.</b>
     */
    char *content;
} aiot_logpost_msg_t;


/**
 * @brief The type of the logpost module event that is returned.
 */
typedef enum {
    /**
     * @brief The logging configuration data that is received from IoT Platform.
     */
    AIOT_LOGPOSTEVT_CONFIG_DATA,
} aiot_logpost_event_type_t;

/**
 * @brief The data of the logpost module event that is returned.
 */
typedef struct {
    /**
     * @brief The type of the event. For more information, see @ref aiot_logpost_event_type_t.
     */
    aiot_logpost_event_type_t  type;

    union {
        /**
         * @brief The struct of the log configuration data.
         */
        struct {
            /**
             * @brief Specifies whether to enable the log report feature. 0: disables the log report feature. 1: enables the log report feature.
             */
            uint8_t on_off;
        } config_data;
    } data;
} aiot_logpost_event_t;


/**
 * @brief The callback function that notifies your applications when a logpost module event is triggered.
 *
 * @param[in] handle The logpost session handler.
 * @param[in] event The content of the logpost module event.
 * @param[in] userdata The pointer that points to the user context cached by the SDK. You can set the @ref AIOT_LOGPOSTOPT_USERDATA parameter of @ref aiot_logpost_setopt to specify the context that needs to be cached.
 *
 * @return void
 */
typedef void (*aiot_logpost_event_handler_t)(void *handle,
        const aiot_logpost_event_t *event, void *userdata);

/**
 * @brief Creates a logpost session and retains the default settings for the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the logpost session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 */
void *aiot_logpost_init(void);

/**
 * @brief Configures the logpost session.
 *
 * @param[in] handle The logpost session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_logpost_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_logpost_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The configuration failed.
 * @retval STATE_SUCCESS The configuration succeeded.
 * @retval STATE_USER_INPUT_NULL_POINTER The handle or data input parameter was null.
 * @retval STATE_USER_INPUT_OUT_RANGE The enumeration value of the option input parameter is greater than or equal to AIOT_LOGPOSTOPT_MAX.
 * @retval others For more information, see @ref aiot_state_api.h.
 */
int32_t aiot_logpost_setopt(void *handle, aiot_logpost_option_t option, void *data);

/**
 * @brief Sends a log message to IoT Platform.
 *
 * @param[in] handle The logpost session handler.
 * @param[in] msg    The message struct that stores the information such as the module and level of a log. For more information, see @ref aiot_logpost_msg_t.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The request was sent.
 * @retval STATE_USER_INPUT_NULL_POINTER The <i>handle</i> or <i>msg</i> input parameter was NULL.
 * @retval STATE_SYS_DEPEND_MALLOC_FAILED Memory allocation failed.
 * @retval STATE_LOGPOST_MQTT_HANDLE_IS_NULL No MQTT connection handler was specified by calling @ref aiot_logpost_setopt.
 * @retval others For more error codes, see @ref aiot_state_api.h or @ref STATE_SHADOW_BASE.
 *
 */
int32_t aiot_logpost_send(void *handle, aiot_logpost_msg_t *msg);

/**
 * @brief Ends the logpost session and releases allocated resources.
 *
 * @param[in] handle The pointer that points to the logpost session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The operation failed.
 * @retval >=STATE_SUCCESS The operation succeeded.
 */
int32_t aiot_logpost_deinit(void **handle);


#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_LOGPOST_API_H__ */

