/**
 * @file aiot_ntp_api.h
 * @brief The header file of the NTP module. This module obtains the UTC time of the IoT Platform server.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 *
 * The NTP module obtains the UTC time of the IoT Platform server. The following procedure describes how to use the API of the NTP module:
 *
 * 1. Make sure that an MQTT connection to IoT Platform is established. For more information, see @ref aiot_mqtt_api.h.
 *
 * 2. Call @ref aiot_ntp_init to initialize an NTP session and obtain the session handler.
 *
 * 3. Call @ref aiot_ntp_setopt to set the parameters of the NTP session. For information about the parameters, see @ref aiot_ntp_setopt.
 *
 * 4. Call @ref aiot_ntp_send_request to send an NTP request.
 *
 * 5. The SDK client parses the UTC time from the response that is received. The callback function that is specified by the @ref AIOT_NTPOPT_RECV_HANDLER parameter of @ref aiot_ntp_setopt then returns the UTC time.
 *
 */
#ifndef __AIOT_NTP_API_H__
#define __AIOT_NTP_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x1100 to -0x11FF are status codes of the NTP module.
 */
#define STATE_NTP_BASE                                             (-0x1100)

/**
 * @brief The MQTT connection handler is not specified. Call @ref aiot_ntp_setopt to specify the MQTT connection handler.
 */
#define STATE_NTP_MISSING_MQTT_HANDLE                              (-0x1101)

/**
 * @brief The type of the message that is returned when the NTP module receives a packet from the network.
 */
typedef enum {
    AIOT_NTPRECV_LOCAL_TIME
} aiot_ntp_recv_type_t;

/**
 * @brief The content of the message that is returned when the NTP module receives a packet from the network.
 */
typedef struct {
    /**
     * @brief The type of the message. For more information, see @ref aiot_ntp_recv_type_t.
     */
    aiot_ntp_recv_type_t  type;
    union {
        /**
         * @brief The UTC timestamp and the date that corresponds to the time zone specified by the @ref AIOT_NTPOPT_TIME_ZONE parameter.
         */
        struct {
            uint64_t timestamp;
            uint32_t year;
            uint32_t mon;
            uint32_t day;
            uint32_t hour;
            uint32_t min;
            uint32_t sec;
            uint32_t msec;
        } local_time;
    } data;
} aiot_ntp_recv_t;

/**
 * @brief The callback function that notifies your applications when the NTP module receives a packet from the network. 
 *
 * @param[in] handle The NTP session handler.
 * @param[in] packet The struct of the NTP packet. The struct stores the content of the NTP packet.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_ntp_recv_handler_t)(void *handle,
        const aiot_ntp_recv_t *packet, void *userdata);

/**
 * @brief The type of the NTP module event.
 */
typedef enum {
    /**
     * @brief Fields in the received NTP response were invalid.
     */
    AIOT_NTPEVT_INVALID_RESPONSE,
    /**
     * @brief The format of the time field in the received NTP response was invalid.
     */
    AIOT_NTPEVT_INVALID_TIME_FORMAT,
} aiot_ntp_event_type_t;

/**
 * @brief The NTP module event.
 */
typedef struct {
    /**
     * @brief The type of the NTP module event. For more information, see @ref aiot_ntp_event_type_t.
     *
     */
    aiot_ntp_event_type_t type;
} aiot_ntp_event_t;

/**
 * @brief The callback function of NTP module events.
 *
 * @details
 *
 * The function is called when an NTP module event is triggered.
 *
 */
typedef void (*aiot_ntp_event_handler_t)(void *handle, const aiot_ntp_event_t *event, void *userdata);

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_ntp_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_ntp_setopt.
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_NTPOPT_MQTT_HANDLE is set.
 *
 *    void *mqtt_handle = aiot_mqtt_init();
 *    aiot_ntp_setopt(ntp_handle, AIOT_NTPOPT_MQTT_HANDLE, mqtt_handle);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_NTPOPT_TIME_ZONE is set.
 *
 *    int8_t time_zone = 8;
 *    aiot_mqtt_setopt(ntp_handle, AIOT_NTPOPT_TIME_ZONE, (void *)&time_zone);
 */
typedef enum {
    /**
     * @brief The MQTT handler that is required for an NTP session. An MQTT connection must be established before you set the MQTT handler.
     *
     * @details
     *
     * Data type: (void *).
     */
    AIOT_NTPOPT_MQTT_HANDLE,

    /**
     * @brief The local time zone. The UTC timestamp received in the NTP session is converted to this time zone. @ref aiot_ntp_recv_handler_t then returns the converted time.
     *
     * @details
     *
     * For example, set this parameter to 8 for UTC+8 and -3 for UTC-3.
     *
     * Data type: (int8_t *).
     */
    AIOT_NTPOPT_TIME_ZONE,

    /**
     * @brief The callback function that notifies your applications when the SDK client receives a network packet.
     * 
     * @details
     * 
     * Data type: ( @ref aiot_ntp_recv_handler_t ).
     */
    AIOT_NTPOPT_RECV_HANDLER,

    /**
     * @brief The callback function that notifies your applications when an NTP module event is triggered.
     *
     * @details
     *
     * Data type: ( @ref aiot_ntp_event_handler_t ).
     */
    AIOT_NTPOPT_EVENT_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details The SDK client returns the context pointer when the callback function that is specified by the AIOT_NTPOPT_RECV_HANDLER or AIOT_NTPOPT_EVENT_HANDLER option is called.
     * 
     * Data type: (void *).
     */
    AIOT_NTPOPT_USERDATA,

    /**
     * @brief The timeout period for other API operations to complete when you end an NTP session.
     *
     * @details
     *
     * If you call @ref aiot_ntp_deinit to end an NTP session and then call another aiot_ntp_xxx API operation, the operation returns the @ref STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_ntp_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_NTPOPT_DEINIT_TIMEOUT_MS,
    AIOT_NTPOPT_MAX
} aiot_ntp_option_t;

/**
 * @brief Creates an NTP session and retains the default settings for the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the NTP session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_ntp_init(void);

/**
 * @brief Configures the NTP session.
 *
 * @details
 *
 * The following list describes the common parameters of the NTP session.
 *
 * + `AIOT_NTPOPT_MQTT_HANDLE`: The handler of the established MQTT connection.
 *
 * + `AIOT_NTPOPT_TIME_ZONE`: The local time zone. The UTC timestamp received in the NTP session is converted to this time zone.
 *
 * + `AIOT_NTPOPT_RECV_HANDLER`: The callback function that is called after the SDK client receives and converts a timestamp.
 *
 * @param[in] handle The NTP session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_ntp_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_ntp_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The configuration failed.
 * @retval >=STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_ntp_setopt(void *handle, aiot_ntp_option_t option, void *data);

/**
 * @brief Ends the NTP session and releases the allocated resources.
 *
 * @param[in] handle The pointer that points to the NTP session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The operation failed.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_ntp_deinit(void **handle);

/**
 * @brief Sends an NTP request to the NTP server.
 *
 * @details
 *
 * The SDK client sends an NTP request and the callback function that is specified by the @ref AIOT_NTPOPT_RECV_HANDLER parameter of @ref aiot_ntp_setopt returns the current time.
 *
 * @param handle The NTP session handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request was sent.
 */
int32_t aiot_ntp_send_request(void *handle);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_NTP_API_H__ */

