/**
 * @file aiot_ota_api.h
 * @brief The header file of the OTA module. This module obtains update notifications and supports remote configuration.
 * @date 2019-12-27
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 * @details
 *
 * The OTA module interacts with the firmware update service of IoT Platform.
 
 *
 *     + To enable the OTA service, the device must call @ref aiot_ota_report_version to report the current firmware version. Otherwise, the device cannot receive update notifications.
 *     + To download firmware from IoT Platform, the device must call @ref aiot_download_send_request. The SDK client cannot automatically download firmware.
 *     + When firmware is being downloaded or after the download is complete, the device writes the firmware in the memory buffer to the flash. The SDK client does not include the logic of firmware burning.
 *     + After a firmware update is complete, the device must call @ref aiot_ota_report_version to report the new firmware version in use. Otherwise, IoT Platform considers the firmware update uncompleted.
 *     + After the device calls @ref aiot_ota_recv_handler_t to receive an update job, the device can call @ref aiot_ota_query_firmware again if the device ignores the former job.
 *
 * + You push the information such as the firmware download URL to the MQTT topic in the IoT Platform console. Therefore, a prerequisite of an OTA update is the establishment of an MQTT persistent connection.
 *
 */

#ifndef __AIOT_OTA_API_H__
#define __AIOT_OTA_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief The type of the OTA message that is received from IoT Platform. Available types include update notification and remote configuration.
 *
 * @details
 * The type of the MQTT packet that is passed to @ref aiot_ota_recv_handler_t.
 *
 */
typedef enum {

    /**
     * @brief The update notification message.
     *
     */
    AIOT_OTARECV_FOTA,

    /**
     * @brief The remote notification message.
     *
     */
    AIOT_OTARECV_COTA
} aiot_ota_recv_type_t;

/**
 * @brief The type of the digest method that is used by the OTA message. Available types include MD5 and SHA-256.
 *
 */

typedef enum {

    /**
     * @brief The MD5 algorithm.
     *
     */
    AIOT_OTA_DIGEST_MD5,

    /**
     * @brief The SHA-256 algorithm.
     *
     */
    AIOT_OTA_DIGEST_SHA256,
    AIOT_OTA_DIGEST_MAX
} aiot_ota_digest_type_t;

/**
 * @brief The information of the firmware update job that is received from IoT Platform. The information include the download URL, size, and signature of the firmware.
 *
 */
typedef struct {

    /**
     * @brief The product key of the device whose firmware is to be updated.
     *
     */
    char       *product_key;

    /**
     * @brief The name of the device whose firmware is to be updated.
     *
     */
    char       *device_name;

    /*
     * @brief The download URL of the new firmware.
     *
     */
    char       *url;

    /*
     * @brief The size of the firmware, in bytes.
     *
     */
    uint32_t    size_total;

    /*
     * @brief The digital signature method that IoT Platform uses to sign the firmware. For more information, see @ref aiot_ota_digest_type_t.
     *
     */
    uint8_t     digest_method;

    /*
     * @brief The digital firmware signature that is generated by IoT Platform.
     *
     */
    char       *expect_digest;

    /**
     * @brief The version of the firmware. If the job is a firmware update job, this property indicates the version of the firmware. If the job is a remote configuration job, this property indicates the configuration ID.
     *
     */
    char       *version;

    /**
     * @brief The module that corresponds to the firmware.
     *
     */
    char       *module;

    /**
     * @brief  *The MQTT connection handler that is used to report data to IoT Platform during the firmware update.
     */
    void       *mqtt_handle;

    /**
     * @brief The extra info delivered in ota message 
     *
     */
    char       *extra_data;
} aiot_download_task_desc_t;

/**
* @brief The OTA message that is received from IoT Platform, including the message type and the job description.
*
*/
typedef struct {

    /**
     * @brief The type of the OTA message that is received from IoT Platform. For more information, see @ref aiot_ota_recv_type_t.
     */
    aiot_ota_recv_type_t        type;

    /**
     * @brief The information of the firmware update job that is received from IoT Platform. The information include the download URL, size, and signature of the firmware. For more information, see @ref aiot_download_task_desc_t.
     */
    aiot_download_task_desc_t   *task_desc;
} aiot_ota_recv_t;

/**
 * @brief The callback function that is called when the device receives an OTA message from the MQTT topic. The function returns the destination firmware version. You can then determine the update policy. The update policy includes whether and when to perform the update.

 * @param[in] handle The OTA session handler.
 * @param[in] msg The OTA message that receives from IoT Platform.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_ota_recv_handler_t)(void *handle, const aiot_ota_recv_t *const msg, void *userdata);

/**
* @brief The type of the packet segments that are received during firmware download.
*
*/
typedef enum {

    /**
    * @brief The packet segments are transferred over HTTP.
    *
    */
    AIOT_DLRECV_HTTPBODY
} aiot_download_recv_type_t;

/**
* @brief The information of the packet segments that are received during firmware download. The information includes the packet type, the buffer address, the buffer size, and the download progress.
*
*/
typedef struct {

    /**
    * @brief The type of the packet segments that are received when the firmware is being downloaded. For more information, see @ref aiot_download_recv_type_t.
    *
    */
    aiot_download_recv_type_t type;

    struct {

        /**
        * @brief The address of the buffer that caches the firmware data during firmware download. The buffer is allocated by the SDK client. The SDK client releases the buffer when the callback function finishes. You must copy, paste, and then save the packet information.
        *
        */
        uint8_t *buffer;

        /**
        * @brief The size of the buffer that caches the firmware data during firmware download. The buffer is allocated by the SDK client. You can modify the buffer size by setting @ref AIOT_DLOPT_BODY_BUFFER_MAX_LEN.
        *
        */
        uint32_t len;

        /**
        * @brief The current download progress, in percentage.
        *
        */
        int32_t  percent;
    } data;
} aiot_download_recv_t;


/**
 * @brief The callback function that receives firmware data by segment after the update starts. By default, packet segments are transferred over HTTPS.

 * @param[in] handle The download session handler.
 * @param[in] packet The packet segments that are received from IoT Platform.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_download_recv_handler_t)(void *handle, const aiot_download_recv_t *packet,
        void *userdata);

/**
 * @brief The error code that both the device and IoT Platform use during an OTA process. IoT Platform locates errors based on the error code.
 *
 */

typedef enum {

    /**
     * @brief OTA update error.
     */
    AIOT_OTAERR_UPGRADE_FAILED = -1,

    /**
     * @brief Firmware download error.
     */
    AIOT_OTAERR_FETCH_FAILED = -2,

    /**
     * @brief Signature verification error.
     */
    AIOT_OTAERR_CHECKSUM_MISMATCH = -3,

    /**
     * @brief Firmware burning error.
     */
    AIOT_OTAERR_BURN_FAILED = -4
} aiot_ota_protocol_errcode_t;

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_ota_setopt.
 *
 */

typedef enum {

    /**
     * @brief The callback function that processes OTA messages.
     *
     * @details
     *
     * Two types of messages may be received: firmware update message and remote configuration message.
     *
     * Both types of messages contain the information such as download URL, version, digest method, and signature.
     *
     * You must determine the update policy during the callback. For example, you must determine whether and when to perform the update. If the update is required, call aiot_download_init to initialize a download session handler.
     * For more information, see fota_xxx_xxx.c in the demos directory.
     *
     * Data type: (void *).
     */
    AIOT_OTAOPT_RECV_HANDLER,

    /**
     * @brief Sets the MQTT session handler.
     *
     * @details
     *
     * An MQTT connection is required to report version, progress, and error codes during an OTA update.
     *
     * Data type: (void *).
     */
    AIOT_OTAOPT_MQTT_HANDLE,

    /**
      * @brief The user context that needs to be cached by the SDK client.
      *
      * @details
      *
      * The user context will be returned by @ref AIOT_OTAOPT_RECV_HANDLER.
      *
      * Data type: (void *).
      */
    AIOT_OTAOPT_USERDATA,

    /**
      * @brief The module name. This option is required for an OTA update of a connected module such as MCU.
      *
      * @details
      *
      * An OTA update may be performed on a connected module. The module name is required when the version is reported.
      * You can specify the module name by using this option.
      *
      * Data type: (void *).
      */
    AIOT_OTAOPT_MODULE,
    AIOT_OTAOPT_MAX
} aiot_ota_option_t;

/**
 * @brief The options that you can set by using the option parameter of @ref aiot_download_setopt.
 *
 */

typedef enum {

    /**
     * @brief The security credential that is used for the establishment of the HTTP connection to the firmware server.
     *
     * @details
     *
     * The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * 1. If this parameter is not specified, an HTTP connection is established over TCP.
     *
     * 2. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_NONE, an HTTP connection is established over TCP.
     *
     * 3. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA, an HTTP connection is established over TLS.
     *
     * 4. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_PSK, an HTTP connection is established over TLS-PSK.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_DLOPT_NETWORK_CRED,

    /**
     * @brief The number of the port that the device uses to access the firmware server over HTTP.
     *
     * @details
     *
     * If a TCP or TLS certificate is used, set the port number to 443.
     *
     * Data type: (uint16_t *).
     */
    AIOT_DLOPT_NETWORK_PORT,

    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when firmware data is received over HTTP.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_DLOPT_RECV_TIMEOUT_MS,

    /**
     * @brief The callback function that is called when HTTP data is received.
     *
     * @details
     *
     * Data type: (aiot_download_recv_handler_t).
     */
    AIOT_DLOPT_RECV_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * The user context will be returned by the userdata parameter of @ref aiot_download_recv_handler_t.
     *
     * Data type: (void *).
     */
    AIOT_DLOPT_USERDATA,

    /**
     * @brief The information of the download job.
     *
     * @details
     *
     * After an OTA message is received from the MQTT connection, you must set this option to allocate memory for the download session handler if you want to perform the update.
     * Pass the URL, version, digest method, signature, and other information from the OTA message to this option. The information is required for download.
     *
     * Data type: (aiot_download_task_desc_t *).
     *
     **/
    AIOT_DLOPT_TASK_DESC,

    /**
     * @brief The start position of an HTTP range request.
     *
     * @details
     *
     * The download request is a range request. This option specifies the byte from which the download starts.
     * If you want to download from the head, set this option to 0.
     *
     * Data type: (uint32_t *).
     *
     **/
    AIOT_DLOPT_RANGE_START,

    /**
     * @brief The end position of an HTTP range request.
     *
     * @details
     * The download request is a range request. This option specifies the byte at which the download ends.
     * For example, You want to download the first 10 bytes.
     * Then, set the start position to 0 and the end position to 9.
     *
     * Data type: (uint32_t *).
     *
     **/
    AIOT_DLOPT_RANGE_END,

    /**
    * @brief The maximum length of the HTTP request body each time @ref aiot_download_recv_handler_t can return. Each HTTP request is received during firmware download.
    *
    * @details
    * If the received data does not reach the maximum length, the aiot_download_recv_handler_t callback function returns the received length of body.
    *
    * Data type: (uint32_t *). Default value: (2 *1024) bytes.
    */
    AIOT_DLOPT_BODY_BUFFER_MAX_LEN,
    AIOT_DLOPT_MAX
} aiot_download_option_t;

/**
 * @brief Queries update jobs from IoT Platform.
 *
 * @details
 * After the device goes online, @ref aiot_ota_recv_handler_t returns an OTA job if the SDK client receives the job from IoT Platform.
 * However, the OTA job may be ignored when the device is busy. You can call this API operation during off-peak hours to ask IoT Platform to re-publish the OTA job.
 * Then, when the SDK client receives the job, @ref aiot_ota_recv_handler_t returns the job again.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The request was sent.
 * @retval STATE_OTA_QUERY_FIRMWARE_HANDLE_IS_NULL The input handler was not initialized. Call @ref aiot_ota_init to initialize the handler.
 *
 */
int32_t aiot_ota_query_firmware(void *handle);

/**
 * @brief Creates an OTA session.
 *
 * @return void*
 * @retval Non-null string: indicates the OTA session handler.
 * @retval Null string: indicates that the initialization of the OTA session fails. The failure may be caused by lack of portfile configurations or a memory allocation error.
 *
 */
void   *aiot_ota_init();

/**
 * @brief Releases the OTA session handler.
 *
 * @param[in] handle The pointer that points to the OTA session handler.
 *
 * @return int32_t
 * @retval STATE_OTA_DEINIT_HANDLE_IS_NULL The handle parameter was not specified or the handler that the handle parameter points was null.
 * @retval STATE_SUCCESS The release succeeded.
 *
 */
int32_t aiot_ota_deinit(void **handle);

/**
 * @brief Reports the firmware version of a device. The device must not be a backend device of a gateway.
 *
 * @details
 *
 * The source firmware version is required by IoT Platform to provide the OTA service. In addition, IoT Platform considers an OTA update successful only if IoT Platform receives the new firmware version from the device.
 *
 * Therefore, to ensure that the OTA service is running as expected, the API operation must be called to report the current firmware version to IoT Platform each time the device is powered on.
 *
 * You can obtain a handler from @ref aiot_ota_init. The following script shows how to report the current firmware version 1.0.0.
 *
 * ```c
 * handle = aiot_ota_init();
 * ...
 * aiot_ota_report_version(handle, "1.0.0");
 * ```
 *
 * @param[in] handle The pointer that points to the OTA session handler.
 * @param[in] version The current firmware version.
 *
 * @return int32_t
 * @retval STATE_OTA_REPORT_HANDLE_IS_NULL The OTA session handler was not specified.
 * @retval STATE_OTA_REPORT_VERSION_IS_NULL The firmware version was not specified.
 * @retval STATE_OTA_REPORT_MQTT_HANDLE_IS_NULL The MQTT connection handler in the OTA session handler was not specified.
 * @retval STATE_OTA_REPORT_FAILED The report failed.
 * @retval STATE_SUCCESS The report succeeded.
 *
 */
int32_t aiot_ota_report_version(void *handle, char *version);

/**
 * @brief Reports the firmware version of a backend device of a gateway.
 *
 * @param[in] handle The OTA session handler.
 * @param[in] product_key The product key of the device.
 * @param[in] device_name The name of the device.
 * @param[in] version The firmware version of the device.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The report succeeded.
 * @retval STATE_OTA_REPORT_EXT_HANELD_IS_NULL The OTA session handler was not specified.
 * @retval STATE_OTA_REPORT_EXT_VERSION_NULL The firmware version was not specified.
 * @retval STATE_OTA_REPORT_EXT_PRODUCT_KEY_IS_NULL The product key of the device was not specified.
 * @retval STATE_OTA_REPORT_EXT_DEVICE_NAME_IS_NULL The name of the device was not specified.
 * @retval STATE_OTA_REPORT_EXT_MQTT_HANDLE_IS_NULL The MQTT connection handler in the OTA session handler was not specified.
 * @retval STATE_OTA_REPORT_FAILED The report failed.
 *
 */
int32_t aiot_ota_report_version_ext(void *handle, char *product_key, char *device_name, char *version);

/**
 * @brief Sets the parameters of the OTA session handler.
 *
 * @details
 *
 * The following list describes the common parameters of the OTA session.
 *
 * + `AIOT_OTAOPT_MQTT_HANDLE`: The handler of an MQTT connection that is returned by @ref aiot_mqtt_init. You can use this parameter to associate an MQTT connection with an OTA session.
 * + `AIOT_OTAOPT_RECV_HANDLER`: The function that processes OTA messages. This function will be called by @ref aiot_mqtt_recv when an OTA message is received.
 *
 * @param[in] handle The OTA session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_ota_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_ota_option_t.
 *
 * @return int32_t
 * @retval STATE_OTA_SETOPT_HANDLE_IS_NULL The OTA session handler is not specified.
 * @retval STATE_OTA_SETOPT_DATA_IS_NULL The data parameter is not specified.
 * @retval STATE_USER_INPUT_UNKNOWN_OPTION This option is not supported.
 * @retval STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_ota_setopt(void *handle, aiot_ota_option_t option, void *data);

/**
 * @brief Initializes a download session and retains the default settings for the session parameters.
 *
 * @return void*
 * @retval Non-null string: indicates the download session handler.
 * @retval Null string: indicates that the initialization of the session fails. The failure may be caused by lack of portfile configurations or a memory allocation error. A memory allocation error means that the memory is insufficient for a download session handler or an HTTP connection handler. 
 *
 */
void   *aiot_download_init();

/**
 * @brief Releases the download session handler.
 *
 * @param[in] handle The pointer that points to the download session handler.
 *
 * @return int32_t
 * @retval STATE_DOWNLOAD_DEINIT_HANDLE_IS_NULL The handle parameter was not specified or the handler that the handle parameter points was null.
 * @retval STATE_SUCCESS The release succeeded.
 *
 */
int32_t aiot_download_deinit(void **handle);

/**
 * @brief Downloads firmware to the local buffer by using the download session handler.
 *
 * @details
 *
 * After the firmware download URL is parsed from an OTA message, you can call this API operation to download the firmware over HTTP.
 *
 * The downloaded firmware is returned by using a callback function. You can set the callback function by calling @ref aiot_download_setopt.
 *
 * @param[in] handle The pointer that points to the download session handler.
 *
 * @return int32_t
 * @retval >STATE_SUCCESS The bytes that have been downloaded.
 * @retval STATE_DOWNLOAD_HTTPRSP_CODE_ERROR The URL was inaccessible. The returned HTTP status code was not 200 or 206.
 * @retval STATE_DOWNLOAD_FINISHED The entire firmware has been downloaded.
 * @retval STATE_DOWNLOAD_RANGE_FINISHED A range was downloaded.
 * @retval STATE_DOWNLOAD_HTTPRSP_HEADER_ERROR No Content-Length field existed in the response packet of the download URL.
 * @retval STATE_DOWNLOAD_RECV_HANDLE_IS_NULL The download session handler was not specified.
 * @retval STATE_DOWNLOAD_RENEWAL_REQUEST_SENT An HTTP range request for download was sent to the firmware server.
 * @retval For more error information, see @ref aiot_state_api.h.
 *
 */
int32_t aiot_download_recv(void *handle); /* Condition: network error | verification | end of file | full buffer */

/**
 * @brief Sets the parameters of the download session handler.
 *
 * @details
 *
 * The following list describes the common parameters of the download session.
 *
 * + `AIOT_DLOPT_RECV_HANDLER`: The callback function that sends the firmware from the buffer. The function is called when the buffer is full.
 * + `AIOT_DLOPT_NETWORK_CRED`: Specifies whether to use HTTP or HTTPS for download.
 * + `AIOT_DLOPT_BODY_BUFFER_MAX_LEN`: The size of the buffer. When the downloaded firmware data reaches the size, the callback function is called. The larger the size, the faster the download. However, the memory overheads also increase.
 *
 * @param[in] handle The download session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_download_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_download_option_t.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The configuration succeeded.
 * @retval STATE_DOWNLOAD_SETOPT_HANDLE_IS_NULL The download session handler was not specified.
 * @retval STATE_DOWNLOAD_SETOPT_DATA_IS_NULL The data parameter was not specified.
 * @retval STATE_DOWNLOAD_SETOPT_COPIED_DATA_IS_NULL Copy of task_desc failed.
 * @retval STATE_DOWNLOAD_SETOPT_MALLOC_SHA256_CTX_FAILED  Memory allocation for SHA-256 context failed.
 * @retval STATE_DOWNLOAD_SETOPT_MALLOC_MD5_CTX_FAILED     Memory allocation for MD5 context failed.
 * @retval For more error information, see @ref aiot_state_api.h.
 *
 */
int32_t aiot_download_setopt(void *handle, aiot_download_option_t option, void *data);

/**
 * @brief Reports the download progress in percentage or the error code.
 *
 * @details
 *
 * After the device starts to download the firmware, you can call this operation to report the progress or error.
 *
 * + If the firmware download is running as expected, the SDK client can calculate the download progress in percentage, round the progress to an integer, and then return the progress.
 * + If an error occurs during the firmware download or the firmware fails, the SDK client can report the error to IoT Platform. For the error codes, see @ref aiot_ota_protocol_errcode_t.
 * + The information reported by @ref aiot_download_report_progress will be displayed in the IoT Platform console. The information includes the OTA update progress or error.
 *
 * @param[in] handle The download session handler.
 * @param[in] percent The download progress in percentage or the error code.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The configuration succeeded.
 * @retval STATE_DOWNLOAD_REPORT_HANDLE_IS_NULL The handler was not specified for reporting.
 * @retval STATE_DOWNLOAD_REPORT_TASK_DESC_IS_NULL The task_desc was not specified. The product key or device name was not specified.
 * @retval For more error information, see @ref aiot_state_api.h.
 *
 */
int32_t aiot_download_report_progress(void *handle, int32_t percent);

/**
 * @brief Sends a GET request to download firmware from the URL.
 *
 * @details
 *
 * After the device obtains the firmware download URL from the callback function that processes the OTA message, you can call this API operation to download part of firmware.
 *
 * + You can set the AIOT_DLOPT_RANGE_START and AIOT_DLOPT_RANGE_END parameters to specify the range for each request.
 * + If you do not specify the range, the SDK client sends a request to download the entire firmware and notifies you each time the buffer is full. You can set the AIOT_DLOPT_BODY_BUFFER_MAX_LEN parameter to specify the buffer size.
 *
 * @param[in] handle The download session handler, including the firmware download URL.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The request was sent.
 * @retval STATE_DOWNLOAD_REQUEST_HANDLE_IS_NULL The handler was not specified in the GET request.
 * @retval STATE_DOWNLOAD_REQUEST_URL_IS_NULL The URL in task_desc was not specified.
 * @retval STATE_DOWNLOAD_SEND_REQUEST_FAILED The logic of sending packets of HTTP GET requests at the bottom layer was incorrect.
 * @retval STATE_DOWNLOAD_REQUEST_TASK_DESC_IS_NULL The task_desc field was not specified.
 * @retval For more error information, see @ref aiot_state_api.h.
 */
int32_t aiot_download_send_request(void *handle);

/**
 * @brief -0x0900 to -0x09FF are status codes of the OTA module, including STATE_DOWNLOAD_XXX.
 *
 */
#define STATE_OTA_BASE                                              (-0x0900)

/**
 * @brief The OTA firmware is downloaded. The verification and authentication succeeded.
 *
 */
#define STATE_OTA_DIGEST_MATCH                                      (-0x0901)

/**
 * @brief The download progress or firmware version failed to be reported to IoT Platform.
 *
 */
#define STATE_OTA_REPORT_FAILED                                     (-0x0902)

/**
 * @brief The OTA module failed to retrieve the firmware data.
 *
 */
#define STATE_DOWNLOAD_RECV_ERROR                                   (-0x0903)

/**
 * @brief The authentication or signature verification failed during the firmware download.
 *
 * @details
 *
 * The MD5 or SHA-256 value of the firmware was different from that expected by IoT Platform.
 *
 */
#define STATE_OTA_DIGEST_MISMATCH                                   (-0x0904)

/**
 * @brief The OTA module failed to parse the JSON packet that is received from IoT Platform.
 *
 * @details
 *
 * The key or value did not exist in the JSON packet that is received from IoT Platform.
 *
 */
#define STATE_OTA_PARSE_JSON_ERROR                                  (-0x0905)

/**
 * @brief The OTA module sent an HTTP packet but the request failed.
 *
 * @details
 *
 * The OTA module failed to send a GET request to the server that store the firmware.
 *
 */
#define STATE_DOWNLOAD_SEND_REQUEST_FAILED                          (-0x0906)

/**
 * @brief The end of the range for the request has reached and the download stops.
 *
 * @details
 *
 * If you have specified a range for each download request, the end of the range has reached. The error code is returned. If you need to continue the download, send another request for download that starts from the end.
 *
 */
#define STATE_DOWNLOAD_RANGE_FINISHED                               (-0x0907)

/**
 * @brief The OTA module failed to obtain the required memory when parsing the JSON packet.
 *
 */
#define STATE_OTA_PARSE_JSON_MALLOC_FAILED                          (-0x0908)

/**
 * @brief The OTA session handler was not specified. Therefore, the OTA session release failed.
 *
 */
#define STATE_OTA_DEINIT_HANDLE_IS_NULL                             (-0x0909)

/**
 * @brief The OTA session handler was not specified. Therefore, the OTA session configuration failed.
 *
 */
#define STATE_OTA_SETOPT_HANDLE_IS_NULL                             (-0x090A)

/**
 * @brief The configuration data was not specified. Therefore, the OTA session configuration failed.
 *
 */
#define STATE_OTA_SETOPT_DATA_IS_NULL                               (-0x090B)

/**
 * @brief The download session handler was not specified. Therefore, the download session release failed.
 *
 */
#define STATE_DOWNLOAD_DEINIT_HANDLE_IS_NULL                        (-0x090C)

/**
 * @brief The download session handler was not specified. Therefore, the download session configuration failed.
 *
 */
#define STATE_DOWNLOAD_SETOPT_HANDLE_IS_NULL                        (-0x090D)

/**
 * @brief The download session configuration data was not specified. Therefore, the download session configuration failed.
 *
 */
#define STATE_DOWNLOAD_SETOPT_DATA_IS_NULL                          (-0x090E)

/**
 * @brief An internal error occurred during the synchronization from the OTA session. Therefore, the download session configuration failed.
 *
 */
#define STATE_DOWNLOAD_SETOPT_COPIED_DATA_IS_NULL                   (-0x090F)

/**
 * @brief The OTA session handler was not specified. Therefore, the firmware version report from the directly connected device failed.
 *
 */
#define STATE_OTA_REPORT_HANDLE_IS_NULL                             (-0x0910)

/**
 * @brief The firmware version was not specified. Therefore, the firmware version report from the directly connected device failed.
 *
 */
#define STATE_OTA_REPORT_VERSION_IS_NULL                            (-0x0911)

/**
 * @brief The MQTT connection handler was not specified. Therefore, the firmware version report from the directly connected device failed.
 *
 */
#define STATE_OTA_REPORT_MQTT_HANDLE_IS_NULL                        (-0x0912)

/**
 * @brief The OTA session handler was not specified. Therefore, the device firmware version report from the gateway failed.
 *
 */
#define STATE_OTA_REPORT_EXT_HANELD_IS_NULL                         (-0x0913)

/**
 * @brief The firmware version was not specified. Therefore, the device firmware version report from the gateway failed.
 *
 */
#define STATE_OTA_REPORT_EXT_VERSION_NULL                           (-0x0914)

/**
 * @brief The product key was not specified. Therefore, the device firmware version report from the gateway failed.
 *
 */
#define STATE_OTA_REPORT_EXT_PRODUCT_KEY_IS_NULL                    (-0x0915)

/**
 * @brief The device name was not specified. Therefore, the device firmware version report from the gateway failed.
 *
 */
#define STATE_OTA_REPORT_EXT_DEVICE_NAME_IS_NULL                    (-0x0916)

/**
 * @brief The MQTT connection handler was not specified. Therefore, the device firmware version report from the gateway failed.
 *
 */
#define STATE_OTA_REPORT_EXT_MQTT_HANDLE_IS_NULL                    (-0x0917)

/**
 * @brief The download session handler was not specified. Therefore, the progress or error report failed.
 *
 */
#define STATE_DOWNLOAD_REPORT_HANDLE_IS_NULL                        (-0x0918)

/**
 * @brief The task_desc field was not specified. Therefore, the progress or error report failed.
 *
 */
#define STATE_DOWNLOAD_REPORT_TASK_DESC_IS_NULL                     (-0x0919)

/**
 * @brief When aiot_download_recv was called to receive firmware data, the callback function to process received data was not specified. Therefore, the reception failed.
 *
 */
#define STATE_DOWNLOAD_RECV_HANDLE_IS_NULL                          (-0x091A)

/**
 * @brief When aiot_download_send_request was called to send a firmware download request, the download session handler failed. Therefore, the send operation failed.
 *
 */
#define STATE_DOWNLOAD_REQUEST_HANDLE_IS_NULL                       (-0x091B)

/**
 * @brief When aiot_download_send_request was called to send a firmware download request, the task_desc field was not specified. Therefore, the send operation failed.
 *
 */
#define STATE_DOWNLOAD_REQUEST_TASK_DESC_IS_NULL                    (-0x091C)

/**
 * @brief When aiot_download_send_request was called to send a firmware download request, the URL in the task_desc field was not specified. Therefore, the send operation failed.
 *
 */
#define STATE_DOWNLOAD_REQUEST_URL_IS_NULL                          (-0x091D)

/**
 * @brief The digest method parsed from the OTA message that was received from the MQTT connection was neither MD5 nor SHA-256. The SDK client supports only MD5 and SHA-256.
 *
 */
#define STATE_OTA_UNKNOWN_DIGEST_METHOD                             (-0x091E)

/**
 * @brief The entire firmware has been downloaded. The downloaded firmware is of the expected size in bytes.
 *
 */
#define STATE_DOWNLOAD_FINISHED                                     (-0x091F)

/**
 * @brief The device received an HTTP packet as the response of the firmware server to the GET request. The HTTP status code of the response indicated an error and was neither 200 nor 206.
 *
 */
#define STATE_DOWNLOAD_HTTPRSP_CODE_ERROR                           (-0x0920)

/**
 * @brief The device received an HTTP packet as the response of the firmware server to the GET request. The header of the HTTP packet does not include the Content-Length field.
 *
 */
#define STATE_DOWNLOAD_HTTPRSP_HEADER_ERROR                         (-0x0921)

/**
 * @brief After the firmware download failed, the SDK re-sends a download request to the server for resuming the download.
 *
 */
#define STATE_DOWNLOAD_RENEWAL_REQUEST_SENT                         (-0x0922)

/**
 * @brief An error occurred in the SHA-256 verification and memory allocation.
 *
 */
#define STATE_DOWNLOAD_SETOPT_MALLOC_SHA256_CTX_FAILED              (-0x0923)

/**
 * @brief An error occurred in the MD5 verification and memory allocation.
 *
 */
#define STATE_DOWNLOAD_SETOPT_MALLOC_MD5_CTX_FAILED                 (-0x0924)

/**
 * @brief When the OTA module parsed the firmware download URL from the task_desc field, the module found that the HOST field was empty. Therefore, the parse operation failed.
 *
 */
#define STATE_OTA_PARSE_URL_HOST_IS_NULL                            (-0x0925)

/**
 * @brief When the OTA module parsed the firmware download URL from the task_desc field, the module found that the PATH field was empty. Therefore, the parse operation failed.
 *
 */
#define STATE_OTA_PARSE_URL_PATH_IS_NULL                            (-0x0926)

/**
 * @brief The total size of the firmware ranges that the OTA module has downloaded exceeded the expected size.
 *
 * @details The reason may be that the ranges overlap.
 *
 */
#define STATE_DOWNLOAD_FETCH_TOO_MANY                               (-0x0927)

/**
 * @brief The OTA session handler was not specified when the device attempted to query OTA jobs from IoT Platform.
 *
 * @details You must call the aiot_ota_init function to initialize an OTA session handler and pass the handler to aiot_ota_query_firmware.  
 * This error is returned if the OTA session handler is not initialized or the initialization failed.
 *
 */
#define STATE_OTA_QUERY_FIRMWARE_HANDLE_IS_NULL                     (-0x0928)

/**
 * @brief The length of the host sub-field of the url field in the OTA message that was received from IoT Platform exceeded the upper limit.
 *
 * @details This error is returned if the host field exceeds the upper limit (1,024 bytes). The host field is contained in the url field of the OTA message that is received from IoT Platform.
 *
 */
#define STATE_OTA_HOST_STRING_OVERFLOW                              (-0x0929)

/**
 * @brief The length of the path sub-field of the url field in the OTA packet that was received from IoT Platform exceeded the upper limit.
 *
 * @details This error is returned if the path field exceeds the upper limit (1,024 bytes). The path field is contained in the url field of the OTA message that is received from IoT Platform.
 *
 */
#define STATE_OTA_PATH_STRING_OVERFLOW                              (-0x092A)

#if defined(__cplusplus)
}
#endif

#endif  /* #ifndef __AIOT_OTA_API_H__ */

