/**
 * @file aiot_shadow_api.h
 * @brief The header file of the shadow module. This module updates, deletes, and obtains the device shadow.
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 *
 * The following procedure describes how to use the API of the shadow module:
 *
 * 1. Create an MQTT session.
 *
 * 2. Call aiot_shadow_init to create a shadow session and save the session handler.
 *
 * 3. Call aiot_shadow_setopt to set the AIOT_SHADOWOPT_MQTT_HANDLE option. This option specifies an MQTT connection handler and is required.
 *
 * 4. Call aiot_shadow_setopt to set the AIOT_SHADOWOPT_RECV_HANDLER and AIOT_SHADOWOPT_USERDATA options. AIOT_SHADOWOPT_RECV_HANDLER specifies the callback function that receives data and AIOT_SHADOWOPT_USERDATA specifies the user context.
 *
 * 5. Establish an MQTT connection before calling aiot_shadow_send.
 *
 * 6. Call aiot_shadow_send to send different types of messages to IoT Platform. The callback function processes the messages that are received from IoT Platform.
 *
 */
#ifndef __AIOT_SHADOW_API_H__
#define __AIOT_SHADOW_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief -0x1300 to -0x13FF are status codes of the shadow module.
 */
#define STATE_SHADOW_BASE                                           (-0x1300)

/**
 * @brief No MQTT connection handler was specified by calling @ref aiot_shadow_setopt.
 */
#define STATE_SHADOW_MQTT_HANDLE_IS_NULL                            (-0x1301)

/**
 * @brief The reported attribute in the struct of the @ref AIOT_SHADOWMSG_UPDATE or @ref AIOT_SHADOWMSG_DELETE_REPORTED message was null.
 *
 */
#define STATE_SHADOW_REPORTED_DATA_IS_NULL                          (-0x1302)

/**
 * @brief The device failed to parse the topic from the received message.
 */
#define STATE_SHADOW_INTERNAL_TOPIC_ERROR                           (-0x1303)

/**
 * @brief The log of receiving a message from IoT Platform.
 */
#define STATE_SHADOW_LOG_RECV                                       (-0x1304)

/**
 * @brief The log of failure in parsing a message that is received from IoT Platform.
 */
#define SATAE_SHADOW_LOG_PARSE_RECV_MSG_FAILED                      (-0x1305)


/**
 * @brief The options that you can set by using the option parameter of @ref aiot_shadow_setopt.
 *
 * @details The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_shadow_setopt.
 *
 */
typedef enum {
    /**
     * @brief The MQTT connection handler that is associated with the shadow module.
     *
     * @details
     *
     * The shadow module depends on the MQTT module. You must specify a valid MQTT connection handler to ensure that the shadow module can work as expected. Data type: (void *).
     */
    AIOT_SHADOWOPT_MQTT_HANDLE,

    /**
     * @brief The callback function that notifies your applications when packets are received from the network. The function is called when the shadow module receives a packet. Data type: (aiot_shadow_recv_handler_t).
     */
    AIOT_SHADOWOPT_RECV_HANDLER,

    /**
     * @brief The user context that needs to be cached by the SDK client. Data type: (void *).
     *
     * @details The SDK client returns the context pointer when the callback function that is specified by the AIOT_SHADOWOPT_RECV_HANDLER option is called.
     */
    AIOT_SHADOWOPT_USERDATA,

    /**
     * @brief The maximum number of configuration options. The number is not configurable.
     */
    AIOT_SHADOWOPT_MAX,
} aiot_shadow_option_t;

/**
 * @brief The type of the message that is sent by the shadow module.
 *
 * @details
 *
 * The enumeration includes all types of messages that the shadow module can send. The message struct varies depending on the message type.
 *
 */
typedef enum {
    /**
     * @brief Updates a reported value of the device shadow. For information about the message struct, see @ref aiot_shadow_msg_update_t.
     *
     */
    AIOT_SHADOWMSG_UPDATE,

    /**
     * @brief Deletes a desired value of the device shadow. For information about the message struct, see @ref aiot_shadow_msg_clean_desired_t.
     *
     */
    AIOT_SHADOWMSG_CLEAN_DESIRED,

    /**
     * @brief Obtains the device shadow. For information about the message struct, see @ref aiot_shadow_msg_get_t.
     *
     */
    AIOT_SHADOWMSG_GET,

    /**
     * @brief Deletes some or all of the reported values of the device shadow. For information about the message struct, see @ref aiot_shadow_msg_delete_reported_t.
     *
     */
    AIOT_SHADOWMSG_DELETE_REPORTED,

    /**
     * @brief The maximum number of messages. This is not a message type.
     */
    AIOT_SHADOWMSG_MAX,
} aiot_shadow_msg_type_t;

/**
 * @brief The struct of the message to <b>update a reported value of the device shadow</b>.
 */
typedef struct {
    /**
     * @brief The reported object string of the device shadow. <b>The string must end with '\0'</b>, for example, "{\"LightSwitch\": 1}".
     */
    char *reported;

    /**
     * @brief The destination version of the device shadow. <b>The destination version must be later than the current version.</b> If you specify -1, the device shadow is reset to 0.
     */
    int64_t version;
} aiot_shadow_msg_update_t;

/**
 * @brief The struct of the message to <b>delete a desired value of the device shadow</b>.
 */
typedef struct {
    /**
     * @brief The destination version of the device shadow. <b>The destination version must be later than the current version.</b>
     */
    int64_t version;
} aiot_shadow_msg_clean_desired_t;

/**
 * @brief The struct of the message to <b>obtain the device shadow</b>.
 */
typedef struct {
    /**
     * @brief A reserved field.
     */
    uint32_t resevered;
} aiot_shadow_msg_get_t;

/**
 * @brief The struct of the message to <b>delete some or all of the reported values of the device shadow</b>.
 */
typedef struct {
    /**
     * @brief The reported data that you want to delete. <b>The string must end with '\0'.</b>
     * If you need to delete all of the reported data, specify "\"null\""
     * If you need to delete part of the reported data, specify null for the properties that you want to delete, for example, "{\"LightSwitch\":\"null\"}".
     */
    char *reported;

    /**
     * @brief The destination version of the device shadow. <b>The destination version must be later than the current version.</b>
     */
    int64_t version;
} aiot_shadow_msg_delete_reported_t;

/**
 * @brief The struct of the message that is sent by the shadow module.
 */
typedef struct {
    /**
     * @brief The product key of the device. If you specify NULL, call aiot_shadow_setopt to specify the product key.
     * If the device is a backend device of a gateway, specify the product key of the backend device.
     */
    char *product_key;
    /**
     * @brief The name of the device. If you specify NULL, call aiot_shadow_setopt to specify the device name.
     * If the device is a backend device of a gateway, specify the product key of the backend device.
     */
    char *device_name;
    /**
     * @brief The type of the message. For more information, see @ref aiot_shadow_msg_type_t.
     */
    aiot_shadow_msg_type_t type;
    /**
     * @brief The union of the message data. The message struct varies with the message type.
     */
    union {
        aiot_shadow_msg_update_t            update;
        aiot_shadow_msg_clean_desired_t     clean_desired;
        aiot_shadow_msg_get_t               get;
        aiot_shadow_msg_delete_reported_t   delete_reporte;
    } data;
} aiot_shadow_msg_t;


/**
 * @brief The type of the message that is returned when the shadow module receives a packet from the network.
 */
typedef enum {
    /**
     * @brief After IoT Platform receives an @ref AIOT_SHADOWMSG_UPDATE, @ref AIOT_SHADOWMSG_CLEAN_DESIRED, or @ref AIOT_SHADOWMSG_DELETE_REPORTED message from the device, IoT Platform returns a response message.
     * For information about the message struct, see @ref aiot_shadow_recv_generic_reply_t.
     */
    AIOT_SHADOWRECV_GENERIC_REPLY,

    /**
     * @brief The shadow data that IoT Platform pushes to the device. The device can receive the data only when the device is online. For information about the message struct, see @ref aiot_shadow_recv_control_t.
     */
    AIOT_SHADOWRECV_CONTROL,

    /**
     * @brief The shadow data that the device obtains from IoT Platform. For information about the message struct, see @ref aiot_shadow_recv_get_reply_t.
     */
    AIOT_SHADOWRECV_GET_REPLY,
} aiot_shadow_recv_type_t;

/**
 * @brief After IoT Platform receives an @ref AIOT_SHADOWMSG_UPDATE, @ref AIOT_SHADOWMSG_CLEAN_DESIRED, or @ref AIOT_SHADOWMSG_DELETE_REPORTED message from the device, IoT Platform returns a response message.
 */
typedef struct {
    /**
     * @brief The pointer that points to the response data.
     */
    char *payload;

    /**
     * @brief The length of the response data.
     */
    uint32_t payload_len;

    /**
     * @brief The operation is successful if the response includes <b>success</b>. If the operation fails, the response includes the error information.
     */
    char *status;

    /**
     * @brief The timestamp of the response packet.
     */
    uint64_t timestamp;
} aiot_shadow_recv_generic_reply_t;

/**
 * @brief The message that your application calls UpdateDeviceShadow to push from IoT Platform to the device when the device is online.
 */
typedef struct {
    /**
     * @brief The pointer that points to the shadow data.
     */
    char *payload;

    /**
     * @brief The length of the shadow data.
     */
    uint32_t payload_len;

    /**
     * @brief The version of the device shadow.
     */
    uint64_t version;
} aiot_shadow_recv_control_t;

/**
 * @brief After IoT Platform receives an @ref AIOT_SHADOWMSG_GET message from the device, IoT Platform returns a response message.
 */
typedef struct {
    /**
     * @brief The pointer that points to the shadow data.
     */
    char *payload;

    /**
     * @brief The length of the shadow data.
     */
    uint32_t payload_len;

    /**
     * @brief The version of the device shadow.
     */
    uint64_t version;
} aiot_shadow_recv_get_reply_t;

/**
 * @brief The content of the message that is returned when the shadow module receives a packet from the network.
 */
typedef struct {
    /**
     * @brief The product key of the device.
     */
    char *product_key;
    /**
     * @brief The name of the device.
     */
    char *device_name;
    /**
     * @brief The type of the packet. For more information, see @ref aiot_shadow_recv_type_t.
     */
    aiot_shadow_recv_type_t  type;
    /**
     * @brief The union of the message data. The message struct varies with the message type.
     */
    union {
        aiot_shadow_recv_generic_reply_t generic_reply;
        aiot_shadow_recv_control_t control;
        aiot_shadow_recv_get_reply_t get_reply;
    } data;
} aiot_shadow_recv_t;


/**
 * @brief The function that notifies your applications when the shadow module receives a packet from the network.
 *
 * @param[in] handle The shadow session handler.
 * @param[in] recv The struct of the message that the shadow module receives. The struct stores the content of the shadow message.
 * @param[in] userdata The pointer that points to the user context cached by the SDK. You can set the @ref AIOT_SHADOWOPT_USERDATA parameter of @ref aiot_shadow_setopt to specify the context that needs to be cached.
 *
 * @return void
 */
typedef void (* aiot_shadow_recv_handler_t)(void *handle,
        const aiot_shadow_recv_t *recv, void *userdata);

/**
 * @brief Creates a shadow session and retains the default settings of the session parameters.
 *
 * @return void *
 * @retval Non-null string: indicates the shadow session handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void *aiot_shadow_init(void);

/**
 * @brief Configures the shadow session.
 *
 * @param[in] handle The shadow session handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_shadow_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_shadow_option_t.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The configuration succeeded.
 * @retval STATE_USER_INPUT_NULL_POINTER The handle or data input parameter was null.
 * @retval STATE_USER_INPUT_OUT_RANGE The enumeration value of the option input parameter is greater than or equal to AIOT_SHADOWOPT_MAX.
 * @retval others For more information, see @ref aiot_state_api.h.
 *
 */
int32_t aiot_shadow_setopt(void *handle, aiot_shadow_option_t option, void *data);

/**
 * @brief Sends a shadow request to IoT Platform.
 *
 * @param[in] handle The shadow session handler.
 * @param[in] msg    The struct of the message. The struct includes the information such as the <i>product key</i> and <i>device name</i> of the device; the message type, the message data. For more information, see @ref aiot_dm_msg_t.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The request was sent.
 * @retval STATE_USER_INPUT_NULL_POINTER The <i>handle</i> or <i>msg</i> input parameter was null.
 * @retval STATE_USER_INPUT_OUT_RANGE The <i>type</i> attribute in the <i>msg</i> struct was greater than or equal to AIOT_SHADOWMSG_MAX.
 * @retval STATE_SYS_DEPEND_MALLOC_FAILED Memory allocation failed.
 * @retval STATE_SHADOW_MQTT_HANDLE_IS_NULL No MQTT connection handler was specified by calling @ref aiot_shadow_setopt.
 * @retval others For more error codes, see @ref aiot_state_api.h or @ref STATE_SHADOW_BASE.
 *
 */
int32_t aiot_shadow_send(void *handle, aiot_shadow_msg_t *msg);

/**
 * @brief Ends the shadow session and releases allocated resources.
 *
 * @param[in] handle The pointer that points to the shadow session handler.
 *
 * @return int32_t
 * @retval STATE_SUCCESS The operation succeeded.
 * @retval <STATE_SUCCESS The operation failed.
 *
 */
int32_t aiot_shadow_deinit(void **handle);


#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_SHADOW_API_H__ */

