/**
 * @file aiot_subdev_api.h
 * @brief subdev The header file of the subdev module. This module allows you to manage sub-devices
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 */
#ifndef __AIOT_SUBDEV_API_H__
#define __AIOT_SUBDEV_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief The codes from -0x0C00 to -0x0CFF represent the status codes in the subdev module of the SDK.
 */
#define STATE_SUBDEV_BASE                                             (-0x0C00)

/**
 * @brief The type of the notification that is sent when the subdev module receives packets from networks.
 */
typedef enum {
    AIOT_SUBDEVRECV_TOPO_ADD_REPLY,
    AIOT_SUBDEVRECV_TOPO_DELETE_REPLY,
    AIOT_SUBDEVRECV_TOPO_GET_REPLY,
    AIOT_SUBDEVRECV_BATCH_LOGIN_REPLY,
    AIOT_SUBDEVRECV_BATCH_LOGOUT_REPLY,
    AIOT_SUBDEVRECV_SUB_REGISTER_REPLY,
    AIOT_SUBDEVRECV_PRODUCT_REGISTER_REPLY,
    AIOT_SUBDEVRECV_TOPO_CHANGE_NOTIFY
} aiot_subdev_recv_type_t;

typedef struct {
    uint32_t     msg_id;
    uint32_t     code;
    char*        data;
    char*        message;
    char*        product_key;
    char*        device_name;
} aiot_subdev_generic_reply_t;

typedef struct {
    uint32_t     msg_id;
    char*        params;
    char*        product_key;
    char*        device_name;
} aiot_subdev_generic_notify_t;

/**
 * @brief The content of the notification that is sent when the subdev module receives packets from networks.
 */
typedef struct {
    /**
     * @brief The notification type varies based on the notification content. For more information, see the description of the @ref aiot_subdev_recv_type_t function.
     */
    aiot_subdev_recv_type_t  type;
    union {
        /**
         * @brief After a device sends a request to IoT Platform, the response is received from IoT Platform.
         */
        aiot_subdev_generic_reply_t  generic_reply;

        /**
         * @brief The notification that is received from IoT Platform.
         */
        aiot_subdev_generic_notify_t generic_notify;
    } data;
} aiot_subdev_recv_t;

/**
 * @brief The callback function that is used to send notifications when the subdev module receives packets from networks.
 *
 * @param[in] handle subdev The handle of the subdev session.
 * @param[in] packet subdev The struct of subdev packets. The struct is used to store the content of a subdev packet that is received.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_subdev_recv_handler_t)(void *handle,
        const aiot_subdev_recv_t *packet, void *user_data);


/* TODO: Optional. Enumerate all possible status changes in the subdev module. For example, an MQTT connection is re-established, the module receives OTA notifications, the token of a CoAP connection expires, and two empty lines exist between two sections.*/
/* TODO: Enter remarks for doxygen.*/
/* TODO: To improve readability, an abbreviation is used when the length of a module name is longer than 6 characters, for example, DYNREG_HTTP_EVT -> DRHTTPEVT.*/

/**
 * @brief The type of events for which notifications are sent when status changes that must be followed occur in the subdev module.
 */
typedef enum {
    /**
     * @brief The response packet is invalid.
     */
    AIOT_SUBDEVEVT_INVALID_RESPONSE,
    /**
     * @brief The id field of the response packet is invalid.
     */
    AIOT_SUBDEVEVT_INVALID_ID,
    /**
     * @brief The code field of the response packet is invalid.
     */
    AIOT_SUBDEVEVT_INVALID_CODE
} aiot_subdev_event_type_t;

/**
 * @brief The content of events for which notifications are sent when status changes that must be followed occur in the subdev module.
 */
typedef struct {
    /**
     * @brief The event type varies based on the event content. For more information, see the description of the @ref aiot_subdev_event_type_t function.
     */
    aiot_subdev_event_type_t  type;
} aiot_subdev_event_t;

/**
 * @brief The event callback function that is invoked to send notifications when status changes that must be followed occur in the subdev module.
 *
 * @param[in] handle, subdev The handle of the subdev session.
 * @param[in] event, The content of the events that are triggered for the subdev module.
 * @param[in] userdata, The user context.
 *
 * @return void
 */
typedef void (*aiot_subdev_event_handler_t)(void *handle,
        const aiot_subdev_event_t *event, void *userdata);

/**
 * @brief 
 * 
 */
typedef struct {
    char *product_key;
    char *device_name;
    char *device_secret;
    char *product_secret;
} aiot_subdev_dev_t;

/**
 * @brief The valid values of the option parameter for the @ref aiot_subdev_setopt function.
 *
 * @details In the following examples, the data types of the data parameter in the @ref aiot_subdev_setopt function are described.
 *
 * 1. The following code shows how to configure @ref AIOT_SUBDEVOPT_MQTT_HANDLE when the data type of the data parameter is char *.
 *
 *    void *mqtt_handle = aiot_mqtt_init();
 *    aiot_subdev_setopt(subdev_handle, AIOT_SUBDEVOPT_MQTT_HANDLE, mqtt_handle);
 */
typedef enum {
    /**
     * @brief The MQTT handler that is required by the subdev session. Before you configure an MQTT handler, make sure that an MQTT connection is established.
     * 
     * @details
     * 
     * Data type: (void *)
     */
    AIOT_SUBDEVOPT_MQTT_HANDLE,

    /**
     * @brief Configures a callback. The callback is invoked to send notifications when the SDK receives network packets. The data type of the callback is aiot_subdev_recv_handler_t.
     * 
     * @details
     * 
     * Data type: aiot_subdev_recv_handler_t
     */
    AIOT_SUBDEVOPT_RECV_HANDLER,

    /**
     * @brief Configures a callback. The callback is invoked to send notifications when status changes occur in the SDK. The data type of the callback is aiot_subdev_event_handler_t.
     * 
     * @details
     * 
     * Data type: aiot_subdev_event_handler_t
     */
    AIOT_SUBDEVOPT_EVENT_HANDLER,

    /**
     * @brief You must obtain the context that is temporarily stored by the SDK. The data type of the context is void *.
     *
     * @details The pointer to the context is returned by the SDK to users when the callback that is configured by using AIOT_SUBDEVOPT_RECV_HANDLER and AIOT_SUBDEVOPT_EVENT_HANDLER is invoked.
     * 
     * Data type: void *
     */
    AIOT_SUBDEVOPT_USERDATA,
    AIOT_SUBDEVOPT_MAX
} aiot_subdev_option_t;

/**
 * @brief Creates a subdev session instance and sets the parameters to the default values.
 *
 * @return void *
 * @retval If a non-null value is returned, the pointer points to the handle of the subdev instance.
 * @retval NULL   If a null value is returned, the initialization failed. In most cases, the failure occurs because the system fails to allocate memory resources to the instance.
 *
 */
void *aiot_subdev_init(void);

/**
 * @brief Configures the subdev session.
 *
 * @param[in] handle subdev The handle of the subdev session.
 * @param[in] option The parameters. For more information, see the description of the @ref aiot_subdev_option_t function.
 * @param[in] data   The values of the parameters. For more information, see the description of the @ref aiot_subdev_option_t function.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The parameters failed to be configured.
 * @retval >=STATE_SUCCESS The parameters are configured.
 *
 */
int32_t aiot_subdev_setopt(void *handle, aiot_subdev_option_t option, void *data);

/**
 * @brief Ends the subdev session, destroys the instance, and releases resources.
 *
 * @param[in] handle The pointer to the handle of the subdev session.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS  The call failed.
 * @retval >=STATE_SUCCESS The call is successful.
 *
 */
int32_t aiot_subdev_deinit(void **handle);

/**
 * @brief Sends a request to create topological relationships between sub-devices and a gateway to IoT Platform.
 *
 * @param handle The handle of the subdev session.
 * @param dev The sub-device array whose sub-devices for which you want to create topological relationships.
 * @param dev_num The number of sub-devices in the sub-device array.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is sent.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_topo_add(void *handle, aiot_subdev_dev_t dev[], uint32_t dev_num);

/**
 * @brief Sends a request to delete topological relationships between sub-devices and a gateway to IoT Platform.
 *
 * @param handle The handle of the subdev session.
 * @param dev The sub-device array whose sub-devices from which you want to delete topologicial relationships.
 * @param dev_num The number of sub-devices in the sub-device array.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is sent.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_topo_delete(void *handle, aiot_subdev_dev_t dev[], uint32_t dev_num);

/**
 * @brief Sends a request to query topological relationships between sub-devices and a gateway to IoT Platform.
 *
 * @param handle subdev The handle of the subdev session.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is sent.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_topo_get(void *handle);

/**
 * @brief Sends a request to connect sub-devices to IoT Platform.
 *
 * @param handle The handle of the subdev session.
 * @param dev The sub-device array whose sub-devices that you want to connect to IoT Platform.
 * @param dev_num The number of sub-devices in the sub-device array.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is sent.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_batch_login(void *handle, aiot_subdev_dev_t dev[], uint32_t dev_num);

/**
 * @brief Sends a request to disconnect sub-devices from IoT Platform.
 *
 * @param handle The handle of the subdev session.
 * @param dev The sub-device array whose sub-devices that you want to disconnect from IoT Platform.
 * @param dev_num The number of sub-devices in the sub-device array.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is successful.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_batch_logout(void *handle, aiot_subdev_dev_t dev[], uint32_t dev_num);

/**
 * @brief Sends a request to dynamically register sub-devices to IoT Platform.
 *
 * @param handle The handle of the subdev session.
 * @param dev The sub-device arrary whose sub-devices that you want to dynamically register.
 * @param dev_num The number of sub-devices in the sub-device array.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is sent.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_sub_register(void *handle, aiot_subdev_dev_t dev[], uint32_t dev_num);

/**
 * @brief Sends a request to dynamically register sub-devices to IoT Platform. You can register sub-devices that are connected to different gateways.
 *
 * @param handle The handle of the subdev session.
 * @param dev The sub-device array whose sub-devices you want to dynamically register.
 * @param dev_num The number of sub-devices in the sub-device array.
 * 
 * @return int32_t
 * @retval <STATE_SUCCESS  The request failed to be sent.
 * @retval >=STATE_SUCCESS The request is sent.
 * 
 * @note
 * 
 * @ref aiot_subdev_dev_t : You must specify a product key and a device name in the structure.
 */
int32_t aiot_subdev_send_product_register(void *handle, aiot_subdev_dev_t dev[], uint32_t dev_num);

#if defined(__cplusplus)
}
#endif

#endif  /* __AIOT_SUBDEV_API_H__ */

