/**
 * @file aiot_task_api.h
 * @brief The header file of the task module. This module allows you to manage tasks.
 * @date 2020-11-25
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 *
 *
 */

#ifndef __AIOT_TASK_API_H__
#define __AIOT_TASK_API_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief The codes from -0x0B00 to -0x0BFF indicate the status codes of the task module in the SDK.
 *
 */
#define STATE_TASK_BASE                                             (-0x0B00)

/**
 * @brief This code is returned if the handle of a task session is empty when the system destroys the instance of the task session. Then, the system stops the operation.
 *
 */
#define STATE_TASK_DEINIT_HANDLE_IS_NULL                            (-0x0B01)

/**
 * @brief This code is returned if the handle of a task session is empty when the system configures the instance of the task session. Then, the system stops the operation.
 *
 */
#define STATE_TASK_SETOPT_HANDLE_IS_NULL                            (-0x0B02)

/**
 * @brief This code is returned when the system receives notify downstreaming messages from the server.
 */
#define STATE_TASK_RECV_NOTIFY                                      (-0x0B03)

/**
 * @brief This code is returned when the DNS server fails to send downstreaming JSON messages over MQTT.
 */
#define STATE_TASK_PARSE_NOTIFY_FAILED                              (-0x0B04)

/**
 * @brief This code is returned when the system fails to parse JSON messages. The failure occurs because the request that is sent to obtain the required memory is rejected.
 */
#define STATE_TASK_PARSE_JSON_MALLOC_FAILED                         (-0x0B05)

/**
 * @brief This code is returned when the system receives downstreaming messages from the server.
 */
#define STATE_TASK_PARSE_JSON_ERROR                                 (-0x0B06)

/**
 * @brief This code is returned when the task ID that you want to query is invalid.
 */
#define STATE_TASK_QUERY_TASK_ID_IS_NULL                            (-0x0B07)

/**
 * @brief This code is returned when the system receives get list reply messages from the server.
 */
#define STATE_TASK_RECV_GET_LIST_REPLY                              (-0x0B08)

/**
 * @brief This code is returned if the handle of a task session is empty when the system configures the instance of the task session. Then, the system stops the operation.
 *
 */
#define STATE_TASK_SETOPT_DATA_IS_NULL                              (-0x0B09)

/**
 * @brief This code is returned if the status of a task is invalid when the system specifies the description of the task.
 *
 */
#define STATE_TASK_UPDATE_STATUS_INVALID                            (-0x0B0A)

/**
 * @brief The valid values of the option parameter for the aiot_task_setopt function.
 */

/**
 * @brief This code is returned if the status_details parameter of a task is not set to NULL or a JSON string object when the system updates the task.
 *
 */
#define STATE_TASK_UPDATE_STATUS_DETAILS_INVALID                     (-0x0B0B)

typedef enum {
    /**
     * @brief Configures the handle of an MQTT session.
     *
     * @details
     *
     * During an OTA update, MQTT tunnels are used to submit the hardware version, progress, and error codes of each device to IoT Platform.
     *
     * Data type: void *
     */
    AIOT_TASKOPT_MQTT_HANDLE,

    /**
     * @brief Configures a user callback function that is used to process task messages.
     *
     * @details
     *
     * The function is used to process data that is returned or received from IoT Platform.
     *
     * Data type: void *
     */
    AIOT_TASKOPT_RECV_HANDLER,

    /**
     * @brief The context that you want to temporarily store by using the SDK.
     *
     * @details
     *
     * This context point is sent by the SDK when the callback that is configured by AIOT_TASKOPT_RECV_HANDLER is invoked.
     *
     * Data type: void *
     */
    AIOT_TASKOPT_USERDATA,
    AIOT_TASKOPT_MAX
} aiot_task_option_t;

/**
 * @brief The status of the task.
 */
typedef enum {
    AIOT_TASK_STATUS_QUEUED,             /* The status that is configured on the server side: The task is in the queue and is not delivered. */
    AIOT_TASK_STATUS_SENT,               /* The status that is configured on the server side: The task is delivered. */
    AIOT_TASK_STATUS_IN_PROGRESS,        /* The status that is configured on the device side: The task is running.*/
    AIOT_TASK_STATUS_SUCCEEDED,          /* The status that is configured on the device side: The task is completed. */
    AIOT_TASK_STATUS_FAILED,             /* The status that is configured on the device side: The task failed. */
    AIOT_TASK_STATUS_REJECTED,           /* The status that is configured on the device side: The device rejects to run the task. */
    AIOT_TASK_STATUS_CANCELLED,          /* The status that is configured on the server side: The task is canceled by the server.*/
    AIOT_TASK_STATUS_REMOVED,            /* The status that is configured on the server side: The task is deleted from the server. */
    AIOT_TASK_STATUS_TIMED_OUT,          /* The status that is configured on the server side: The task timed out. */
    AIOT_TASK_STATUS_NOT_FOUND           /* The status that is configured on the server side: No information about the task is returned. */
} aiot_task_status_t;

/**
 * @brief The data structure about downstreaming data.
 */
typedef enum {
    AIOT_TASKRECV_NOTIFY,               /* The data corresponds to the /sys/{productKey}/{deviceName}/thing/job/notify downstreaming topic. The data includes the details of a task that is delivered by IoT Platform. */
    AIOT_TASKRECV_GET_DETAIL_REPLY,     /* The data corresponds to the /sys/{productKey}/{deviceName}/thing/job/get_reply downstreaming topic. The data includes the details of a single task or the description of a task list.*/
    AIOT_TASKRECV_GET_LIST_REPLY,       /* This data corresponds to the /sys/{productKey}/{deviceName}/thing/job/get_reply downstreaming topic. The data includes the details of a single task or the description of a task list. */
    AIOT_TASKRECV_UPDATE_REPLY          /* The data corresponds to the /sys/{productKey}/{deviceName}/thing/job/update_reply downstreaming topic. The data includes the result that indicates whether the task is successful.*/
} aiot_task_recv_type_t;

/**
 * @brief The data structure of a task.
 */
typedef struct {
    char *task_id;                      /* The task ID. */
    aiot_task_status_t status;          /* The task status. */
    char *job_document;                 /* The rules of the task. */
    char *sign_method;                  /* The signature method. */
    char *sign;                         /* The signature of a file. */
    char *document_file_url;            /* The URL from which you can download the task file. */
    char *status_details;               /* The custom status. The value is a JSON object, for example, "{\"key\": \"value\"", strlen("\"key\": \"value\"}". The status is passed through to IoT Platform.*/
    uint8_t progress;                   /* The progress of the task. Valid values: 0 to 100. */
    void *handle;                       /* The handle of the task. */
} task_desc_t;

/**
 * @brief The description of each task when the system pulls a task list from IoT Platform.
 */
typedef struct {
    char *task_id;                      /* The task ID. */
    aiot_task_status_t status;          /* The task status. */
} task_summary_t;

/**
 * @brief The data that is returned when the system pulls a task list from IoT Platform.
 */
typedef struct {
    uint32_t number;                    /* The number of tasks in a task list that is pulled from IoT Platform. */
    task_summary_t *tasks;              /* The pointer to the task array. */
} task_get_list_reply_t;

/**
 * @brief The data that is returned when the system pulls the details of a task from IoT Platform.
 */
typedef struct {
    uint32_t code;                      /* The code that is returned. */
    task_desc_t task;                   /* The details of the task. */
} task_get_detail_reply_t;

/**
 * @brief The data that is returned when the system updates the status of a task to IoT Platform.
 */
typedef struct {
    uint32_t code;                      /* The code that is returned. */
    char *task_id;                      /* The task ID that is returned after the task is updated. */
    aiot_task_status_t status;          /* The task status that is returned after the task is updated. */
} task_update_reply_t;

/**
 * @brief The data that is returned when IoT Platform delivers or updates a task.
 */
typedef struct {
    aiot_task_recv_type_t type;                     /* The type of the returned data. */
    union {
        task_desc_t notify;                         /* The data of the tasks that are delivered by IoT Platform. */
        task_get_list_reply_t get_list_reply;       /* The data that is returned when the system queries tasks. */
        task_get_detail_reply_t get_detail_reply;   /* The data that is returned when the system queries the details of a task. */
        task_update_reply_t update_reply;           /* The data that is returned when the system updates the status of a task. */
    } data;
} aiot_task_recv_t;

/**
 * @brief The callback function that is used to process MQTT downstreaming messages when a device receives the messages of a task
 *
 * @param[in] handle task The handle of the task instance.
 * @param[in] recv The messages that are received from IoT Platform.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (* aiot_task_recv_handler_t)(void *handle, const aiot_task_recv_t *recv, void *userdata);

/**
 * @brief Creates a task instance.
 *
 * @return void*
 * @retval a non-null value means a handler of the task instance has been created
 * @retval NULL If a null value is returned, the initialization failed. In most cases, the failure occurs because the portfile parameter is not configured or the system fails to allocate memory resources to the instance.
 *
 */
void   *aiot_task_init(void);

/**
 * @brief Destroys the handle of the task instance.
 *
 * @param[in] handle The pointer to the handle of the task instance.
 *
 * @return int32_t
 * @retval STATE_USER_INPUT_NULL_POINTER indicates that the handle or the address to which the handle points is empty.
 * @retval STATE_SUCCESS means that the call is successful.
 *
 */
int32_t aiot_task_deinit(void **handle);

/**
 * @brief Configures the parameters of the task handle.
 *
 * @details
 *
 * Configures a task session. The following common parameters are described:
 *
 * @param[in] handle The handle of the task.
 * @param[in] option The valid values of the option parameter. For more information, see the description of the @ref aiot_task_option_t function.
 * @param[in] data   The valid values of the data parameter. For more information, see the description of the @ref aiot_task_option_t function.
 *
 * @return int32_t
 * @retval STATE_TASK_SETOPT_HANDLE_IS_NULL means that the handle of the task is empty.
 * @retval STATE_TASK_SETOPT_DATA_IS_NULL means the data parameter is empty.
 * @retval STATE_USER_INPUT_UNKNOWN_OPTION means the option parameter is unavailable.
 * @retval STATE_SUCCESS means that the parameters are configured.
 *
 */
int32_t aiot_task_setopt(void *handle, aiot_task_option_t option, void *data);

/**
 * @brief Obtains tasks from IoT Platform.
 *
 * @details
 *
 * Obtains tasks from IoT Platform.
 *
 * @param[in] handle The handle of the task.
 *
 * @return int32_t
 * @retval STATE_TASK_SETOPT_DATA_IS_NULL means the handle parameter is empty.
 * @retval STATE_SUCCESS                  If STATE_SUCCESS is returned, the request is sent.
 */
int32_t aiot_task_get_task_list(void *handle);

/* Sends messages to the /sys/{productKey}/{deviceName}/thing/job/get. topic. If the user_task_id parameter is not empty, the payload of an upstreaming message is "taskId": user_task_id. The details of the task are returned. */
/* If the user_task_id parameter is empty, the payload of an upstreaming message is "taskId": "$next". The first task that is not in the desired state in the task queue is returned from IoT Platform. The task is in the QUEUED, SENT, or IN_PROGRESS state. */

/**
 * @brief Obtains the details of the task from IoT Platform.
 *
 * @details
 *
 * Obtains the details of the task from IoT Platform.
 *
 * @param[in] handle The handle of the task.
 * @param[in] user_task_id task parameter is set to a task ID or $next.
 *
 * @return int32_t
 * @retval STATE_TASK_SETOPT_DATA_IS_NULL means the user_task_id parameter or the handle parameter is empty.
 * @retval STATE_SUCCESS means the request is sent.
 *
 */
int32_t aiot_task_get_task_detail(void *handle, char *user_task_id);

/**
 * @brief Updates the status of the task to IoT Platform.
 *
 * @details
 *
 * Updates the status of the task to IoT Platform.
 *
 * @param[in] handle The handle of the task.
 * @param[in] task The information about the task.
 *
 * @return int32_t
 * @retval ISTATE_TASK_SETOPT_DATA_IS_NULL means the task parameter or the handle parameter is empty.
 * @retval STATE_SUCCESS means the update is successful.
 *
 */
int32_t aiot_task_update(void *handle, task_desc_t *task);
#if defined(__cplusplus)
}
#endif

#endif  /* #ifndef __AIOT_TASK_API_H__ */


