/**
 * @file aiot_http_api.h
 * @brief The header file of the HTTP module. This module reports data to IoT Platform over HTTP.
 * @date 2019-12-27
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 */

#ifndef _AIOT_HTTP_API_H_
#define _AIOT_HTTP_API_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include "core_stdinc.h"

/**
 * @brief The service-level status code that is returned from IoT Platform.
 *
 * @details
 *
 * The status code is parsed from the JSON packet that carries the response from IoT Platform.
 */
typedef enum {
    /**
     * @brief 0: IoT Platform received the message that carries the reported data.
     */
    AIOT_HTTP_RSPCODE_SUCCESS                   = 0,
    /**
     * @brief 10000: IoT Platform returned an unknown error.
     */
    AIOT_HTTP_RSPCODE_COMMON_ERROR              = 10000,
    /**
     * @brief 10001: The request parameters were invalid.
     */
    AIOT_HTTP_RSPCODE_PARAM_ERROR               = 10001,
    /**
     * @brief 20001: The token was expired. Call @ref aiot_http_auth to obtain a new token.
     */
    AIOT_HTTP_RSPCODE_TOKEN_EXPIRED             = 20001,
    /**
     * @brief 20002: The request headers failed to include a token that can be used to authenticate the device. Call @ref aiot_http_auth to obtain a new token.
     */
    AIOT_HTTP_RSPCODE_TOKEN_NULL                = 20002,
    /**
     * @brief 20003: The token was invalid. Call @ref aiot_http_auth to obtain a new token.
     */
    AIOT_HTTP_RSPCODE_TOKEN_CHECK_ERROR         = 20003,
    /**
     * @brief 30001: The message failed to be reported.
     */
    AIOT_HTTP_RSPCODE_PUBLISH_MESSAGE_ERROR     = 30001,
    /**
     * @brief 40000: Too many messages have been reported. The bandwidth is throttled.
     */
    AIOT_HTTP_RSPCODE_REQUEST_TOO_MANY          = 40000,
} aiot_http_response_code_t;

/**
 * @brief The option parameter of @ref aiot_http_setopt. The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_mqtt_setopt.
 *
 */
typedef enum {
    /**
     * @brief The domain name or IP address of the HTTP server.
     *
     * @details
     *
     * The endpoint of IoT Platform HTTP servers. Port 80 is used for TCP connections. Port 443 is used for TLS connections.
     *
     * |              Domain name              |       Region        |  Port  |
     * |---------------------------------------|---------------------|--------|
     * | iot-as-http.cn-shanghai.aliyuncs.com  |  China (Shanghai)   |  443   |
     *
     * Data type: (char *).
     */
    AIOT_HTTPOPT_HOST,
    /**
     * @brief The number of the port that is used by the HTTP server.
     *
     * @details
     *
     * For connections to IoT Platform:
     *
     * 1. If TCP is used, set the port number to 80.
     *
     * 2. If TLS is used, set the port number to 443.
     *
     * Data type: (uint16_t *).
     */
    AIOT_HTTPOPT_PORT,
    /**
     * @brief The security credential that is used for HTTP connection establishment.
     *
     * @details
     *
     * The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * 1. If this parameter is not specified, an MQTT connection is established over TCP.
     *
     * 2. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_NONE, an HTTP connection is established over TCP.
     *
     * 3. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA, an HTTP connection is established over TLS.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_HTTPOPT_NETWORK_CRED,
    /**
     * @brief The period of time to wait before the request for HTTP connection establishment times out.
     *
     * @details
     *
     * This parameter specifies the period of time to wait before the request for socket connection establishment times out.
     *
     * Data type: (uint32_t *). Default value: (5 *1000) ms.
     *
     */
    AIOT_HTTPOPT_CONNECT_TIMEOUT_MS,
    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when data is sent over HTTP.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_HTTPOPT_SEND_TIMEOUT_MS,
    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when data is received over HTTP.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_HTTPOPT_RECV_TIMEOUT_MS,
    /**
     * @brief The timeout period for other API operations to complete when you end an HTTP connection.
     *
     * @details
     *
     * If you call @ref aiot_http_deinit to end an HTTP connection and then call another aiot_http_xxx API operation, the operation returns the STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_http_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_HTTPOPT_DEINIT_TIMEOUT_MS,
    /**
     * @brief The maximum length of each line in the header of the HTTP packet that is received from IoT Platform.
     *
     * @details
     *
     * If you set this parameter to an excessively small value, @ref aiot_http_recv returns the @ref STATE_HTTP_HEADER_BUFFER_TOO_SHORT error.
     *
     * Data type: (uint32_t *). Default value: 128.
     */
    AIOT_HTTPOPT_HEADER_BUFFER_LEN,
    /**
     * @brief The maximum length of the body in the HTTP packet that is received from IoT Platform. The body is returned by @ref aiot_http_recv_handler_t.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: 128.
     */
    AIOT_HTTPOPT_BODY_BUFFER_LEN,
    /**
     * @brief The callback function that is called when an HTTP event is triggered.
     *
     * @details
     *
     * Data type: (aiot_http_event_handler_t)
     */
    AIOT_HTTPOPT_EVENT_HANDLER,

    /* The preceding parameters are used together with CORE_HTTPOPT_XXX. */

    /**
     * @brief The user context that needs to be cached by the SDK.
     *
     * @details
     *
     * 1. For received HTTP messages, the context is indicated by the userdata parameter of @ref aiot_http_recv_handler_t.
     *
     * 2. For HTTP events, the context is indicated by the userdata parameter of @ref aiot_http_event_handler_t.
     *
     * Data type: (void *)
     */
    AIOT_HTTPOPT_USERDATA,
    /**
     * @brief The callback function that is called when an HTTP message is received.
     *
     * @details
     *
     * Data type: (aiot_http_recv_handler_t)
     */
    AIOT_HTTPOPT_RECV_HANDLER,
    /**
     * @brief The product key of the device. You can obtain the product key from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *)
     */
    AIOT_HTTPOPT_PRODUCT_KEY,
    /**
     * @brief The name of the device. You can obtain the device name from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *)
     */
    AIOT_HTTPOPT_DEVICE_NAME,
    /**
     * @brief The secret of the device. You can obtain the device secret from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *)
     */
    AIOT_HTTPOPT_DEVICE_SECRET,
    /**
     * @brief The extended attributes of the device that reports data. An extended attribute can be the ID of the module provider or module.
     *
     * @details
     *
     * 1. The format of a module provider ID: pid=xxx
     *
     * 2. The format of a module ID: mid=xxx
     *
     * If multiple extended attributes need to be reported, separate them with ampersands (&), for example, pid=xxx&mid=xxx.
     *
     * Data type: (char *)
     */
    AIOT_HTTPOPT_EXTEND_DEVINFO,
    /**
     * @brief The period of time to wait before an @ref aiot_http_auth request times out. You can call @ref aiot_http_auth to send an authentication request and obtain a new token.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_HTTPOPT_AUTH_TIMEOUT_MS,
    /**
     * @brief Specifies whether to establish an HTTP persistent connection.
     *
     * @details
     *
     * If you set this parameter to 0, the SDK client sets up a new connection to the HTTP server each time you call @ref aiot_http_auth or @ref aiot_http_send.
     *
     * Data type: (uint8_t *). Default value: (5 * 1000) ms.
     */
    AIOT_HTTPOPT_LONG_CONNECTION,

    AIOT_HTTPOPT_MAX
} aiot_http_option_t;

/**
 * @brief The type of the message that is returned when the SDK client receives an HTTP packet. The type is then passed to the callback function.
 */
typedef enum {
    /**
     * @brief The HTTP status code that is received.
     */
    AIOT_HTTPRECV_STATUS_CODE,
    /**
     * @brief The HTTP header that is received. A key-value pair in the header is returned each time.
     */
    AIOT_HTTPRECV_HEADER,
    /**
     * @brief The entire HTTP body that is received.
     */
    AIOT_HTTPRECV_BODY
} aiot_http_recv_type_t;

/**
 * @brief The content of the message that is returned when the SDK client receives an HTTP packet. The content is then passed to the callback function.
 */
typedef struct {
    /**
     * @brief The type of the message. For more information, see @ref aiot_http_recv_type_t.
     */
    aiot_http_recv_type_t type;
    union {
        /**
         * @brief The message of the @ref AIOT_HTTPRECV_STATUS_CODE type.
         */
        struct {
            /**
             * @brief The HTTP status code.
             */
            uint32_t code;
        } status_code;
        /**
         * @brief The message of the @ref AIOT_HTTPRECV_HEADER type.
         */
        struct {
            /**
             * @brief The key in a line of the HTTP header.
             */
            char *key;
            /**
             * @brief The value in a line of the HTTP header.
             */
            char *value;
        } header;
        /**
         * @brief The message of the @ref AIOT_HTTPRECV_BODY type.
         */
        struct {
            /**
             * @brief The content of the HTTP body.
             */
            uint8_t *buffer;
            /**
             * @brief The length of the HTTP body.
             */
            uint32_t len;
        } body;
    } data;
} aiot_http_recv_t;

/**
 * @brief The prototype of the callback function that is called when an HTTP message is received. You can set the @ref AIOT_HTTPOPT_RECV_HANDLER parameter of @ref aiot_http_setopt to specify the prototype.
 *
 * @details
 *
 * The response that the SDK client receives from the HTTP server is returned by this callback function.
 *
 * @param[out] handle The HTTP connection handler.
 * @param[out] packet The data that is received from the HTTP server.
 * @param[out] userdata The user context that is cached by the SDK. You can set the @ref AIOT_HTTPOPT_USERDATA parameter to specify the context that needs to be cached.
 *
 * @return void
 */
typedef void (*aiot_http_recv_handler_t)(void *handle, const aiot_http_recv_t *packet, void *userdata);

/**
 * @brief The type of an event. When an event occurs within the SDK, the callback function returns the event information by using this parameter.
 */
typedef enum {
    /**
     * @brief The token is invalid. Call @ref aiot_http_auth to obtain a new token.
     */
    AIOT_HTTPEVT_TOKEN_INVALID
} aiot_http_event_type_t;

/**
 * @brief The information of an event. When an event occurs within the SDK, the callback function returns the event information by using this parameter.
 */
typedef struct {
    aiot_http_event_type_t type;
} aiot_http_event_t;

/**
 * @brief The prototype of the callback function that is called when an HTTP event is triggered. You can set the @ref AIOT_HTTPOPT_RECV_HANDLER parameter of @ref aiot_http_setopt to specify the prototype.
 *
 * @param[out] handle The HTTP connection handler.
 * @param[out] event The struct of the event.
 * @param[out] user_data The pointer that points to the user context cached by the SDK. You can set the @ref AIOT_HTTPOPT_USERDATA parameter to specify the context that needs to be cached.
 */
typedef void (* aiot_http_event_handler_t)(void *handle, const aiot_http_event_t *event, void *userdata);

/**
 * @brief Creates an HTTP connection.
 *
 * @return void*
 *
 * @retval Non-null string: indicates the HTTP connection handler of the connection.
 * @retval Null string: indicates that the initialization of the connection fails.
 */
void *aiot_http_init(void);

/**
 * @brief Sets the parameters of the HTTP connection.
 *
 * @param[in] handle The HTTP connection handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_http_option_t.
 * @param[in] data The configuration data. For more information, see @ref aiot_http_option_t.
 *
 * @return int32_t
 *
 * @retval STATE_SUCCESS: The operation was successful.
 * @retval STATE_HTTP_HANDLE_IS_NULL: The HTTP connection handler was null.
 * @retval STATE_USER_INPUT_OUT_RANGE: The input parameters were invalid.
 * @retval STATE_SYS_DEPEND_MALLOC_FAILED: The memory allocation failed.
 */
int32_t aiot_http_setopt(void *handle, aiot_http_option_t option, void *data);

/**
 * @brief Sends an authentication request and obtains a new token.
 *
 * @param[in] handle The HTTP connection handler.
 *
 * @return int32_t
 *
 * @retval STATE_SUCCESS: The authentication was successful.
 * @retval STATE_HTTP_HANDLE_IS_NULL: The HTTP connection handler was null.
 * @retval STATE_USER_INPUT_MISSING_PRODUCT_KEY: The required product key was not specified.
 * @retval STATE_USER_INPUT_MISSING_DEVICE_NAME: The required device name was not specified.
 * @retval STATE_USER_INPUT_MISSING_DEVICE_SECRET: The required device secret was not specified.
 * @retval STATE_HTTP_TOKEN_LEN_ERROR: The length of the token was invalid.
 * @retval STATE_HTTP_GET_TOKEN_FAILED: No new token was obtained.
 */
int32_t aiot_http_auth(void *handle);

/**
 * @brief Reports data to IoT Platform.
 *
 * @param[in] handle The HTTP connection handler.
 * @param[in] topic The topic that is used for data reporting. You can view the list of topics on the product details page of the IoT Platform console.
 * @param[in] payload The pointer that points to the data to be reported.
 * @param[in] payload_len The size of the data to be reported.
 *
 * @return int32_t
 *
 * @retval STATE_SUCCESS: The data was reported.
 * @retval STATE_HTTP_HANDLE_IS_NULL: The HTTP connection handler was null.
 * @retval STATE_USER_INPUT_OUT_RANGE: The input parameters were invalid.
 * @retval STATE_HTTP_NOT_AUTH: The device failed to be authenticated.
 */
int32_t aiot_http_send(void *handle, char *topic, uint8_t *payload, uint32_t payload_len);

/**
 * The response data that is received from the server follows the following format:
 * {
 *     "code": 0,              // The service-level status code.
 *     "message": "success",   // The message that corresponds to the status code.
 *     "info": {
 *         "messageId": 892687627916247040,
 *     }
 * }
 */

/**
 * @brief The received HTTP response data. The data is returned by the callback function that you specify by using @ref aiot_http_event_handler_t.
 *
 * @param[in] handle The HTTP connection handler.
 *
 * @return int32_t
 *
 * @retval >= 0: The length of the HTTP body.
 * @retval STATE_HTTP_HANDLE_IS_NULLL: The HTTP connection handler was null.
 * @retval STATE_USER_INPUT_NULL_POINTER: The input parameter was null.
 * @retval STATE_USER_INPUT_OUT_RANGE: The value of buffer_len was 0.
 * @retval STATE_HTTP_RSP_MSG_ERROR: The response message received from the server was invalid.
 * @retval STATE_SYS_DEPEND_NWK_CLOSED: The network connection was ended.
 * @retval STATE_SYS_DEPEND_NWK_READ_OVERTIME: The request timed out.
 * @retval STATE_HTTP_RECV_LINE_TOO_LONG: The length of an HTTP header line exceeded the maximum value. This led to a failure of parsing the header.
 * @retval STATE_HTTP_PARSE_STATUS_LINE_FAILED: The parsing of the service-level status code failed.
 * @retval STATE_HTTP_GET_CONTENT_LEN_FAILED: The obtaining of the body length failed.
 */
int32_t aiot_http_recv(void *handle);

/**
 * @brief Ends the HTTP connection that is specified by the p_handle parameter.
 *
 * @param[in] p_handle The pointer that points to the HTTP connection handler.
 *
 * @return int32_t
 *
 * @retval STATE_SUCCESS The operation was successful.
 * @retval STATE_USER_INPUT_NULL_POINTER The p_handle parameter was empty or the handler that p_handle points was null.
 */
int32_t aiot_http_deinit(void **p_handle);


#if defined(__cplusplus)
}
#endif

#endif /* #ifndef _AIOT_HTTP_API_H_ */

