/**
 * @file aiot_mqtt_api.h
 * @brief The header file of the MQTT module. This module establishes MQTT connections to IoT Platform.
 * @date 2019-12-27
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 * @details
 *
 * The MQTT module is used to establish connections to IoT Platform. The following procedure describes how to use the API of the data-model module:
 *
 * 1. Call @ref aiot_mqtt_init to initialize an MQTT session and save the session handler.
 *
 * 2. Call @ref aiot_mqtt_setopt to configure session parameters. For the information about the configuration items, see @ref aiot_mqtt_setopt.
 *
 * 3. Call @ref aiot_mqtt_connect to establish a connection to IoT Platform.
 *
 * 4. Start a process that intermittently calls @ref aiot_mqtt_process to process heartbeat and QoS 1 messages.
 *
 * 5. Start a process that continently calls @ref aiot_mqtt_process to process received MQTT messages.
 *
 *    + When a packet is received, the MQTT module checks the session parameters in the following order. If a condition is met, the corresponding callback function is called and the check stops.
 *
 *      + Checks whether a callback function is specified by setting the @ref AIOT_MQTTOPT_APPEND_TOPIC_MAP parameter of @ref aiot_mqtt_setopt.
 *
 *      + Checks whether a callback function is specified for the topic from which the packet is received. You can call the @ref aiot_mqtt_sub API operation to specify the function.
 *
 *      + Checks whether the default callback function is specified by setting the @ref AIOT_MQTTOPT_RECV_HANDLER parameter of @ref aiot_mqtt_setopt.
 *
 * 6. An MQTT connection to IoT Platform is established. Then, call @ref aiot_mqtt_sub, @ref aiot_mqtt_pub, and other API operations to implement your business logic.
 *
 */

#ifndef _AIOT_MQTT_API_H_
#define _AIOT_MQTT_API_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

/**
 * @brief The type of the MQTT packet.
 *
 * @details
 *
 * The MQTT packet type is passed to @ref aiot_mqtt_recv_handler_t.
 */
typedef enum {
    /**
     * @brief The MQTT PUBLISH packet.
     */
    AIOT_MQTTRECV_PUB,

    /**
     * @brief The MQTT PINGRESP packet.
     */
    AIOT_MQTTRECV_HEARTBEAT_RESPONSE,

    /**
     * @brief The MQTT SUBACK packet.
     */
    AIOT_MQTTRECV_SUB_ACK,

    /**
     * @brief The MQTT UNSUB packet.
     */
    AIOT_MQTTRECV_UNSUB_ACK,

    /**
     * @brief The MQTT PUBACK packet.
     */
    AIOT_MQTTRECV_PUB_ACK,
} aiot_mqtt_recv_type_t;


typedef struct {
    /**
     * @brief The type of the MQTT packet. For more information, see @ref aiot_mqtt_recv_type_t.
     */
    aiot_mqtt_recv_type_t type;
    /**
     * @brief The union of the MQTT packet data. Custom the content based on the packet type.
     */
    union {
        /**
         * @brief The MQTT PUBLISH packet.
         */
        struct {
            uint8_t qos;
            char *topic;
            uint16_t topic_len;
            uint8_t *payload;
            uint32_t payload_len;
        } pub;
        /**
         * @brief AIOT_MQTTRECV_SUB_ACK
         */
        struct {
            int32_t res;
            uint8_t max_qos;
            uint16_t packet_id;
        } sub_ack;
        /**
         * @brief AIOT_MQTTRECV_UNSUB_ACK
         */
        struct {
            uint16_t packet_id;
        } unsub_ack;
        /**
         * @brief AIOT_MQTTRECV_PUB_ACK
         */
        struct {
            uint16_t packet_id;
        } pub_ack;
    } data;
} aiot_mqtt_recv_t;

/**
 * @brief The prototype of the callback function that receives MQTT packets.
 *
 * @param[in] handle The MQTT connection handler.
 * @param[in] packet The struct of the MQTT packet.
 * @param[in] userdata The user context.
 *
 * @return void
 */
typedef void (*aiot_mqtt_recv_handler_t)(void *handle, const aiot_mqtt_recv_t *packet, void *userdata);

/**
 * @brief The MQTT module type.
 */
typedef enum {
    /**
     * @ brief This event is triggered when a network connection of the MQTT session is established for the first time.
     */
    AIOT_MQTTEVT_CONNECT,
    /**
     * @ brief This event is triggered when the network connection of the MQTT session is re-established.
     */
    AIOT_MQTTEVT_RECONNECT,
    /**
     * @brief This event is triggered when the network connection of the MQTT session is ended.
     */
    AIOT_MQTTEVT_DISCONNECT
} aiot_mqtt_event_type_t;

typedef enum {
    /**
     * @brief The network connection of the MQTT session is ended because of a network error.
     */
    AIOT_MQTTDISCONNEVT_NETWORK_DISCONNECT,
    /**
     * @brief The network connection of the MQTT session is ended because no heartbeats are detected for a specified number of times (@ref AIOT_MQTTOPT_HEARTBEAT_MAX_LOST).
     */
    AIOT_MQTTDISCONNEVT_HEARTBEAT_DISCONNECT
} aiot_mqtt_disconnect_event_type_t;

/**
 * @brief The MQTT module event.
 */
typedef struct {
    /**
     * @brief The type of the MQTT module event. For more information, see @ref aiot_mqtt_event_type_t.
     *
     */
    aiot_mqtt_event_type_t type;
    /**
     * @brief The union of the MQTT event data.
     */
    union {
        /**
         * @brief The reason why an MQTT connection is ended.
         */
        aiot_mqtt_disconnect_event_type_t disconnect;
    } data;
} aiot_mqtt_event_t;

/**
 * @brief The callback function of MQTT events.
 *
 * @details
 *
 * The function that is called when an MQTT module event is triggered. The event include connection establishment, disconnection, connection re-establishment.
 *
 */
typedef void (*aiot_mqtt_event_handler_t)(void *handle, const aiot_mqtt_event_t *event, void *userdata);

/**
 * @brief The @ref AIOT_MQTTOPT_APPEND_TOPIC_MAP configuration data of @ref aiot_mqtt_setopt.
 *
 * @details
 *
 * The configuration data includes a topic and a callback function that need to be configured before an MQTT connection is established.
 *
 */
typedef struct {
    char *topic;
    aiot_mqtt_recv_handler_t handler;
    void *userdata;
} aiot_mqtt_topic_map_t;

/**
 * @brief The option parameter of @ref aiot_mqtt_setopt. The data type of each option refers to the data type of the corresponding data parameter of @ref aiot_mqtt_setopt.
 *
 * @details
 *
 * 1. The following script uses an example to describe how to set an option when the data parameter is of the char * type. In this example, @ref AIOT_MQTTOPT_HOST is set.
 *
 *    char *host = "xxx";
 *
 *    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_HOST, host);
 *
 * 2. The following script uses an example to describe how to set an option when the data parameter is of another type. In this example, @ref AIOT_MQTTOPT_PORT is set.
 *
 *    uint16_t port = 443;
 *
 *    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PORT, (void *)&port);
 */
typedef enum {
    /**
     * @brief The domain name or IP address of the MQTT server.
     *
     * @details
     *
     *The endpoints of IoT Platform MQTT servers. Replace ${pk} with the product key.
     *
     * For connections based on TCP or TLS certificates
     *
     * | Domain name                                        | Region    | Port
     * |-------------------------------------------------|---------|---------
     * | ${pk}.iot-as-mqtt.cn-shanghai.aliyuncs.com      | China (Shanghai)    | 443
     * | ${pk}.iot-as-mqtt.ap-southeast-1.aliyuncs.com   | Singapore (Singapore)  | 443
     * | ${pk}.iot-as-mqtt.ap-northeast-1.aliyuncs.com   | Japan (Tokyo)    | 443
     * | ${pk}.iot-as-mqtt.us-west-1.aliyuncs.com        | US (Silicon Valley)    | 443
     * | ${pk}.iot-as-mqtt.eu-central-1.aliyuncs.com     | Germany (Frankfurt)    | 443
     *
     * For connections over TLS-PSK
     *
     * | Domain name                               | Region    | Port
     * |-----------------------------------------|---------|---------
     * | ${pk}.itls.cn-shanghai.aliyuncs.com     | China (Shanghai)    | 1883
     *
     * For connections based on TSL x509 client certificates
     *
     * | Domain name                            | Region    | Port |
     * |-------------------------------------|---------|---------
     * | x509.itls.cn-shanghai.aliyuncs.com  | China (Shanghai)    | 1883
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_HOST,

    /**
     * @brief The number of the port that is used by the MQTT server.
     *
     * @details
     *
     * For connections to IoT Platform:
     *
     * 1. If a TCP or TSL certificate is used, set the port number to 443.
     *
     * 2. If a TSL-PSK or TLS x509 client certificate is used, set the port number to 1883.
     *
     * Data type: (uint16_t *).
     */
    AIOT_MQTTOPT_PORT,

    /**
     * @brief The product key of the device. You can obtain the product key from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_PRODUCT_KEY,

    /**
     * @brief The name of the device. You can obtain the device name from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_DEVICE_NAME,

    /**
     * @brief The secret of the device. You can obtain the device secret from the <a href="http://iot.console.aliyun.com/">IoT Platform console</a>.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_DEVICE_SECRET,

    /**
     * @brief The extended attributes of the device that reports data. An extended attribute can be the ID of the module provider or module or the operating system.
     *
     * @details
     *
     * Format:
     *
     * "mid=<Module ID>,pid=<Module provider ID>,os=<Operating system>"
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_EXTEND_CLIENTID,

    /**
     * @brief The security mode of the connection between the device and IoT Platform. Do not specify this field if standard TCP or TSL is used.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_SECURITY_MODE,

    /**
     * @brief The username of the credential if a custom credential is used to connect to the MQTT server.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_USERNAME,

    /**
     * @brief The password of the credential if a custom credential is used to connect to the MQTT server.
     *
     * @brief
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_PASSWORD,

    /**
     * @brief The client ID of the credential if a custom credential is used to connect to the MQTT server.
     *
     * @details
     *
     * Data type: (char *).
     */
    AIOT_MQTTOPT_CLIENTID,

    /**
     * @brief The maximum interval between two consecutive heartbeats in the CONNECT packets during MQTT connection establishment.
     *
     * @details
     *
     * The value range is 30s to 1200s due to the limit of IoT Platform.
     *
     * 1. If you specify a value less than 30, IoT Platform rejects the MQTT connection. @ref aiot_mqtt_connect returns the @ref STATE_MQTT_CONNACK_RCODE_SERVER_UNAVAILABLE error.
     *
     * 2. If you specify a value greater than 1200, IoT Platform accepts the MQTT connection. However, the maximum heartbeat interval is set to 1200.
     *
     * Data type: (uint16_t *). Valid values: 30s to 1200s. Default value: 1200s.
     */
    AIOT_MQTTOPT_KEEPALIVE_SEC,

    /**
     * @brief The clean session parameter in the CONNECT packet during MQTT connection establishment.
     *
     * @details
     *
     * 1. If the clean session parameter is set to 0 when the device goes online, QoS 1 messages that the server pushes before the device goes online are pushed to the device.
     *
     * 2. If the clean session parameter is set to 0 when the device goes online, QoS 1 messages that the server pushes before the device goes online are dropped.
     *
     * Data type: (uint8_t *). Valid values: 0 and 1. Default value: 1.
     */
    AIOT_MQTTOPT_CLEAN_SESSION,

    /**
     * @brief The security credential that is used for MQTT connection establishment.
     *
     * @details
     *
     * The security credential is used by @ref aiot_sysdep_network_cred_t at the bottom-layer network.
     *
     * 1. If this parameter is not specified, an MQTT connection is established over TCP.
     *
     * 2. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_NONE, an HTTP connection is established over TCP.
     *
     * 3. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA, MQTT, an HTTP connection is established over TLS.
     *
     * 4. If the option parameter of @ref aiot_sysdep_network_cred_t is set to @ref AIOT_SYSDEP_NETWORK_CRED_SVRCERT_PSK, MQTT, an HTTP connection is established over TLS-PSK.
     *
     * Data type: (aiot_sysdep_network_cred_t *).
     */
    AIOT_MQTTOPT_NETWORK_CRED,

    /**
     * @brief The period of time to wait before the request for MQTT connection establishment times out.
     *
     * @details
     *
     This parameter specifies the period of time to wait before the request for socket connection establishment times out.
     *
     * Data type: (uint32_t *). Default value: (5 *1000) ms.
     *
     */
    AIOT_MQTTOPT_CONNECT_TIMEOUT_MS,

    /**
     * @brief The interval at which MQTT PINGREQ packets (heartbeats) are sent.  
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (25 * 1000) ms.
     */
    AIOT_MQTTOPT_HEARTBEAT_INTERVAL_MS,

    /**
     * @brief The number of MQTT PINGRESP packets that are lost before the connection is re-established.
     *
     * @details
     *
     * Data type: (uint8_t *). Default value: (2).
     */
    AIOT_MQTTOPT_HEARTBEAT_MAX_LOST,

    /**
     * @brief Specifies whether to enable the MQTT connection re-establishment feature.
     *
     * @details
     *
     * Data type: (uint8_t *). Valid values: 0 and 1. Default value: 1.
     */
    AIOT_MQTTOPT_RECONN_ENABLED,

    /**
     * @brief The interval of two consecutive re-establishment attempts. The re-establishment may be triggered due to heartbeat loss or network disconnection.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_MQTTOPT_RECONN_INTERVAL_MS,

    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when data is sent over MQTT.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_MQTTOPT_SEND_TIMEOUT_MS,

    /**
     * @brief The upper limit of the latency that is caused by the operations related to protocol stacks when data is received over MQTT.
     *
     * @details
     *
     * Data type: (uint32_t *). Default value: (5 * 1000) ms.
     */
    AIOT_MQTTOPT_RECV_TIMEOUT_MS,

    /**
     * @brief The interval at which a QoS 1 message is re-sent.
     *
     * @details
     *
     * If a QoS 1 MQTT PUBLISH packet is sent and no MQTT PUBACK packet is received within the period specified by @ref AIOT_MQTTOPT_REPUB_TIMEOUT_MS,
     * @ref aiot_mqtt_process will re-send the QoS 1 MQTT PUBLISH packet until an MQTT PUBACK packet is received.
     *
     * Data type: (uint32_t *). Default value: (3 * 1000) ms.
     */
    AIOT_MQTTOPT_REPUB_TIMEOUT_MS,

    /**
     * @brief The timeout period for other API operations to complete when you end an MQTT connection.
     *
     * @details
     *
     * If you call @ref aiot_mqtt_deinit to end an MQTT connection and then call another aiot_mqtt_xxx API operation, the operation returns the @ref STATE_USER_INPUT_EXEC_DISABLED error.
     *
     * In this case, stop calling other aiot_mqtt_xxx API operations.
     *
     * Data type: (uint32_t *). Default value: (2 * 1000) ms.
     */
    AIOT_MQTTOPT_DEINIT_TIMEOUT_MS,

    /**
     * @brief The callback function that notifies your applications when data is received from the MQTT server.
     *
     * @details
     *
     * 1. If the callback function is not specified and no corresponding topic is registered, received messages from the topic are dropped.
     *
     * 2. If the callback function is specified, it is called when no corresponding topic is registered.
     *
     * Data type: ( @ref aiot_mqtt_recv_handler_t ).
     */
    AIOT_MQTTOPT_RECV_HANDLER,

    /**
     * @brief The callback function that notifies your applications of MQTT client events. For example, the function is called when the MQTT client goes online or offline.
     *
     * @details
     *
     * Data type: ( @ref aiot_mqtt_event_handler_t ).
     */
    AIOT_MQTTOPT_EVENT_HANDLER,

    /**
     * @brief Configures the MQTT topic and corresponding callback function before MQTT connection establishment.
     *
     * @details
     *
     * Data type: ( @ref aiot_mqtt_topic_map_t ).
     */
    AIOT_MQTTOPT_APPEND_TOPIC_MAP,

    /**
     * @brief Deletes the MQTT topic and corresponding callback function.
     *
     * @details
     *
     * Data type: ( @ref aiot_mqtt_topic_map_t ).
     */
    AIOT_MQTTOPT_REMOVE_TOPIC_MAP,

    /**
     * @brief Attaches a request ID in the string format to the topic that is used to publish messages. The request ID is used for end-to-end log tracking.
     *
     * @details
     *
     * Data type: (uint8_t *). Default value: 0.
     *
     * If you specify 0, no request ID is attached. If you specify 1, a request ID is attached.
     */
    AIOT_MQTTOPT_APPEND_REQUESTID,

    /**
     * @brief The user context that needs to be cached by the SDK client.
     *
     * @details
     *
     * 1. The user context will be returned by @ref AIOT_MQTTOPT_RECV_HANDLER or @ref AIOT_MQTTOPT_EVENT_HANDLER.
     *
     * 2. If userdata is not specified for @ref AIOT_MQTTOPT_APPEND_TOPIC_MAP or @ref aiot_mqtt_sub, the context will be passed to the callback functions.
     *
     * Data type: (void *).
     */
    AIOT_MQTTOPT_USERDATA,

    /**
     * @brief The max num of cached qos 1 messages 
     *
     */
    AIOT_MQTTOPT_MAX_REPUB_NUM,

    AIOT_MQTTOPT_MAX
} aiot_mqtt_option_t;

/**
 * @brief Initializes an MQTT session and retains the default settings for the session parameters.
 *
 * @return void*
 * @retval Non-null string: indicates the MQTT connection handler.
 * @retval Null string: indicates that the initialization of the session fails. In most cases, the initialization failure is caused by a memory allocation error.
 *
 */
void   *aiot_mqtt_init(void);

/**
 * @brief Configures the MQTT session.
 *
 * @details
 *
 * The following list describes the required configuration items.
 *
 * You can retain the default values for the optional configuration items or change the values based on your business requirements.
 *
 * + `AIOT_MQTTOPT_HOST`: The address of the Alibaba Cloud MQTT host.
 *
 * + `AIOT_MQTTOPT_PORT`: The number of the MQTT host port that is used for connection.
 *
 * + `AIOT_MQTTOPT_PRODUCT_KEY`: The product key.
 *
 * + `AIOT_MQTTOPT_DEVICE_NAME`: The device name.
 *
 * + `AIOT_MQTTOPT_DEVICE_SECRET`: The device secret.
 *
 * + `AIOT_MQTTOPT_NETWORK_CRED`: The security credentials that are used to establish an MQTT connection.
 *
 * + `AIOT_MQTTOPT_RECV_HANDLER`: The default callback function that receives data.
 *
 * + `AIOT_MQTTOPT_EVENT_HANDLER`: The callback function that notifies your applications of MQTT events.
 *
 * @param[in] handle The MQTT connection handler.
 * @param[in] option The configuration options. For more information, see @ref aiot_mqtt_option_t.
 * @param[in] data   The configuration data. For more information, see @ref aiot_mqtt_option_t.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The configuration succeeded.
 *
 */
int32_t aiot_mqtt_setopt(void *handle, aiot_mqtt_option_t option, void *data);

/**
 * @brief Releases the resources of an MQTT session handler.
 *
 * @param[in] handle The pointer that points to the MQTT connection handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 */
int32_t aiot_mqtt_deinit(void **handle);

/**
 * @brief Establishes a connection to the MQTT server.
 *
 * @details
 *
 * If you call @ref aiot_mqtt_setopt to set the MQTT connection parameters, the SDK client selects the connection parameters in the following order:
 *
 * 1. If the following parameters are set, the configured connection parameters are used to connect to an MQTT server that is specified by @ref AIOT_MQTTOPT_HOST.
 *
 *    + @ref AIOT_MQTTOPT_USERNAME
 *    + @ref AIOT_MQTTOPT_PASSWORD
 *    + @ref AIOT_MQTTOPT_CLIENTID
 *
 * 2. If the following parameters are specified, the SDK client connects to IoT Platform by using the username and password that are calculated by the signature algorithm.
 *
 *    + @ref AIOT_MQTTOPT_PRODUCT_KEY
 *    + @ref AIOT_MQTTOPT_DEVICE_NAME
 *    + @ref AIOT_MQTTOPT_DEVICE_SECRET
 *
 * @param[in] handle The MQTT connection handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 * @note
 *
 * If you specify a custom credential by using @ref AIOT_MQTTOPT_USERNAME, @ref AIOT_MQTTOPT_PASSWORD, and @ref AIOT_MQTTOPT_CLIENTID,
 *
 * the function ignores @ref AIOT_MQTTOPT_PRODUCT_KEY, @ref AIOT_MQTTOPT_DEVICE_NAME, and @ref AIOT_MQTTOPT_DEVICE_SECRET.
 *
 * The function uses the custom credential to connect to the specified MQTT server.
 */
int32_t aiot_mqtt_connect(void *handle);

/**
 * @brief Ends the connection to the MQTT server.
 *
 * @details
 *
 * The function sends an MQTT DISCONNECT packet to the MQTT server to end the connection.
 *
 * If you need to re-establish the connection to the MQTT server, call @ref aiot_mqtt_connect.
 *
 * @param[in] handle The MQTT connection handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 */
int32_t aiot_mqtt_disconnect(void *handle);

/**
 * @brief Sends an MQTT PINGREQ packet to send a heartbeat.
 *
 * @details
 *
 * @ref aiot_mqtt_process includes a mechanism that sends heartbeats on a regular basis. If you need to send an additional heartbeat, call this function.
 *
 * @param[in] handle The MQTT connection handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 */
int32_t aiot_mqtt_heartbeat(void *handle);

/**
 * @brief The function that re-sent regular heartbeats and QoS 1 messages.
 *
 * @details
 *
 * 1. Heartbeats are sent to the MQTT broker to maintain the MQTT connection. @ref AIOT_MQTTOPT_HEARTBEAT_INTERVAL_MS specifies the heartbeat interval.
 *
 * 2. If no MQTT PUBACK packet is received for a QoS1 MQTT PUBLISH packet within the period specified by @ref AIOT_MQTTOPT_REPUB_TIMEOUT_MS, the function re-sends the MQTT PUBLISH packet until an MQTT PUBACK packet is received.
 *
 * @param[in] handle The MQTT connection handler.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 *
 * @note
 *
 * The function is a non-blocking function and is intermittently called. The interval between two consecutive calls must be smaller than the smaller value between @ref AIOT_MQTTOPT_HEARTBEAT_INTERVAL_MS and @ref AIOT_MQTTOPT_REPUB_TIMEOUT_MS.
 *
 * The function provides the mechanism to re-sent heartbeats and QoS 1 messages.
 */
int32_t aiot_mqtt_process(void *handle);

/**
 * @brief Publishes a QoS 0 PUBLISH packet to an MQTT server to publish a message.
 *
 * @param[in] handle The MQTT connection handler.
 * @param[in] topic The topic of the MQTT PUBLISH packet.
 * @param[in] payload The payload of the MQTT PUBLISH packet.
 * @param[in] payload_len The length of the payload in the MQTT PUBLISH packet.
 * @param[in] qos The MQTT QoS level. QoS 0 and QoS 1 are supported.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 */
int32_t aiot_mqtt_pub(void *handle, char *topic, uint8_t *payload, uint32_t payload_len, uint8_t qos);

/**
 * @brief Sends an MQTT SUBSCRIBE packet to the MQTT server to subscribe to a topic.
 *
 * @param[in] handle The MQTT connection handler.
 * @param[in] topic The topic of the MQTT SUBSCRIBE packet.
 * @param[in] handler The callback function that corresponds to the topic and processes MQTT PUBLISH packets. If a message is published to the topic, the function is called.
                    If you set handler to NULL, the SDK client calls the function specified by @ref AIOT_MQTTOPT_RECV_HANDLER.
                    If you have called aiot_mqtt_sub() for multiple times and specified different callback functions for the same topic, the SDK client calls all of the callback functions.
 * @param[in] qos The maximum QoS level to be supported by the MQTT server for the topic. QoS 0 and QoS 1 are supported.
 * @param[in] userdata The user context that is cached by the SDK client. When a callback function is called, the context is returned by using the handler.
 *                  If no context is specified, the context that is specified by @ref AIOT_MQTTOPT_USERDATA is returned by the callback function.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 */
int32_t aiot_mqtt_sub(void *handle, char *topic, aiot_mqtt_recv_handler_t handler, uint8_t qos, void *userdata);

/**
 * @brief Sends an MQTT UNSUBSCRIBE packet to the MQTT server to cancel the subscription to a specified topic.
 *
 * @param[in] handle The MQTT connection handler.
 * @param[in] topic The topic of the MQTT UNSUBSCRIBE packet.
 *
 * @return int32_t
 * @retval <STATE_SUCCESS The configuration failed. For more information, see @ref aiot_state_api.h.
 * @retval >=STATE_SUCCESS The operation succeeded.
 */
int32_t aiot_mqtt_unsub(void *handle, char *topic);

/**
 * @brief Attempts to receive an MQTT packet form the network.
 *
 * @details
 *
 * This function includes a re-establishment mechanism.
 *
 * 1. When the number of lost MQTT heartbeats exceeds @ref AIOT_MQTTOPT_HEARTBEAT_MAX_LOST, the connection re-establishment mechanism is triggered.
 *
 *    + @ref AIOT_MQTTOPT_RECONN_INTERVAL_MS specifies the interval of two consecutive re-establishment attempts.
 *
 * 2. If the SDK client detects a network disconnection, the connection re-establishment mechanism is triggered.
 *
 *    + @ref AIOT_MQTTOPT_RECONN_INTERVAL_MS specifies the interval of two consecutive re-establishment attempts.
 *
 * @param[in] handle
 *
 * @retval STATE_SYS_DEPEND_NWK_READ_LESSDATA The operation succeeded and no MQTT packet is available to receive.
 * @retval >=STATE_SUCCESS The operation succeeded.
 * @retval others The configuration failed. For more information, see @ref aiot_state_api.h.
 *
 * @note
 *
 * If the network connection is normal and @ref aiot_mqtt_deinit is not called, the function is blocking and is continently called.
 *
 * 1. If the network is disconnected, the function returns @ref STATE_SYS_DEPEND_NWK_CLOSED.
 *
 * 2. If @ref aiot_mqtt_deinit is called, the function returns @ref STATE_USER_INPUT_EXEC_DISABLED.
 */
int32_t aiot_mqtt_recv(void *handle);

#if defined(__cplusplus)
}
#endif

#endif

