/**
 * @file aiot_sysdep_api.h
 * @brief This header file includes all dependencies that are required by the core modules in the SDK.
 * @date 2019-12-27
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 */

#ifndef _AIOT_SYSDEP_API_H_
#define _AIOT_SYSDEP_API_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>

typedef enum {
    AIOT_SYSDEP_NETWORK_CRED_NONE,
    AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA,
    AIOT_SYSDEP_NETWORK_CRED_SVRCERT_PSK,
    AIOT_SYSDEP_NETWORK_CRED_MAX
} aiot_sysdep_network_cred_option_t;

typedef struct {
    aiot_sysdep_network_cred_option_t option;  /* Security policy */
    uint32_t      max_tls_fragment;
    uint8_t       sni_enabled;
    const char   *x509_server_cert;     /* This parameter must be stored in a static storage area. The SDK does not create a replica for this parameter. */
    uint32_t      x509_server_cert_len;
    const char   *x509_client_cert;     /* This parameter must be stored in a static storage area. The SDK does not create a replica for this parameter. */
    uint32_t      x509_client_cert_len;
    const char   *x509_client_privkey;  /* This parameter must be stored in a static storage area. The SDK does not create a replica for this parameter. */
    uint32_t      x509_client_privkey_len;
    char         *tls_extend_info;
} aiot_sysdep_network_cred_t;

typedef enum {
    CORE_SYSDEP_SOCKET_TCP_CLIENT,
    CORE_SYSDEP_SOCKET_TCP_SERVER,
    CORE_SYSDEP_SOCKET_UDP_CLIENT,
    CORE_SYSDEP_SOCKET_UDP_SERVER
} core_sysdep_socket_type_t;

typedef struct {
    char *psk_id;
    char *psk;
} core_sysdep_psk_t;

typedef enum {
    CORE_SYSDEP_NETWORK_SOCKET_TYPE,             /* The type of the socket that you want to create. Data type: (core_sysdep_socket_type_t *) */
    CORE_SYSDEP_NETWORK_HOST,                    /* The FQDN or IP address that you want to use to establish a network connection.  Data type: (char *) */
    CORE_SYSDEP_NETWORK_BACKUP_IP,               /* The backup IP address. If the system fails to establish a connection because the FQDN cannot be parsed, the system reconnects to IoT Platform by using the backup IP address. */
    CORE_SYSDEP_NETWORK_PORT,                    /* The number of the port that you want to use to establish a network connection. Data type: (uint16_t *) */
    CORE_SYSDEP_NETWORK_CONNECT_TIMEOUT_MS,      /* The timeout period that specifies the maximum amount of time that is required to establish a network connection. Data type: (uint32_t *) */
    CORE_SYSDEP_NETWORK_CRED,                    /* The parameter that is used to ensure the security of the network layer. Data type: (aiot_sysdep_network_cred_t *) */
    CORE_SYSDEP_NETWORK_PSK,                     /* The parameter that is used together with the psk-id and psk parameters in PSK mode.  Data type: (core_sysdep_psk_t *) */
    CORE_SYSDEP_NETWORK_MAX
} core_sysdep_network_option_t;

typedef struct {
    char addr[16]; /* Specifies an IPv4 address that is in the dot-decimal notation. The IPv4 address can be up to 15 characters in length. */
    uint16_t port; /* Specifies a port number. */
} core_sysdep_addr_t;

/* This parameter is used for a compilation. In most cases, the parameter is automatically configured by the system. */

/**
 * @brief The structure that specifies how the SDK can use the resources of the hardware platform on which the SDK runs.
 */
typedef struct {
    /**
     * @brief Applies for memory.
     */
    void    *(*core_sysdep_malloc)(uint32_t size, char *name);
    /**
     * @brief Releases memory.
     */
    void (*core_sysdep_free)(void *ptr);
    /**
     * @brief Obtains the current timestamp that is used by the SDK to calculate a difference.
     */
    uint64_t (*core_sysdep_time)(void);
    /**
     * @brief The number of milliseconds that are required for a hibernation.
     */
    void (*core_sysdep_sleep)(uint64_t time_ms);
    /**
     * @brief Creates a Layer 3 network session.
     */
    void    *(*core_sysdep_network_init)(void);

    /**
     * @brief Configures the connection parameters of a network session.
     */
    int32_t (*core_sysdep_network_setopt)(void *handle, core_sysdep_network_option_t option, void *data);
    /**
     * @brief Creates a network session based on which an MQTT or HTTP application runs.
     */
    int32_t (*core_sysdep_network_establish)(void *handle);
    /**
     * @brief Reads data from a specified network session.
     */
    int32_t (*core_sysdep_network_recv)(void *handle, uint8_t *buffer, uint32_t len, uint32_t timeout_ms,
                                        core_sysdep_addr_t *addr);
    /**
     * @brief Sends data to a specified network session.
     */
    int32_t (*core_sysdep_network_send)(void *handle, uint8_t *buffer, uint32_t len, uint32_t timeout_ms,
                                        core_sysdep_addr_t *addr);
    /**
     * @brief Destroys a network session.
     */
    int32_t (*core_sysdep_network_deinit)(void **handle);
    /**
     * @brief Generates random numbers.
     */
    void (*core_sysdep_rand)(uint8_t *output, uint32_t output_len);
    /**
     * @brief Creates a mutex.
     */
    void    *(*core_sysdep_mutex_init)(void);
    /**
     * @brief Applies for a mutex.
     */
    void (*core_sysdep_mutex_lock)(void *mutex);
    /**
     * @brief Releases a mutex.
     */
    void (*core_sysdep_mutex_unlock)(void *mutex);
    /**
     * @brief Destroys a mutex.
     */
    void (*core_sysdep_mutex_deinit)(void **mutex);
} aiot_sysdep_portfile_t;

void aiot_sysdep_set_portfile(aiot_sysdep_portfile_t *portfile);
aiot_sysdep_portfile_t *aiot_sysdep_get_portfile(void);

#if defined(__cplusplus)
}
#endif

#endif

