/* This routine demonstrates using the SDK to configure parameters to dynamically register a device, initiate a request and receive a reply
 *
 * + If the receive and reply fails, the instance is destroyed, the resource is reclaimed, and the program exits
 * + If the reply is received successfully, it is demonstrated in the reply processing callback function of 'demo_dynreg_recv_handler()' to parse the contents of the server reply
 *
 * The parts that require user attention or modification are indicated in the comments with 'TODO'
 *
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#include "aiot_state_api.h"
#include "aiot_sysdep_api.h"
#include "aiot_dynreg_api.h"

/* TODO: input your device's auth info */
char *product_key       = "${YourProductKey}";
char *device_name       = "${YourDeviceName}";
char *product_secret     = "${YourProductSecret}";

/*
    TODO: fill your region info in dynamic registration auth access point in the format
    "iot-auth.${YourRegionId}.aliyuncs.com"

     * | iot-auth.cn-shanghai.aliyuncs.com      | Shanghai  | 443
     * | iot-auth.ap-southeast-1.aliyuncs.com   | Singapore | 443
     * | iot-auth.ap-northeast-1.aliyuncs.com   | Japan     | 443
     * | iot-auth.us-west-1.aliyuncs.com        | West-US   | 443
     * | iot-auth.eu-central-1.aliyuncs.com     | EU        | 443
     * 
*/
char       *host = "iot-auth.ap-southeast-1.aliyuncs.com"; /* demo registration auth access point */

typedef struct {
    uint32_t code;
    char *device_secret;
} demo_info_t;

/* Set of system adaptation functions, located in portfiles/aiot_port */
extern aiot_sysdep_portfile_t g_aiot_sysdep_portfile;

/* The server certificate in external/ali_ca_cert.c */
extern const char *ali_ca_cert;

/* 
 * TODO: If you want to turn logging off, implement this function as null. If you want to reduce logging, choose not to print according to code
 * For example, [1577589489.033][LK-0317] mqTT_basic_demo &gb80sFmX7yX
 * The code for this log is 0317(hexadecimal). See core/aiot_state_api.h for the code value
 */

/* The log callback function from which the SDK logs are output */
static int32_t demo_state_logcb(int32_t code, char *message)
{
    printf("%s", message);
    return 0;
}

/* Data processing callback, called when the SDK receives a dynreg message from the Iot Platform */
void demo_dynreg_recv_handler(void *handle, const aiot_dynreg_recv_t *packet, void *userdata)
{
    demo_info_t *demo_info = (demo_info_t *)userdata;

    switch (packet->type) {
        case AIOT_DYNREGRECV_STATUS_CODE: {
            demo_info->code = packet->data.status_code.code;
        }
        break;
        /* TODO: In the callback, you need to copy and save the contents of the space pointed to by the package, 
           because after the callback returns, these spaces will be released by the SDK
        */
        case AIOT_DYNREGRECV_DEVICE_INFO: {
            demo_info->device_secret = malloc(strlen(packet->data.device_info.device_secret) + 1);
            if (demo_info->device_secret != NULL) {
                memset(demo_info->device_secret, 0, strlen(packet->data.device_info.device_secret) + 1);
                memcpy(demo_info->device_secret, packet->data.device_info.device_secret,
                       strlen(packet->data.device_info.device_secret));
            }
        }
        break;
        default: {

        }
        break;
    }
}

int main(int argc, char *argv[])
{
    int32_t     res = STATE_SUCCESS;
    void       *dynreg_handle = NULL;

    uint16_t    port = 443;       /* the destination port */
    aiot_sysdep_network_cred_t cred; /* Security credential data structure to configure the CA certificate and other parameters, applicable to TLS connection scenario */
    demo_info_t demo_info;

    memset(&demo_info, 0, sizeof(demo_info_t));

    /* Configure the underlying system dependencies of the SDK */
    aiot_sysdep_set_portfile(&g_aiot_sysdep_portfile);
    /* Configure the log output of the SDK */
    aiot_state_set_logcb(demo_state_logcb);

    /* Create the SDK's security credentials to establish a TLS connection */
    memset(&cred, 0, sizeof(aiot_sysdep_network_cred_t));
    cred.option = AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA;   /* Use the RSA to verify the MQTT server */
    cred.max_tls_fragment = 16384;  /* The maximum shard length is 16K, and other optional values are 4K, 2K, 1K, 0.5K */
    cred.sni_enabled = 1;                                /* enable the feature of the Server Name Indicator of TLS */
    cred.x509_server_cert = ali_ca_cert;                 /* The RSA root certificate used to verify the MQTT server */
    cred.x509_server_cert_len = strlen(ali_ca_cert);     /* length of RSA certificate */

    /* Create a dynreg client instance and internally initialize the default parameters */
    dynreg_handle = aiot_dynreg_init();
    if (dynreg_handle == NULL) {
        printf("aiot_dynreg_init failed\n");
        return -1;
    }

    /* Configure the server address of the connection */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_HOST, (void *)host);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_HOST failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Configure the server port for the connection */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_PORT, (void *)&port);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_PORT failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Configure the device's productKey */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_PRODUCT_KEY, (void *)product_key);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_PRODUCT_KEY failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Configure the device's productSecret */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_PRODUCT_SECRET, (void *)product_secret);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_PRODUCT_SECRET failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Configure the device's deviceName */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_DEVICE_NAME, (void *)device_name);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_DEVICE_NAME failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Configure the security credentials for the network connection, which have been created above */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_NETWORK_CRED, (void *)&cred);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_NETWORK_CRED failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Configure default callback function for received messages */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_RECV_HANDLER, (void *)demo_dynreg_recv_handler);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_RECV_HANDLER failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Sets the user context, which is passed back when the demo_dynreg_recv_handler is invoked */
    res = aiot_dynreg_setopt(dynreg_handle, AIOT_DYNREGOPT_USERDATA, (void *)&demo_info);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_setopt AIOT_DYNREGOPT_USERDATA failed, res: -0x%04X\n", -res);
        aiot_dynreg_deinit(&dynreg_handle);
        return -1;
    }

    /* Send a dynamic registration request */
    res = aiot_dynreg_send_request(dynreg_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_send_request failed: -0x%04X\n", -res);
        return -1;
    }

    /* Receive dynamic registration reply */
    res = aiot_dynreg_recv(dynreg_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_recv failed: -0x%04X\n", -res);
        return -1;
    }

    printf("status code: %d\n", demo_info.code);

    /* Print out the deviceSecret in the service reply */
    if (demo_info.device_secret != NULL) {
        printf("device secret: %s\n", demo_info.device_secret);
        free(demo_info.device_secret);
    }

    /* Destroys the dynamic registration instance */
    res = aiot_dynreg_deinit(&dynreg_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_dynreg_deinit failed: -0x%04X\n", -res);
        return -1;
    }

    return 0;
}

