/*
 * This example is suitable for POSIX devices that support pthread such as 'Linux', which demonstrates configuring MQTT parameters with the SDK and establishing a connection, after which 2 threads are created
 *
 * + One thread is used to keep long connections alive
 * + The other thread is used to receive messages and enter the default data callback when a message arrives, and an event callback when the connection state changes
 *
 * Parts that require user attention or modification have been indicated in the comments with TODO
 *
 */


#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>

#include "aiot_state_api.h"
#include "aiot_sysdep_api.h"
#include "aiot_mqtt_api.h"
#include "aiot_dm_api.h"

/* TODO: input your device's auth info */
char *product_key       = "${YourProductKey}";
char *device_name       = "${YourDeviceName}";
char *device_secret     = "${YourDeviceSecret}";

/*
    TODO: Replace the access point with your own instance's access point in the format
    "${YourProductKey}.iot-as-mqtt.${YourRegionId}.aliyuncs.com"
    where ${YourProductKey}: Please replace with the ProductKey of the product to which the device belongs. 
    You can login to the IoT Platform console and obtain it on the device details page of the corresponding instance.
    ${YourRegionId}: Please replace the domain code for your IoT Platform device, such as cn-shanghai, etc
    Example of the complete mqtt_host in this case: a1TTmBPIChA.iot-as-mqtt.cn-shanghai.aliyuncs.com

     * | ${YourProductKey}.iot-as-mqtt.cn-shanghai.aliyuncs.com      | Shanghai  | 443
     * | ${YourProductKey}.iot-as-mqtt.ap-southeast-1.aliyuncs.com   | Singapore | 443
     * | ${YourProductKey}.iot-as-mqtt.ap-northeast-1.aliyuncs.com   | Japan     | 443
     * | ${YourProductKey}.iot-as-mqtt.us-west-1.aliyuncs.com        | West-US   | 443
     * | ${YourProductKey}.iot-as-mqtt.eu-central-1.aliyuncs.com     | EU        | 443
     *
*/
char  *mqtt_host = "${YourProductKey}.iot-as-mqtt.cn-shanghai.aliyuncs.com";

/* Set of system adaptation functions, located in portfiles/aiot_port */
extern aiot_sysdep_portfile_t g_aiot_sysdep_portfile;

/* The server certificate in external/ali_ca_cert.c */
extern const char *ali_ca_cert;

static pthread_t g_mqtt_process_thread;
static pthread_t g_mqtt_recv_thread;
static uint8_t g_mqtt_process_thread_running = 0;
static uint8_t g_mqtt_recv_thread_running = 0;

/* 
 * TODO: If you want to turn logging off, implement this function as null. If you want to reduce logging, choose not to print according to code
 * For example, [1577589489.033][LK-0317] mqTT_basic_demo &gb80sFmX7yX
 * The code for this log is 0317(hexadecimal). See core/aiot_state_api.h for the code value
 */

/* The log callback function from which the SDK logs are output */
int32_t demo_state_logcb(int32_t code, char *message)
{
    printf("%s", message);
    return 0;
}


/* MQTT event callback function that is triggered when the network is connected/reconnected/disconnected. The event is defined in core/aiot_mqtt_api.h */
void demo_mqtt_event_handler(void *handle, const aiot_mqtt_event_t *event, void *userdata)
{
    switch (event->type) {
        /* SDK successfully established a connection to the MQTT server because the user called the aiot_mqtt_connect () function */
        case AIOT_MQTTEVT_CONNECT: {
            printf("AIOT_MQTTEVT_CONNECT\n");
        }
        break;

        /* The SDK was disconnected due to the network condition, and the automatic reconnection was successfully initiated */
        case AIOT_MQTTEVT_RECONNECT: {
            printf("AIOT_MQTTEVT_RECONNECT\n");
        }
        break;

        /* The SDK disconnects passively due to network conditions. The network indicates that the underlying read and write fails. 
         * The heartbeat indicates that the server does not receive a heartbeat response as expected 
         */
        case AIOT_MQTTEVT_DISCONNECT: {
            char *cause = (event->data.disconnect == AIOT_MQTTDISCONNEVT_NETWORK_DISCONNECT) ? ("network disconnect") :
                          ("heartbeat disconnect");
            printf("AIOT_MQTTEVT_DISCONNECT: %s\n", cause);
        }
        break;

        default: {

        }
    }
}

/* The thread that performs aiot_mqtt_process, including heartbeat sending and QoS1 message resending */
void *demo_mqtt_process_thread(void *args)
{
    int32_t res = STATE_SUCCESS;

    while (g_mqtt_process_thread_running) {
        res = aiot_mqtt_process(args);
        if (res == STATE_USER_INPUT_EXEC_DISABLED) {
            break;
        }
        sleep(1);
    }
    return NULL;
}

/* The thread that performs aiot_mqtt_recv, handles network auto-reconnection and reception of MQTT messages from the server */
void *demo_mqtt_recv_thread(void *args)
{
    int32_t res = STATE_SUCCESS;

    while (g_mqtt_recv_thread_running) {
        res = aiot_mqtt_recv(args);
        if (res < STATE_SUCCESS) {
            if (res == STATE_USER_INPUT_EXEC_DISABLED) {
                break;
            }
            sleep(1);
        }
    }
    return NULL;
}

static void demo_dm_recv_generic_reply(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_generic_reply msg_id = %d, code = %d, data = %.*s, message = %.*s\r\n",
           recv->data.generic_reply.msg_id,
           recv->data.generic_reply.code,
           recv->data.generic_reply.data_len,
           recv->data.generic_reply.data,
           recv->data.generic_reply.message_len,
           recv->data.generic_reply.message);
}

static void demo_dm_recv_property_set(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_property_set msg_id = %ld, params = %.*s\r\n",
           (unsigned long)recv->data.property_set.msg_id,
           recv->data.property_set.params_len,
           recv->data.property_set.params);

    /* TODO: The following demo code shows how to respond to a property setting command from the cloud platform. The user can uncomment it to see the demonstration */
    /*
    {
        aiot_dm_msg_t msg;

        memset(&msg, 0, sizeof(aiot_dm_msg_t));
        msg.type = AIOT_DMMSG_PROPERTY_SET_REPLY;
        msg.data.property_set_reply.msg_id = recv->data.property_set.msg_id;
        msg.data.property_set_reply.code = 200;
        msg.data.property_set_reply.data = "{}";
        int32_t res = aiot_dm_send(dm_handle, &msg);
        if (res < 0) {
            printf("aiot_dm_send failed\r\n");
        }
    }
    */
}

static void demo_dm_recv_async_service_invoke(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_async_service_invoke msg_id = %ld, service_id = %s, params = %.*s\r\n",
           (unsigned long)recv->data.async_service_invoke.msg_id,
           recv->data.async_service_invoke.service_id,
           recv->data.async_service_invoke.params_len,
           recv->data.async_service_invoke.params);

    /* TODO: The following code shows how to respond to an asynchronous service call from the cloud platform. The user can uncomment it to see the demonstration
        *
        * Note: If the user answers outside the callback function, the msg_id needs to be saved by the user, because the callback function's entry parameter
        * will be destroyed by the SDK after exiting the callback function and is no longer accessible
        */

    /*
    {
        aiot_dm_msg_t msg;

        memset(&msg, 0, sizeof(aiot_dm_msg_t));
        msg.type = AIOT_DMMSG_ASYNC_SERVICE_REPLY;
        msg.data.async_service_reply.msg_id = recv->data.async_service_invoke.msg_id;
        msg.data.async_service_reply.code = 200;
        msg.data.async_service_reply.service_id = "ToggleLightSwitch";
        msg.data.async_service_reply.data = "{\"dataA\": 20}";
        int32_t res = aiot_dm_send(dm_handle, &msg);
        if (res < 0) {
            printf("aiot_dm_send failed\r\n");
        }
    }
    */
}

static void demo_dm_recv_sync_service_invoke(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_sync_service_invoke msg_id = %ld, rrpc_id = %s, service_id = %s, params = %.*s\r\n",
           (unsigned long)recv->data.sync_service_invoke.msg_id,
           recv->data.sync_service_invoke.rrpc_id,
           recv->data.sync_service_invoke.service_id,
           recv->data.sync_service_invoke.params_len,
           recv->data.sync_service_invoke.params);

    /* TODO: The following code shows how to respond to a synchronous service call from the cloud platform. The user can uncomment it to see the demo
        *
        * Note: If the user answers outside of the callback function, you need to save the msg_id and rrpc_id strings yourself, because the callback
        * function input parameters will be destroyed by the SDK after exiting the callback function and cannot be accessed again
        */

    /*
    {
        aiot_dm_msg_t msg;

        memset(&msg, 0, sizeof(aiot_dm_msg_t));
        msg.type = AIOT_DMMSG_SYNC_SERVICE_REPLY;
        msg.data.sync_service_reply.rrpc_id = recv->data.sync_service_invoke.rrpc_id;
        msg.data.sync_service_reply.msg_id = recv->data.sync_service_invoke.msg_id;
        msg.data.sync_service_reply.code = 200;
        msg.data.sync_service_reply.service_id = "SetLightSwitchTimer";
        msg.data.sync_service_reply.data = "{}";
        int32_t res = aiot_dm_send(dm_handle, &msg);
        if (res < 0) {
            printf("aiot_dm_send failed\r\n");
        }
    }
    */
}

static void demo_dm_recv_raw_data(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_raw_data raw data len = %d\r\n", recv->data.raw_data.data_len);
    /* TODO: The following code demonstrates how to send binary format data, 
     * if you need to have the corresponding data passthrough script deployed in the cloud 
     */
    /*
    {
        aiot_dm_msg_t msg;
        uint8_t raw_data[] = {0x01, 0x02};

        memset(&msg, 0, sizeof(aiot_dm_msg_t));
        msg.type = AIOT_DMMSG_RAW_DATA;
        msg.data.raw_data.data = raw_data;
        msg.data.raw_data.data_len = sizeof(raw_data);
        aiot_dm_send(dm_handle, &msg);
    }
    */
}

static void demo_dm_recv_raw_sync_service_invoke(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_raw_sync_service_invoke raw sync service rrpc_id = %s, data_len = %d\r\n",
           recv->data.raw_service_invoke.rrpc_id,
           recv->data.raw_service_invoke.data_len);
}

static void demo_dm_recv_raw_data_reply(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_raw_data_reply receive reply for up_raw msg, data len = %d\r\n", recv->data.raw_data.data_len);
    /* TODO: The user processes downstream binary data in recv->data.raw_data.data */
}

/* User data receiving and processing callback function */
static void demo_dm_recv_handler(void *dm_handle, const aiot_dm_recv_t *recv, void *userdata)
{
    printf("demo_dm_recv_handler, type = %d\r\n", recv->type);

    switch (recv->type) {

        /* The IoT platform's reply to the device's property post, event post request */
        case AIOT_DMRECV_GENERIC_REPLY: {
            demo_dm_recv_generic_reply(dm_handle, recv, userdata);
        }
        break;

        /* The property set command sent from IoT Platform */
        case AIOT_DMRECV_PROPERTY_SET: {
            demo_dm_recv_property_set(dm_handle, recv, userdata);
        }
        break;

        /* Asynchronous service invocation */
        case AIOT_DMRECV_ASYNC_SERVICE_INVOKE: {
            demo_dm_recv_async_service_invoke(dm_handle, recv, userdata);
        }
        break;

        /* Synchronous service invocation */
        case AIOT_DMRECV_SYNC_SERVICE_INVOKE: {
            demo_dm_recv_sync_service_invoke(dm_handle, recv, userdata);
        }
        break;

        /* Downlink binary data */
        case AIOT_DMRECV_RAW_DATA: {
            demo_dm_recv_raw_data(dm_handle, recv, userdata);
        }
        break;

        /* Synchronous service calls in binary format */
        case AIOT_DMRECV_RAW_SYNC_SERVICE_INVOKE: {
            demo_dm_recv_raw_sync_service_invoke(dm_handle, recv, userdata);
        }
        break;

        /* The cloud responds to the raw uplink binary data */
        case AIOT_DMRECV_RAW_DATA_REPLY: {
            demo_dm_recv_raw_data_reply(dm_handle, recv, userdata);
        }
        break;

        default:
            break;
    }
}

/* Property post function demonstration */
int32_t demo_send_property_post(void *dm_handle, char *params)
{
    aiot_dm_msg_t msg;

    memset(&msg, 0, sizeof(aiot_dm_msg_t));
    msg.type = AIOT_DMMSG_PROPERTY_POST;
    msg.data.property_post.params = params;

    return aiot_dm_send(dm_handle, &msg);
}

int32_t demo_send_property_batch_post(void *dm_handle, char *params)
{
    aiot_dm_msg_t msg;

    memset(&msg, 0, sizeof(aiot_dm_msg_t));
    msg.type = AIOT_DMMSG_PROPERTY_BATCH_POST;
    msg.data.property_post.params = params;

    return aiot_dm_send(dm_handle, &msg);
}

/* Event post function demonstration */
int32_t demo_send_event_post(void *dm_handle, char *event_id, char *params)
{
    aiot_dm_msg_t msg;

    memset(&msg, 0, sizeof(aiot_dm_msg_t));
    msg.type = AIOT_DMMSG_EVENT_POST;
    msg.data.event_post.event_id = event_id;
    msg.data.event_post.params = params;

    return aiot_dm_send(dm_handle, &msg);
}

/* Demonstrates the acquisition of the expectation value of LightSwitch property. 
   The user can run the demonstration by adding this function to the main function 
*/
int32_t demo_send_get_desired_requset(void *dm_handle)
{
    aiot_dm_msg_t msg;

    memset(&msg, 0, sizeof(aiot_dm_msg_t));
    msg.type = AIOT_DMMSG_GET_DESIRED;
    msg.data.get_desired.params = "[\"LightSwitch\"]";

    return aiot_dm_send(dm_handle, &msg);
}

/* The removal of the expectation value of LightSwitch property is demonstrated, 
   and the user can add this function to the main function to run the demo 
*/
int32_t demo_send_delete_desired_requset(void *dm_handle)
{
    aiot_dm_msg_t msg;

    memset(&msg, 0, sizeof(aiot_dm_msg_t));
    msg.type = AIOT_DMMSG_DELETE_DESIRED;
    msg.data.get_desired.params = "{\"LightSwitch\":{}}";

    return aiot_dm_send(dm_handle, &msg);
}


int main(int argc, char *argv[])
{
    int32_t     res = STATE_SUCCESS;
    void       *dm_handle = NULL;
    void       *mqtt_handle = NULL;
    uint16_t    port = 443;      /* Regardless of whether the device uses TLS, the destination port is 443 */
    aiot_sysdep_network_cred_t cred; /* Security credential data structure to configure the CA certificate and other parameters, applicable to TLS connection scenario  */
    uint8_t post_reply = 1;

    /* Configure the underlying system dependencies of the SDK */
    aiot_sysdep_set_portfile(&g_aiot_sysdep_portfile);
    /* Configure the log output of the SDK */
    aiot_state_set_logcb(demo_state_logcb);

    /* Create the SDK's security credentials to establish a TLS connection */
    memset(&cred, 0, sizeof(aiot_sysdep_network_cred_t));
    cred.option = AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA;  /* Use the RSA to verify the MQTT server */
    cred.max_tls_fragment = 16384;  /* The maximum shard length is 16K, and other optional values are 4K, 2K, 1K, 0.5K */
    cred.sni_enabled = 1;                                 /* enable the feature of the Server Name Indicator of TLS */
    cred.x509_server_cert = ali_ca_cert;                 /* The RSA root certificate used to verify the MQTT server  */
    cred.x509_server_cert_len = strlen(ali_ca_cert);     /* length of RSA certificate */

   /* Create an MQTT client instance and initialize the default parameters internally */
    mqtt_handle = aiot_mqtt_init();
    if (mqtt_handle == NULL) {
        printf("aiot_mqtt_init failed\n");
        return -1;
    }

    /* Configure the MQTT server address */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_HOST, (void *)mqtt_host);
    /* Configure the MQTT server port */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PORT, (void *)&port);
    /* Configure device's productKey */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PRODUCT_KEY, (void *)product_key);
    /* Configure device's deviceName */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_DEVICE_NAME, (void *)device_name);
    /* Configure device's deviceSecret */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_DEVICE_SECRET, (void *)device_secret);
    /* Configure the security credentials for the network connection, which have been created above */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_NETWORK_CRED, (void *)&cred);
    /* Configure MQTT event callback functions */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_EVENT_HANDLER, (void *)demo_mqtt_event_handler);

    /* Create a DATA-MODEL instance */
    dm_handle = aiot_dm_init();
    if (dm_handle == NULL) {
        printf("aiot_dm_init failed");
        return -1;
    }
    /* Configure the MQTT instance handle */
    aiot_dm_setopt(dm_handle, AIOT_DMOPT_MQTT_HANDLE, mqtt_handle);
    /* Configure the message receive processing callback function */
    aiot_dm_setopt(dm_handle, AIOT_DMOPT_RECV_HANDLER, (void *)demo_dm_recv_handler);

    /* Whether the configuration is cloud or not requires a reply post_reply to the device. 
       If it is 1, it means that a cloud reply is required, otherwise it means that there is no reply 
    */
    aiot_dm_setopt(dm_handle, AIOT_DMOPT_POST_REPLY, (void *)&post_reply);

    /* Establish an MQTT connection to the server */
    res = aiot_mqtt_connect(mqtt_handle);
    if (res < STATE_SUCCESS) {
        /* Attempt to establish a connection fails, destroy the MQTT instance, reclaim resources */
        aiot_dm_deinit(&dm_handle);
        aiot_mqtt_deinit(&mqtt_handle);
        printf("aiot_mqtt_connect failed: -0x%04X\n\r\n", -res);
        printf("please check variables like mqtt_host, produt_key, device_name, device_secret in demo\r\n");
        return -1;
    }

    /* subsribe the topic of property/batch/post_reply */
    aiot_mqtt_sub(mqtt_handle, "/sys/${YourProductKey}/${YourDeviceName}/thing/event/property/batch/post_reply", NULL, 1,
                  NULL);

    /* Create a separate thread dedicated to executing aiot_mqtt_process, which automatically sends heartbeat keep alive, as well as resends unanswered packets for QoS1 */
    g_mqtt_process_thread_running = 1;
    res = pthread_create(&g_mqtt_process_thread, NULL, demo_mqtt_process_thread, mqtt_handle);
    if (res < 0) {
        printf("pthread_create demo_mqtt_process_thread failed: %d\n", res);
        aiot_dm_deinit(&dm_handle);
        aiot_mqtt_disconnect(mqtt_handle);
        aiot_mqtt_deinit(&mqtt_handle);
        return -1;
    }

    /* Create a separate thread to execute the aiot_mqtt_recv, which loops through the MQTT messages issued by the server and automatically reconnects when disconnected */
    g_mqtt_recv_thread_running = 1;
    res = pthread_create(&g_mqtt_recv_thread, NULL, demo_mqtt_recv_thread, mqtt_handle);
    if (res < 0) {
        printf("pthread_create demo_mqtt_recv_thread failed: %d\n", res);
        aiot_dm_deinit(&dm_handle);
        aiot_mqtt_disconnect(mqtt_handle);
        aiot_mqtt_deinit(&mqtt_handle);
        return -1;
    }

    /* The main loop goes into hibernation */
    while (1) {
        /* TODO: The following code demonstrates basic property and event post. Users can uncomment to see how this works */
        demo_send_property_post(dm_handle, "{\"LightSwitch\": 0}");
        /*
        demo_send_event_post(dm_handle, "Error", "{\"ErrorCode\": 0}");
        */

       /* TODO: the following code shows the batch report user data, users can uncomment to observe the effect of the demo
        * specific data format, please see https://help.aliyun.com/document_detail/89301.html "batch report properties, events" section
        * /
        */
        /*
        demo_send_property_batch_post(dm_handle,
                                      "{\"properties\":{\"Power\": [ {\"value\":\"on\",\"time\":1612684518}],\"WF\": [{\"value\": 3,\"time\":1612684518}]}}");
        */

        sleep(5);
    }

    /* Stop sending and receiving action */
    g_mqtt_process_thread_running = 0;
    g_mqtt_recv_thread_running = 0;

    /* Disconnect the MQTT connection. Normally, it would not arrive here */
    res = aiot_mqtt_disconnect(mqtt_handle);
    if (res < STATE_SUCCESS) {
        aiot_dm_deinit(&dm_handle);
        aiot_mqtt_deinit(&mqtt_handle);
        printf("aiot_mqtt_disconnect failed: -0x%04X\n", -res);
        return -1;
    }

    /* Destroying a DATA-MODEL instance.  Normally, it would not arrive here */
    res = aiot_dm_deinit(&dm_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_dm_deinit failed: -0x%04X\n", -res);
        return -1;
    }

    /* Destroy the MQTT instance */
    res = aiot_mqtt_deinit(&mqtt_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_mqtt_deinit failed: -0x%04X\n", -res);
        return -1;
    }

    pthread_join(g_mqtt_process_thread, NULL);
    pthread_join(g_mqtt_recv_thread, NULL);

    return 0;
}

