/*
 * This example is suitable for POSIX devices that support pthread such as 'Linux'. 
 * It demonstrates configuring MQTT parameters with the SDK and establishing a connection, after which 3 threads are created.
 *
 * + One thread is used to keep long connections alive.
 * + One thread is used to receive messages and enter the default data callback when a message arrives, and an event callback when the connection state changes.
 * + One thread is used to download the firmware to be upgraded from HTTP on the network, which is started by the receiving message thread after getting the MQTT message for the OTA upgrade
 *
 * Parts that require user attention or modification have been marked in the comments with 'TODO'
 *
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <stdlib.h>

#include "aiot_state_api.h"
#include "aiot_sysdep_api.h"
#include "aiot_ota_api.h"
#include "aiot_mqtt_api.h"

/* TODO: input your device's auth info */
char *product_key       = "${YourProductKey}";
char *device_name       = "${YourDeviceName}";
char *device_secret     = "${YourDeviceSecret}";

/*
    TODO: Replace the access point with your own instance's access point in the format
    "${YourProductKey}.iot-as-mqtt.${YourRegionId}.aliyuncs.com"
    where ${YourProductKey}: Please replace with the ProductKey of the product to which the device belongs. 
    You can login to the IoT Platform console and obtain it on the device details page of the corresponding instance.
    ${YourRegionId}: Please replace the domain code for your IoT Platform device, such as cn-shanghai, etc
    Example of the complete mqtt_host in this case: a1TTmBPIChA.iot-as-mqtt.cn-shanghai.aliyuncs.com

     * | ${YourProductKey}.iot-as-mqtt.cn-shanghai.aliyuncs.com      | Shanghai  | 443
     * | ${YourProductKey}.iot-as-mqtt.ap-southeast-1.aliyuncs.com   | Singapore | 443
     * | ${YourProductKey}.iot-as-mqtt.ap-northeast-1.aliyuncs.com   | Japan     | 443
     * | ${YourProductKey}.iot-as-mqtt.us-west-1.aliyuncs.com        | West-US   | 443
     * | ${YourProductKey}.iot-as-mqtt.eu-central-1.aliyuncs.com     | EU        | 443
     * 
*/
char  *url = "${YourProductKey}.iot-as-mqtt.cn-shanghai.aliyuncs.com";

/* Set of system adaptation functions, located in portfiles/aiot_port */
extern aiot_sysdep_portfile_t g_aiot_sysdep_portfile;

/* The server certificate in external/ali_ca_cert.c */
extern const char *ali_ca_cert;

static pthread_t g_mqtt_process_thread; /* Long-connection keep-alive thread for MQTT */
static pthread_t g_mqtt_recv_thread;    /* The message reception thread used for MQTT */
static pthread_t g_download_thread;     /* Firmware download thread for HTTP */

/* 
 * TODO: If you want to turn logging off, implement this function as null. If you want to reduce logging, choose not to print according to code
 * For example, [1577589489.033][LK-0317] mqTT_basic_demo &gb80sFmX7yX
 * The code for this log is 0317(hexadecimal). See core/aiot_state_api.h for the code value
 */

/* The log callback function from which the SDK logs are output */
int32_t demo_state_logcb(int32_t code, char *message)
{
    /* When downloading the firmware, there will be a large number of HTTP packet logs, which will be filtered out through the following code */
    if (STATE_HTTP_LOG_RECV_CONTENT != code) {
        printf("%s", message);
    }
    return 0;
}

/* Download packet callback, after the user calls aiot_download_recv(), the SDK receives the data and enters the function, and the downloaded data is handed over to the user */
/* TODO: Generally speaking, when the device is upgraded, the downloaded data will be written to flash in this callback */
void demo_download_recv_handler(void *handle, const aiot_download_recv_t *packet, void *userdata)
{
    uint32_t data_buffer_len = 0;
    uint32_t last_percent = 0;
    int32_t  percent = 0;

    /* Currently only packet->type is supported in cases where the type is AIOT_DLRECV_HTTPBODY */
    if (!packet || AIOT_DLRECV_HTTPBODY != packet->type) {
        return;
    }
    percent = packet->data.percent;

    /* userdata can hold data that needs to be shared between different entries of demo_download_recv_handler() */
    /* Here is used to store the percentage of firmware progress downloaded the last time you entered this callback function */
    if (userdata) {
        last_percent = *((uint32_t *)(userdata));
    }

    data_buffer_len = packet->data.len;

    /* If percent is negative, a packet exception or a digest check error has occurred */
    if (percent < 0) {
        printf("exception: percent = %d\r\n", percent);
        if (userdata) {
            free(userdata);
        }
        return;
    }

    /*
     * TODO: Download a piece of firmware successfully, at this time, the user should save the data in the memory
     * with the starting address of packet->data.buffer and the length of packet->data.len into flash
     *
     * If flash write fails, you should also call aiot_download_report_progress (handle, -4) to report the failure to the cloud platform
     * Note: In the protocol, the error code agreed with the cloud platform is in the aiot_ota_protocol_errcode_t type, for example
     * -1: Indicates that the upgrade failed
     * -2: Indicates that the download failed
     * -3: Indicates that the validation failed
     * -4: Indicates a flash failure
     *
     * Details can be found in https://help.aliyun.com/document_detail/85700.html
     */

    /* When the value of the percent input parameter is 100, the download task is completed */
    if (percent == 100) {
        /*
         * TODO: At this time, the average user should complete all the firmware burning, save the current work, 
                 restart the device, switch to the new firmware and boot.
                 The new firmware should use aiot_ota_report_version to report new version to IoT platform.

                 After the cloud platform receives the new version number report, it will determine that the upgrade is successful, 
                 otherwise it will be considered that the upgrade has failed
                 If the upgrade fails after the download is successful, you should also call aiot_download_report_progress (handle, -1) 
                 to report the failure to the cloud platform.
         */
    }

    /* Only when the progress of the last download increases by more than 5%, the progress is printed and the progress is reported to the serve */
    if (percent - last_percent >= 5 || percent == 100) {
        printf("download %03d%% done, +%d bytes\r\n", percent, data_buffer_len);
        aiot_download_report_progress(handle, percent);

        if (userdata) {
            *((uint32_t *)(userdata)) = percent;
        }
        if (percent == 100 && userdata) {
            free(userdata);
        }
    }
}

/* Thread implements requests and receipts for firmware content by periodically runns aiot_download_recv. */
void *demo_ota_download_thread(void *dl_handle)
{
    int32_t     ret = 0;

    printf("starting download thread in 2 seconds ......\r\n");
    sleep(2);

    /* Request download from firmware server */
    aiot_download_send_request(dl_handle);
    while (1) {
        /* Receives the firmware content of the server response from the network */
        ret = aiot_download_recv(dl_handle);

        /* When the firmware is fully downloaded, the return value of aiot_download_recv() 
           will be equal to STATE_DOWNLOAD_FINISHED, otherwise it is the number of bytes acquired 
        */
        if (STATE_DOWNLOAD_FINISHED == ret) {
            printf("download completed\r\n");
            break;
        }
        if (STATE_DOWNLOAD_RENEWAL_REQUEST_SENT == ret) {
            printf("download renewal request has been sent successfully\r\n");
            continue;
        }
        if (ret <= STATE_SUCCESS) {
            printf("download failed, error code is %d, try to send renewal request\r\n", ret);
            continue;
        }
    }

    /* Downloading all firmware content completes, the download session is destroyed, and the thread exits on its own */
    aiot_download_deinit(&dl_handle);
    printf("download thread exit\r\n");

    return NULL;
}

/* The OTA message processing callback registered by the user through aiot_ota_setopt(),
   if the SDK receives an OTA-related MQTT message, it will be automatically recognized and call this callback function */
void demo_ota_recv_handler(void *ota_handle, aiot_ota_recv_t *ota_msg, void *userdata)
{
    switch (ota_msg->type) {
        case AIOT_OTARECV_FOTA: {
            uint32_t res = 0;
            uint16_t port = 443;
            uint32_t max_buffer_len = (8 * 1024);
            aiot_sysdep_network_cred_t cred;
            void *dl_handle = NULL;
            void *last_percent = NULL;

            if (NULL == ota_msg->task_desc) {
                break;
            }

            dl_handle = aiot_download_init();
            if (NULL == dl_handle) {
                break;
            }
            printf("OTA target firmware version: %s, size: %u Bytes \r\n", ota_msg->task_desc->version,
                   ota_msg->task_desc->size_total);

            if (NULL != ota_msg->task_desc->extra_data) {
                printf("extra data: %s\r\n", ota_msg->task_desc->extra_data);
            }

            memset(&cred, 0, sizeof(aiot_sysdep_network_cred_t));
            cred.option = AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA;
            cred.max_tls_fragment = 16384;
            cred.x509_server_cert = ali_ca_cert;
            cred.x509_server_cert_len = strlen(ali_ca_cert);

            /* apply TLS for security */
            aiot_download_setopt(dl_handle, AIOT_DLOPT_NETWORK_CRED, (void *)(&cred));
            /* Sets the server port number to access when downloading */
            aiot_download_setopt(dl_handle, AIOT_DLOPT_NETWORK_PORT, (void *)(&port));
            /* Set the downloaded task information, including the download address, firmware size, 
               firmware signature, and etc, obtained from the task_desc member in the parameter ota_msg 
            */
            aiot_download_setopt(dl_handle, AIOT_DLOPT_TASK_DESC, (void *)(ota_msg->task_desc));
            /* Sets the callback function that the SDK will call when the download arrives */
            aiot_download_setopt(dl_handle, AIOT_DLOPT_RECV_HANDLER, (void *)(demo_download_recv_handler));
            /* Sets the maximum buffer length per download, and notifies the user whenever this length of memory is fully filled with data */
            aiot_download_setopt(dl_handle, AIOT_DLOPT_BODY_BUFFER_MAX_LEN, (void *)(&max_buffer_len));
            /* Set the data shared between the different calls of the AIOT_DLOPT_RECV_HANDLER, such as the download progress stored by the example */
            last_percent = malloc(sizeof(uint32_t));
            if (NULL == last_percent) {
                aiot_download_deinit(&dl_handle);
                break;
            }
            memset(last_percent, 0, sizeof(uint32_t));
            aiot_download_setopt(dl_handle, AIOT_DLOPT_USERDATA, (void *)last_percent);

            /* Start a dedicated download thread to complete the download of the firmware content */
            res = pthread_create(&g_download_thread, NULL, demo_ota_download_thread, dl_handle);
            if (res != 0) {
                printf("pthread_create demo_ota_download_thread failed: %d\r\n", res);
                aiot_download_deinit(&dl_handle);
                free(last_percent);
            } else {
                /* The download thread is set to detach type */
                pthread_detach(g_download_thread);
            }
            break;
        }

        default:
            break;
    }
}

/*  MQTT event callback function, which is triggered when network connected/reconnected/disconnected, the event definition can be see core/aiot_mqtt_api.h */
void demo_mqtt_event_handler(void *handle, const aiot_mqtt_event_t *event, void *userdata)
{
    switch (event->type) {
        /* SDK successfully established a connection to the MQTT server because the user called the aiot_mqtt_connect () function */
        case AIOT_MQTTEVT_CONNECT: {
            printf("AIOT_MQTTEVT_CONNECT\r\n");
            /* TODO: handle SDK connection success, can not call a long blocking function */
        }
        break;

        /* SDK automatically reconnects after disconnected */
        case AIOT_MQTTEVT_RECONNECT: {
            printf("AIOT_MQTTEVT_RECONNECT\r\n");
            /* TODO: If the SDK is successfully reconnected, do not call the blocking function that takes a long time here */
        }
        break;

        /* The SDK disconnects passively due to network conditions. The network indicates that the underlying read and write fails. 
         * The heartbeat indicates that the server does not receive a heartbeat response as expected 
         */
        case AIOT_MQTTEVT_DISCONNECT: {
            char *cause = (event->data.disconnect == AIOT_MQTTDISCONNEVT_NETWORK_DISCONNECT) ? ("network disconnect") :
                          ("heartbeat disconnect");
            printf("AIOT_MQTTEVT_DISCONNECT: %s\r\n", cause);
            /* TODO: To handle SDK passive disconnection, do not call blocking functions that take a long time here */
        }
        break;

        default: {

        }
    }
}

/* The MQTT default message processing callback is invoked when the SDK receives an MQTT message from the server and there is no corresponding user callback processing */
void demo_mqtt_default_recv_handler(void *handle, const aiot_mqtt_recv_t *packet, void *userdata)
{
    switch (packet->type) {
        case AIOT_MQTTRECV_HEARTBEAT_RESPONSE: {
            printf("heartbeat response\r\n");
            /* TODO: Process the heartbeat response of the server */
        }
        break;

        case AIOT_MQTTRECV_SUB_ACK: {
            printf("suback, res: -0x%04X, packet id: %d, max qos: %d\r\n",
                   -packet->data.sub_ack.res, packet->data.sub_ack.packet_id, packet->data.sub_ack.max_qos);
            /* TODO: Process the server's response to a subscription request, generally not */
        }
        break;

        case AIOT_MQTTRECV_PUB: {
            printf("pub, qos: %d, topic: %.*s\r\n", packet->data.pub.qos, packet->data.pub.topic_len, packet->data.pub.topic);
            printf("pub, payload: %.*s\r\n", packet->data.pub.payload_len, packet->data.pub.payload);
            /* TODO: Processes service packets sent by the server */
        }
        break;

        case AIOT_MQTTRECV_PUB_ACK: {
            printf("puback, packet id: %d\r\n", packet->data.pub_ack.packet_id);
             /* TODO: Processes the response of the server to the message reported by QoS1 */
        }
        break;

        default: {

        }
    }
}

/* The thread that performs aiot_mqtt_process, including heartbeat sending and QoS1 message resending */
void *demo_mqtt_process_thread(void *args)
{
    while (1) {
        aiot_mqtt_process(args);
        sleep(1);
    }
    return NULL;
}

/* The thread that performs aiot_mqtt_recv, handles network auto-reconnection and reception of MQTT messages from the server */
void *demo_mqtt_recv_thread(void *args)
{
    int32_t res = STATE_SUCCESS;

    while (1) {
        res = aiot_mqtt_recv(args);
        if (res < STATE_SUCCESS) {
            sleep(1);
        }
    }
    return NULL;
}

int main(int argc, char *argv[])
{
    int32_t     res = STATE_SUCCESS;
    void       *mqtt_handle = NULL;
    uint16_t    port = 443;       /* Regardless of whether the device uses TLS, the destination port is 443 */
    aiot_sysdep_network_cred_t cred; /* Security credential data structure to configure the CA certificate and other parameters, applicable to TLS connection scenario */
    void       *ota_handle = NULL;
    char       *cur_version = NULL;

    /* Configure the underlying system dependencies of the SDK */
    aiot_sysdep_set_portfile(&g_aiot_sysdep_portfile);
    /* Configure the log output of the SDK */
    aiot_state_set_logcb(demo_state_logcb);

  /* Create the SDK's security credentials to establish a TLS connection */
    memset(&cred, 0, sizeof(aiot_sysdep_network_cred_t));
    cred.option = AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA;  /* Use the RSA to verify the MQTT server */
    cred.max_tls_fragment = 16384; /* The maximum shard length is 16K, and other optional values are 4K, 2K, 1K, 0.5K */
    cred.sni_enabled = 1;                               /* enable the feature of the Server Name Indicator of TLS */
    cred.x509_server_cert = ali_ca_cert;                /* The RSA root certificate used to verify the MQTT server */
    cred.x509_server_cert_len = strlen(ali_ca_cert);    /* length of RSA certificate */

    /* Create an MQTT client instance and initialize the default parameters internally */
    mqtt_handle = aiot_mqtt_init();
    if (NULL == mqtt_handle) {
        printf("aiot_mqtt_init failed\r\n");
        return -1;
    }

    /* Configure the MQTT server address */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_HOST, (void *)url);
    /* Configure the MQTT server port */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PORT, (void *)&port);
    /* Configure device's productKey */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PRODUCT_KEY, (void *)product_key);
    /* Configure device's deviceName */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_DEVICE_NAME, (void *)device_name);
    /* Configure device's deviceSecret */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_DEVICE_SECRET, (void *)device_secret);
    /* Configure the security credentials for the network connection, which have been created above */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_NETWORK_CRED, (void *)&cred);
    /* Configure the MQTT default message receive callback function */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_RECV_HANDLER, (void *)demo_mqtt_default_recv_handler);
    /* Configure the MQTT event callback function */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_EVENT_HANDLER, (void *)demo_mqtt_event_handler);

    /* An OTA session instance needs to be created */
    ota_handle = aiot_ota_init();
    if (NULL == ota_handle) {
        printf("aiot_ota_init failed\r\n");
        aiot_mqtt_deinit(&mqtt_handle);
        return -2;
    }

    /* Use the following statement to associate an OTA instance with an MQTT instance */
    aiot_ota_setopt(ota_handle, AIOT_OTAOPT_MQTT_HANDLE, mqtt_handle);
    /* Use the following statement to set the OTA message receive callback, which the SDK will invoke when it receives an OTA message */
    aiot_ota_setopt(ota_handle, AIOT_OTAOPT_RECV_HANDLER, demo_ota_recv_handler);

     /* Establish an MQTT connection to the server */
    res = aiot_mqtt_connect(mqtt_handle);
    if (res < STATE_SUCCESS) {
        /* Attempt to establish a connection fails, destroy the MQTT instance, reclaim resources */
        aiot_mqtt_deinit(&mqtt_handle);
        aiot_ota_deinit(&ota_handle);
        printf("aiot_mqtt_connect failed: -0x%04X\r\n", -res);
        return -3;
    }

    /*   TODO: Very important!!
     *
     *   cur_version should be the actual version number of current firmware (no matter upgraded or not), not a fixed value as in the example
     *
     *   The upgrade progress would be failed on the console page if the device reports a version number different from the one described in OTA task.
     *
     */

    /* After the MQTT connection is established, the version number of the current device can be reported */
    cur_version = "1.0.0";
    res = aiot_ota_report_version(ota_handle, cur_version);
    if (res < STATE_SUCCESS) {
        printf("aiot_ota_report_version failed: -0x%04X\r\n", -res);
    }

    /* Create a separate thread dedicated to executing aiot_mqtt_process, which automatically sends heartbeat keep alive, as well as resends unanswered packets for QoS1 */
    res = pthread_create(&g_mqtt_process_thread, NULL, demo_mqtt_process_thread, mqtt_handle);
    if (res != 0) {
        printf("pthread_create demo_mqtt_process_thread failed: %d\r\n", res);
        return -1;
    }

    /* Create a separate thread to execute the aiot_mqtt_recv, which loops through the MQTT messages issued by the server and automatically reconnects when disconnected */
    res = pthread_create(&g_mqtt_recv_thread, NULL, demo_mqtt_recv_thread, mqtt_handle);
    if (res != 0) {
        printf("pthread_create demo_mqtt_recv_thread failed: %d\r\n", res);
        return -1;
    }

    /* The main loop goes into hibernation */
    while (1) {
        sleep(1);
    }

     /* Disconnect the MQTT connection. Generally the code would not arrive here */
    res = aiot_mqtt_disconnect(mqtt_handle);
    if (res < STATE_SUCCESS) {
        aiot_mqtt_deinit(&mqtt_handle);
        aiot_ota_deinit(&ota_handle);
        printf("aiot_mqtt_disconnect failed: -0x%04X\r\n", -res);
        return -1;
    }

    /* Destroy the MQTT instance, and it will not normally run here */
    res = aiot_mqtt_deinit(&mqtt_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_mqtt_deinit failed: -0x%04X\r\n", -res);
        aiot_ota_deinit(&ota_handle);
        return -1;
    }

    /* Destroy the OTA instance, and it will not normally run here */
    aiot_ota_deinit(&ota_handle);
    return 0;
}

