/*
 * This example is suitable for POSIX devices that support pthread such as 'Linux', which demonstrates configuring MQTT parameters with the SDK and establishing a connection, after which 2 threads are created
 *
 * + One thread is used to keep long connections alive
 * + The other thread is used to receive messages and enter the default data callback when a message arrives, and an event callback when the connection state changes
 *
 * Parts that require user attention or modification have been indicated in the comments with TODO
 *
 */


#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>

#include "aiot_state_api.h"
#include "aiot_sysdep_api.h"
#include "aiot_mqtt_api.h"

/* TODO: input your device's auth info */
char *product_key       = "${YourProductKey}";
char *device_name       = "${YourDeviceName}";
char *device_secret     = "${YourDeviceSecret}";

/*
    TODO: Replace the access point with your own instance's access point in the format
    "${YourProductKey}.iot-as-mqtt.${YourRegionId}.aliyuncs.com"
    where ${YourProductKey}: Please replace with the ProductKey of the product to which the device belongs. 
    You can login to the IoT Platform console and obtain it on the device details page of the corresponding instance.
    ${YourRegionId}: Please replace the domain code for your IoT Platform device, such as cn-shanghai, etc
    Example of the complete mqtt_host in this case: a1TTmBPIChA.iot-as-mqtt.cn-shanghai.aliyuncs.com

     * | ${YourProductKey}.iot-as-mqtt.cn-shanghai.aliyuncs.com      | Shanghai  | 443
     * | ${YourProductKey}.iot-as-mqtt.ap-southeast-1.aliyuncs.com   | Singapore | 443
     * | ${YourProductKey}.iot-as-mqtt.ap-northeast-1.aliyuncs.com   | Japan     | 443
     * | ${YourProductKey}.iot-as-mqtt.us-west-1.aliyuncs.com        | West-US   | 443
     * | ${YourProductKey}.iot-as-mqtt.eu-central-1.aliyuncs.com     | EU        | 443
     *
*/
char  *mqtt_host = "${YourProductKey}.iot-as-mqtt.cn-shanghai.aliyuncs.com";

/* Set of system adaptation functions, located in portfiles/aiot_port */
extern aiot_sysdep_portfile_t g_aiot_sysdep_portfile;

/* The server certificate in external/ali_ca_cert.c */
extern const char *ali_ca_cert;

static pthread_t g_mqtt_process_thread;
static pthread_t g_mqtt_recv_thread;
static uint8_t g_mqtt_process_thread_running = 0;
static uint8_t g_mqtt_recv_thread_running = 0;

/* 
 * TODO: If you want to turn logging off, implement this function as null. If you want to reduce logging, choose not to print according to code
 * For example, [1577589489.033][LK-0317] mqTT_basic_demo &gb80sFmX7yX
 * The code for this log is 0317(hexadecimal). See core/aiot_state_api.h for the code value
 */

/* The log callback function from which the SDK logs are output */
int32_t demo_state_logcb(int32_t code, char *message)
{
    printf("%s", message);
    return 0;
}

/* MQTT event callback function that is triggered when the network is connected/reconnected/disconnected. The event is defined in core/aiot_mqtt_api.h */
void demo_mqtt_event_handler(void *handle, const aiot_mqtt_event_t *event, void *userdata)
{
    switch (event->type) {
        /* SDK successfully established a connection to the MQTT server because the user called the aiot_mqtt_connect () function */
        case AIOT_MQTTEVT_CONNECT: {
            printf("AIOT_MQTTEVT_CONNECT\n");
            /* TODO: handle SDK connection success, can not call a long blocking function */
        }
        break;

        /* The SDK was disconnected due to the network condition, and the automatic reconnection was successfully initiated */
        case AIOT_MQTTEVT_RECONNECT: {
            printf("AIOT_MQTTEVT_RECONNECT\n");
            /* TODO: If the SDK is successfully reconnected, do not call the blocking function that takes a long time here */
        }
        break;

        /* The SDK disconnects passively due to network conditions. The network indicates that the underlying read and write fails. 
         *  The heartbeat indicates that the server does not receive a heartbeat response as expected 
         */
        case AIOT_MQTTEVT_DISCONNECT: {
            char *cause = (event->data.disconnect == AIOT_MQTTDISCONNEVT_NETWORK_DISCONNECT) ? ("network disconnect") :
                          ("heartbeat disconnect");
            printf("AIOT_MQTTEVT_DISCONNECT: %s\n", cause);
            /* TODO: To handle SDK passive disconnection, do not call blocking functions that take a long time here */
        }
        break;

        default: {

        }
    }
}

/* The MQTT default message processing callback is invoked when the SDK receives an MQTT message from the server and there is no corresponding user callback processing */
void demo_mqtt_default_recv_handler(void *handle, const aiot_mqtt_recv_t *packet, void *userdata)
{
    switch (packet->type) {
        case AIOT_MQTTRECV_HEARTBEAT_RESPONSE: {
            printf("heartbeat response\n");
            /* TODO: Process the heartbeat response of the server */
        }
        break;

        case AIOT_MQTTRECV_SUB_ACK: {
            printf("suback, res: -0x%04X, packet id: %d, max qos: %d\n",
                   -packet->data.sub_ack.res, packet->data.sub_ack.packet_id, packet->data.sub_ack.max_qos);
            /* TODO: Process the server's response to a subscription request, generally not */
        }
        break;

        case AIOT_MQTTRECV_PUB: {
            printf("pub, qos: %d, topic: %.*s\n", packet->data.pub.qos, packet->data.pub.topic_len, packet->data.pub.topic);
            printf("pub, payload: %.*s\n", packet->data.pub.payload_len, packet->data.pub.payload);
            /* TODO: Processes service packets sent by the server */
        }
        break;

        case AIOT_MQTTRECV_PUB_ACK: {
            printf("puback, packet id: %d\n", packet->data.pub_ack.packet_id);
            /* TODO: Processes the response of the server to the message reported by QoS1 */
        }
        break;

        default: {

        }
    }
}

/* The thread that performs aiot_mqtt_process, including heartbeat sending and QoS1 message resending */
void *demo_mqtt_process_thread(void *args)
{
    int32_t res = STATE_SUCCESS;

    while (g_mqtt_process_thread_running) {
        res = aiot_mqtt_process(args);
        if (res == STATE_USER_INPUT_EXEC_DISABLED) {
            break;
        }
        sleep(1);
    }
    return NULL;
}

/* The thread that performs aiot_mqtt_recv, handles network auto-reconnection and reception of MQTT messages from the server */
void *demo_mqtt_recv_thread(void *args)
{
    int32_t res = STATE_SUCCESS;

    while (g_mqtt_recv_thread_running) {
        res = aiot_mqtt_recv(args);
        if (res < STATE_SUCCESS) {
            if (res == STATE_USER_INPUT_EXEC_DISABLED) {
                break;
            }
            sleep(1);
        }
    }
    return NULL;
}

int main(int argc, char *argv[])
{
    int32_t     res = STATE_SUCCESS;
    void       *mqtt_handle = NULL;
    uint16_t    port = 443;      /* Regardless of whether the device uses TLS, the destination port is 443 */
    aiot_sysdep_network_cred_t cred; /* Security credential data structure to configure the CA certificate and other parameters, applicable to TLS connection scenario */

    /* Configure the underlying system dependencies of the SDK */
    aiot_sysdep_set_portfile(&g_aiot_sysdep_portfile);
    /* Configure the log output of the SDK */
    aiot_state_set_logcb(demo_state_logcb);

    /* Create the SDK's security credentials to establish a TLS connection */
    memset(&cred, 0, sizeof(aiot_sysdep_network_cred_t));
    cred.option = AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA;  /* Use the RSA to verify the MQTT server */
    cred.max_tls_fragment = 16384; /* The maximum shard length is 16K, and other optional values are 4K, 2K, 1K, 0.5K */
    cred.sni_enabled = 1;                               /* enable the feature of the Server Name Indicator of TLS */
    cred.x509_server_cert = ali_ca_cert;                /* The RSA root certificate used to verify the MQTT server */
    cred.x509_server_cert_len = strlen(ali_ca_cert);    /* length of RSA certificate */

    /* Create an MQTT client instance and initialize the default parameters internally */
    mqtt_handle = aiot_mqtt_init();
    if (mqtt_handle == NULL) {
        printf("aiot_mqtt_init failed\n");
        return -1;
    }

    /* TODO: If the following code is not commented, the routine connects to the cloud platform using TCP instead of TLS */
    /*
    {
        memset(&cred, 0, sizeof(aiot_sysdep_network_cred_t));
        cred.option = AIOT_SYSDEP_NETWORK_CRED_NONE;
    }
    */

    /* Configure the MQTT server address */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_HOST, (void *)mqtt_host);
    /* Configure the MQTT server port */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PORT, (void *)&port);
    /* Configure device's productKey */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_PRODUCT_KEY, (void *)product_key);
    /* Configure device's deviceName */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_DEVICE_NAME, (void *)device_name);
    /* Configure device's deviceSecret */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_DEVICE_SECRET, (void *)device_secret);
    /* Configure the security credentials for the network connection, which have been created above */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_NETWORK_CRED, (void *)&cred);
    /* Configure the MQTT default message receive callback function */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_RECV_HANDLER, (void *)demo_mqtt_default_recv_handler);
    /* Configure the MQTT event callback function */
    aiot_mqtt_setopt(mqtt_handle, AIOT_MQTTOPT_EVENT_HANDLER, (void *)demo_mqtt_event_handler);

    /* Establish an MQTT connection to the server */
    res = aiot_mqtt_connect(mqtt_handle);
    if (res < STATE_SUCCESS) {
        /* Attempt to establish a connection fails, destroy the MQTT instance, reclaim resources */
        aiot_mqtt_deinit(&mqtt_handle);
        printf("aiot_mqtt_connect failed: -0x%04X\n\r\n", -res);
        printf("please check variables like mqtt_host, produt_key, device_name, device_secret in demo\r\n");
        return -1;
    }

    /* MQTT subscription example. Please adapt the topic according to your cases */
    /* {
        char *sub_topic = "/sys/${YourProductKey}/${YourDeviceName}/thing/event/+/post_reply";

        res = aiot_mqtt_sub(mqtt_handle, sub_topic, NULL, 1, NULL);
        if (res < 0) {
            printf("aiot_mqtt_sub failed, res: -0x%04X\n", -res);
            return -1;
        }
    } */

    /* MQTT publish example. Please adapt the topic and payload according to your cases */
    /* {
        char *pub_topic = "/sys/${YourProductKey}/${YourDeviceName}/thing/event/property/post";
        char *pub_payload = "{\"id\":\"1\",\"version\":\"1.0\",\"params\":{\"LightSwitch\":0}}";

        res = aiot_mqtt_pub(mqtt_handle, pub_topic, (uint8_t *)pub_payload, (uint32_t)strlen(pub_payload), 0);
        if (res < 0) {
            printf("aiot_mqtt_sub failed, res: -0x%04X\n", -res);
            return -1;
        }
    } */

    /* Create a separate thread dedicated to executing aiot_mqtt_process, which automatically sends heartbeat keep alive, as well as resends unanswered packets for QoS1 */
    g_mqtt_process_thread_running = 1;
    res = pthread_create(&g_mqtt_process_thread, NULL, demo_mqtt_process_thread, mqtt_handle);
    if (res < 0) {
        printf("pthread_create demo_mqtt_process_thread failed: %d\n", res);
        return -1;
    }

    /* Create a separate thread to execute the aiot_mqtt_recv, which loops through the MQTT messages issued by the server and automatically reconnects when disconnected */
    g_mqtt_recv_thread_running = 1;
    res = pthread_create(&g_mqtt_recv_thread, NULL, demo_mqtt_recv_thread, mqtt_handle);
    if (res < 0) {
        printf("pthread_create demo_mqtt_recv_thread failed: %d\n", res);
        return -1;
    }

    /* The main loop goes into hibernation */
    while (1) {
        sleep(1);
    }

    /* Disconnect the MQTT connection. Generally the code would not arrive here */
    g_mqtt_process_thread_running = 0;
    g_mqtt_recv_thread_running = 0;
    sleep(1);
    pthread_join(g_mqtt_process_thread, NULL);
    pthread_join(g_mqtt_recv_thread, NULL);

    res = aiot_mqtt_disconnect(mqtt_handle);
    if (res < STATE_SUCCESS) {
        aiot_mqtt_deinit(&mqtt_handle);
        printf("aiot_mqtt_disconnect failed: -0x%04X\n", -res);
        return -1;
    }

    /* Destroy the MQTT instance, and it will not normally run here */
    res = aiot_mqtt_deinit(&mqtt_handle);
    if (res < STATE_SUCCESS) {
        printf("aiot_mqtt_deinit failed: -0x%04X\n", -res);
        return -1;
    }

    return 0;
}

